﻿/*
* Copyright 2010-2013 Team Robot, Inc.
* 
* Author: Peter Coley
*/

//
// TeamRobot.Swtor.Calc
// ----------------------------------------
// Stubs of all required dependencies are included in this javascript file.
//

var TeamRobot = Tr = (function ($) {
    var tr = {};
    return tr;
}(jQuery));

TeamRobot.Swtor = (function ($) {
    var swtor = {};

    swtor.isMod = function (item) {

        switch (item.Slot) {
            case 'Armoring':
            case 'Augment':
            case 'Barrel':
            case 'ColorCrystal':
            case 'Enhancement':
            case 'Hilt':
            case 'Mod':
                return true;
            default:
                return false;
        }
    };

    swtor.getClassFromAdvancedClass = function (advCls) {

        switch (advCls) {
            case 'None':
                return 'None';
            case 'Guardian':
            case 'Sentinel':
                return 'JediKnight';
            case 'Commando':
            case 'Vanguard':
                return 'Trooper';
            case 'Sage':
            case 'Shadow':
                return 'JediConsular';
            case 'Gunslinger':
            case 'Scoundrel':
                return 'Smuggler';
            case 'Juggernaut':
            case 'Marauder':
                return 'SithWarrior';
            case 'Mercenary':
            case 'Powertech':
                return 'BountyHunter';
            case 'Assassin':
            case 'Sorcerer':
                return 'SithInquisitor';
            case 'Operative':
            case 'Sniper':
                return 'ImperialAgent';
            default:
                return null;
        }
    };

    swtor.getArmorType = function (advCls, comp) {

        var cls = advCls;
        if (comp != null && comp != 'None')
            cls = comp;

        switch (cls) {
            case 'Guardian':
            case 'Vanguard':
            case 'Commando':
            case 'Juggernaut':
            case 'Powertech':
            case 'Mercenary':
                return 'Heavy';
            case 'Sentinel':
            case 'Scoundrel':
            case 'Gunslinger':
            case 'Marauder':
            case 'Operative':
            case 'Sniper':
                return 'Medium';
            case 'Shadow':
            case 'Sage':
            case 'Assassin':
            case 'Sorcerer':
                return 'Light';

            case 'T701':
            case 'M14x':
            case 'Scorpio':
                return 'Droid';
            case 'Kira':
            case 'Nadia':
            case 'Jaesa':
            case 'Xalek':
            case 'Vector':
                return 'Light';
            case 'Doc':
            case 'Tharan':
            case 'Zenith':
            case 'Risha':
            case 'Guss':
            case 'Vette':
            case 'Malavai':
            case 'Andronikos':
            case 'Ashara':
            case 'Talos':
            case 'Mako':
            case 'Gault':
            case 'Eckard':
            case 'Raina':
                return 'Medium';
            case 'Fideltin':
            case 'Scourge':
            case 'QyzenFess':
            case 'Felix':
            case 'Aric':
            case 'Tanno':
            case 'Elara':
            case 'Yuun':
            case 'Corso':
            case 'Bowdaar':
            case 'Akaavi':
            case 'Pierce':
            case 'Broonmark':
            case 'KhemVal':
            case 'Torian':
            case 'Blizz':
            case 'Skadge':
            case 'Kaliyo':
                return 'Heavy';

            default:
                return null;
        }
    };

    swtor.getItemSetId = function (item) {

        if (item == null)
            return null;
        else if (item.ItemSet != null)
            return item.ItemSet.Id;
        else if (item.ItemSetId != null)
            return item.ItemSetId;

        if (item.Mods != null) {
            for (var i = 0; i < item.Mods.length; i++) {
                if (item.Mods[i].ItemSet != null)
                    return item.Mods[i].ItemSet.Id;
                else if (item.Mods[i].ItemSetId != null)
                    return item.Mods[i].ItemSetId;
            }
        }

        return null;
    };

    return swtor;
}(jQuery));


TeamRobot.Swtor.Calc = (function (Tr, $) {
    var calc = Tr.Swtor.Calc = Tr.Swtor.Calc || {};

    var baseStats = ['Strength', 'Aim', 'Endurance', 'Cunning', 'Will', 'Presence'];
    var secondaryStats = ['Expertise', 'Power', 'Accuracy', 'Alacrity', 'Crit', 'Surge', 'Defense', 'Shielding', 'Absorb'];

    // base health for 1-55 is now correct.
    var healthLookup = { '1': 130, '2': 140, '3': 150, '4': 160, '5': 170, '6': 185, '7': 200, '8': 220, '9': 240, '10': 260, '11': 285, '12': 310, '13': 335, '14': 365, '15': 395, '16': 430, '17': 465, '18': 500, '19': 540, '20': 580, '21': 620, '22': 665, '23': 710, '24': 755, '25': 805, '26': 855, '27': 905, '28': 960, '29': 1015, '30': 1070, '31': 1125, '32': 1185, '33': 1245, '34': 1305, '35': 1370, '36': 1435, '37': 1500, '38': 1570, '39': 1640, '40': 1710, '41': 1780, '42': 1855, '43': 1930, '44': 2005, '45': 2085, '46': 2165, '47': 2245, '48': 2330, '49': 2415, '50': 2500, '51': 2750, '52': 3025, '53': 3325, '54': 3650, '55': 4000 };

    var companionToAdvancedClass = {
        'T701': 'Vanguard',
        'Kira': 'Shadow',
        'Doc': 'Scoundrel',
        'Fideltin': 'Commando',
        'Scourge': 'Guardian',
        'Aric': 'Commando',
        'Tanno': 'MeleeAim',
        'M14x': 'Vanguard',
        'Elara': 'Commando',
        'Yuun': 'MeleeAim',
        'QyzenFess': 'MeleeAim',
        'Tharan': 'Scoundrel',
        'Zenith': 'Sniper',
        'Felix': 'Vanguard',
        'Nadia': 'Shadow',
        'Corso': 'Vanguard',
        'Bowdaar': 'Guardian',
        'Risha': 'Sniper',
        'Akaavi': 'MeleeAim',
        'Guss': 'Scoundrel',
        'Vette': 'Gunslinger',
        'Jaesa': 'Assassin',
        'Malavai': 'Operative',
        'Pierce': 'Vanguard',
        'Broonmark': 'Juggernaut',
        'Mako': 'Operative',
        'Gault': 'Sniper',
        'Torian': 'MeleeAim',
        'Blizz': 'Powertech',
        'Skadge': 'Juggernaut',
        'KhemVal': 'Juggernaut',
        'Andronikos': 'Gunslinger',
        'Ashara': 'Marauder',
        'Talos': 'Operative',
        'Xalek': 'Assassin',
        'Kaliyo': 'Vanguard',
        'Vector': 'Assassin',
        'Eckard': 'Operative',
        'Raina': 'Gunslinger',
        'Scorpio': 'MeleeAim'
    };

    // MeleeAim is a special case for certain companions who are aim-based melee fighters

    var primaryByCls = {
        'Smuggler': 'Cunning',
        'ImperialAgent': 'Cunning',
        'Trooper': 'Aim',
        'BountyHunter': 'Aim',
        'JediKnight': 'Strength',
        'SithWarrior': 'Strength',
        'JediConsular': 'Will',
        'SithInquisitor': 'Will',
        'MeleeAim': 'Aim'
    };

    var secondaryByCls = {
        'Smuggler': 'Aim',
        'ImperialAgent': 'Aim',
        'Trooper': 'Cunning',
        'BountyHunter': 'Cunning',
        'JediKnight': 'Will',
        'SithWarrior': 'Will',
        'JediConsular': 'Strength',
        'SithInquisitor': 'Strength',
        'MeleeAim': 'Cunning'
    };

    // b is the max contribution from rating/stat and is mostly stable
    // for all stats, all levels.  as of 2.0 however, b of crit chance
    // from main stat has a lower cap for levels 51-55.  the matrix expresses
    // change in cap and allows for further growth/variability.
    // pvp calculation has been removed as it no longer has a DR.
    // note: first entry is for lvl50 and below

    var dr_b = {
        'Absorb': [0.5, 0.5, 0.5, 0.5, 0.5, 0.5],
        'Accuracy': [0.3, 0.3, 0.3, 0.3, 0.3, 0.3],
        'Defense': [0.3, 0.3, 0.3, 0.3, 0.3, 0.3],
        'Alacrity': [0.3, 0.3, 0.3, 0.3, 0.3, 0.3],
        'Crit': [0.3, 0.3, 0.3, 0.3, 0.3, 0.3],
        'CritPrimary': [0.3, 0.2, 0.2, 0.2, 0.2, 0.2],
        'Shielding': [0.5, 0.5, 0.5, 0.5, 0.5, 0.5],
        'Surge': [0.3, 0.3, 0.3, 0.3, 0.3, 0.3],
    };

    // N is the divisor of the formula and is stable for levels 1-50
    // as of 2.0, N now varies for every level 51-55

    // tank stats (absorb, defense, shield) are not complete for 51-54
    // pvp calculation has been removed as it no longer has a DR.

    var dr_N = {
        'Absorb': [9, 13.5, 18, 23, 28, 32.5],
        'Accuracy': [27.5, 34, 40.5, 47, 53.5, 60],
        'Defense': [27.5, 34, 40.5, 47, 53.5, 60],
        'Alacrity': [27.5, 34.5, 41.5, 48.5, 55.5, 62.5],
        'Crit': [22.5, 27, 31.5, 36, 40.5, 45],
        'CritPrimary': [125, 155, 185, 215, 245, 275],
        'Shielding': [16, 20.5, 25, 30, 34.5, 39],
        'Surge': [5.5, 6.5, 7.5, 9, 10, 11],
    };


    function getDiminishedStat(stat, lvl, rating, N) {

        var idx = (lvl < 50 ? 0 : lvl - 50)
        var b = dr_b[stat][idx];
        if (N === undefined)
            N = dr_N[stat][idx];
        var r = rating / Math.max(lvl, 20);
        return b * (1 - Math.pow(1 - (0.01 / b), r / (N / 50)));
    }

    // updated base primary stat/end/pres calculation for lvls 51-55
    function getBaseStats(cls, lvl, comp) {

        var ret = {};

        ret[primaryByCls[cls]] = 50 + Math.min(lvl, 50) * 4 + Math.max((lvl - 50), 0) * 30;
        ret[secondaryByCls[cls]] = 20 + lvl * 1.6;
        ret['Endurance'] = 45 + Math.min(lvl, 50) * 3.6 + Math.max((lvl - 50), 0) * 27;

        ret['Presence'] = 45 + Math.min(lvl, 50) * 3.6 + Math.max((lvl - 50), 0) * 27;

        ret['Health'] = healthLookup[lvl];
        for (var i = 0; i < baseStats.length; i++) {
            if (ret[baseStats[i]] == null)
                ret[baseStats[i]] = 10 + lvl * 0.8;
        }

        return ret;
    }


    function addRangedStats(stats, lvl, cls, mhDmg, ohDmg, dw, isComp) {

        var isCun = cls == 'ImperialAgent' || cls == 'Smuggler';

        if (mhDmg != null)
            stats['RangedBaseDamage'] = mhDmg.min.toFixed(1) + ' - ' + mhDmg.max.toFixed(1);

        var totalBonus = 0;
        stats['RangedAimBonusDamage'] = stats['Aim'] * 0.2;
        totalBonus += stats['RangedAimBonusDamage'];

        if (isCun) {
            stats['RangedCunningBonusDamage'] = stats['Cunning'] * 0.2;
            totalBonus += stats['RangedCunningBonusDamage'];
        }

        stats['RangedPowerBonusDamage'] = stats['Power'] * 0.23;
        totalBonus += stats['RangedPowerBonusDamage'];

        if (isComp)
            totalBonus += stats['PresenceBonusDamage'];

        if (stats['BuffRangedBonusDamage'] != null)
            totalBonus *= 1 + stats['BuffRangedBonusDamage'];

        stats['RangedBonusDamage'] = totalBonus;

        if (mhDmg != null)
            stats['RangedDamage'] = (mhDmg.min + totalBonus).toFixed(1) + ' - ' + (mhDmg.max + totalBonus).toFixed(1);

        incrementStat(stats, 'BuffRangedAccuracy', 0);
        stats['TotalRangedAccuracyBonus'] = stats['BuffRangedAccuracy'] + stats['AccuracyBonus'];
        stats['RangedAccuracyBasic'] = 0.9 + stats['TotalRangedAccuracyBonus'];
        stats['RangedAccuracySpecial'] = 1 + stats['TotalRangedAccuracyBonus'];
        if (dw) {
            stats['RangedAccuracyBasic2'] = 0.57 + stats['TotalRangedAccuracyBonus'];
            stats['RangedAccuracySpecial2'] = 0.67 + stats['TotalRangedAccuracyBonus'];
        }

        totalBonus = 0.05 + stats['CritAimBonus'] + stats['CritBonus'];
        if (isCun) {
            stats['RangedCritCunningBonus'] = stats['CritCunningBonus'];
            totalBonus += stats['CritCunningBonus'];
        }
        if (stats['BuffRangedCritChance'] != null)
            totalBonus += stats['BuffRangedCritChance'];
        stats['RangedCritChance'] = totalBonus;

        incrementStat(stats, 'BuffSurge', 0);

        stats['RangedCritMult'] = 0.5 + stats['SurgeBonus'] + stats['BuffSurge'];

        if (dw && ohDmg != null) {
            incrementStat(stats, 'RangedDamage2Mult', 0.3);
            stats['RangedBaseDamage2'] = ohDmg.min.toFixed(1) + ' - ' + ohDmg.max.toFixed(1);
            stats['RangedDamage2'] = (ohDmg.min * stats['RangedDamage2Mult']).toFixed(1) + ' - ' + (ohDmg.max * stats['RangedDamage2Mult']).toFixed(1);
        }
    }

    function addMeleeStats(stats, lvl, cls, acls, mhDmg, ohDmg, dw, isComp) {

        var isAim = cls == 'MeleeAim';
        var isWill = acls == 'Shadow' || acls == 'Assassin';

        if (mhDmg != null)
            stats['MeleeBaseDamage'] = mhDmg.min.toFixed(1) + ' - ' + mhDmg.max.toFixed(1);

        var totalBonus = 0;
        stats['MeleeStrengthBonusDamage'] = stats['Strength'] * 0.2;
        totalBonus += stats['MeleeStrengthBonusDamage'];

        if (isAim) {
            stats['MeleeAimBonusDamage'] = stats['Aim'] * 0.2;
            totalBonus += stats['MeleeAimBonusDamage'];
        }

        if (isWill) {
            stats['MeleeWillBonusDamage'] = stats['Will'] * 0.2;
            totalBonus += stats['MeleeWillBonusDamage'];
        }

        stats['MeleePowerBonusDamage'] = stats['Power'] * 0.23;
        totalBonus += stats['MeleePowerBonusDamage'];

        if (isComp)
            totalBonus += stats['PresenceBonusDamage'];

        if (stats['BuffMeleeBonusDamage'] != null)
            totalBonus *= 1 + stats['BuffMeleeBonusDamage'];

        stats['MeleeBonusDamage'] = totalBonus;

        if (mhDmg != null)
            stats['MeleeDamage'] = (mhDmg.min + totalBonus).toFixed(1) + ' - ' + (mhDmg.max + totalBonus).toFixed(1);

        incrementStat(stats, 'BuffMeleeAccuracy', 0);
        stats['TotalMeleeAccuracyBonus'] = stats['BuffMeleeAccuracy'] + stats['AccuracyBonus'];
        stats['MeleeAccuracyBasic'] = 0.9 + stats['TotalMeleeAccuracyBonus'];
        stats['MeleeAccuracySpecial'] = 1 + stats['TotalMeleeAccuracyBonus'];
        if (dw) {
            stats['MeleeAccuracyBasic2'] = 0.57 + stats['TotalMeleeAccuracyBonus'];
            stats['MeleeAccuracySpecial2'] = 0.67 + stats['TotalMeleeAccuracyBonus'];
        }

        totalBonus = 0.05 + stats['CritStrengthBonus'] + stats['CritBonus'];

        if (stats['BuffMeleeCritChance'] != null)
            totalBonus += stats['BuffMeleeCritChance'];

        if (isAim) {
            stats['MeleeCritAimBonus'] = stats['CritAimBonus'];
            totalBonus += stats['CritAimBonus'];
        }

        if (isWill) {
            stats['MeleeCritWillBonus'] = stats['CritWillBonus'];
            totalBonus += stats['CritWillBonus'];
        }

        stats['MeleeCritChance'] = totalBonus;

        incrementStat(stats, 'BuffSurge', 0);

        stats['MeleeCritMult'] = 0.5 + stats['SurgeBonus'] + stats['BuffSurge'];

        if (dw && ohDmg != null) {
            incrementStat(stats, 'MeleeDamage2Mult', 0.3);
            stats['MeleeBaseDamage2'] = ohDmg.min.toFixed(1) + ' - ' + ohDmg.max.toFixed(1);
            stats['MeleeDamage2'] = (ohDmg.min * stats['MeleeDamage2Mult']).toFixed(1) + ' - ' + (ohDmg.max * stats['MeleeDamage2Mult']).toFixed(1);
        }
    }

    function addTechStats(stats, lvl, cls, acls, mhDmg, ohDmg, isComp) {

        var isAim = cls == 'BountyHunter' || cls == 'Trooper' || cls == 'MeleeAim';
        var isCun = cls == 'ImperialAgent' || cls == 'Smuggler';
        var isHeal = acls == 'Operative' || acls == 'Scoundrel' || acls == 'Mercenary' || acls == 'Commando';

        // damage
        var totalBonus = 0;

        if (isAim) {
            stats['TechAimBonusDamage'] = stats['Aim'] * 0.2;
            totalBonus += stats['TechAimBonusDamage'];
        }

        stats['TechCunningBonusDamage'] = stats['Cunning'] * 0.2;
        totalBonus += stats['TechCunningBonusDamage'];

        stats['TechPowerBonusDamage'] = stats['Power'] * 0.23;
        totalBonus += stats['TechPowerBonusDamage'];

        var techPower = 0;
        if (mhDmg != null && mhDmg.powerType == 'Tech')
            techPower += mhDmg.power;
        if (ohDmg != null && ohDmg.powerType == 'Tech')
            techPower += ohDmg.power;
        stats['TechPower'] = techPower;

        stats['TechTechPowerBonusDamage'] = techPower * 0.23;
        totalBonus += stats['TechTechPowerBonusDamage'];

        if (isComp)
            totalBonus += stats['PresenceBonusDamage'];

        if (stats['BuffTechBonusDamage'] != null)
            totalBonus *= 1 + stats['BuffTechBonusDamage'];

        stats['TechBonusDamage'] = totalBonus;

        // healing
        if (isHeal) {
            stats['TechTechPowerBonusHealing'] = techPower * 0.17;
            stats['TechPowerBonusHealing'] = stats['Power'] * 0.17;

            totalBonus = stats['TechTechPowerBonusHealing'] + stats['TechPowerBonusHealing'];
            if (isAim) {
                stats['TechAimBonusHealing'] = stats['Aim'] * 0.14;
                totalBonus += stats['TechAimBonusHealing'];
            }
            if (isCun) {
                stats['TechCunningBonusHealing'] = stats['Cunning'] * 0.14;
                totalBonus += stats['TechCunningBonusHealing'];
            }

            if (isComp)
                totalBonus += stats['PresenceBonusHealing'];

            if (stats['BuffTechBonusHealing'] != null)
                totalBonus *= 1 + stats['BuffTechBonusHealing'];

            stats['TechBonusHealing'] = totalBonus;
        }

        incrementStat(stats, 'BuffTechAccuracy', 0);
        stats['TotalTechAccuracyBonus'] = stats['BuffTechAccuracy'] + stats['AccuracyBonus'];
        stats['TechAccuracy'] = 1 + stats['TotalTechAccuracyBonus'];

        // crit
        stats['TechCritCunningBonus'] = stats['CritCunningBonus'];
        totalBonus = 0.05 + stats['CritCunningBonus'] + stats['CritBonus'];
        if (isAim) {
            stats['TechCritAimBonus'] = stats['CritAimBonus'];
            totalBonus += stats['CritAimBonus'];
        }

        if (stats['BuffTechCritChance'] != null)
            totalBonus += stats['BuffTechCritChance'];

        stats['TechCritChance'] = totalBonus;

        incrementStat(stats, 'BuffTechCritMult', 0);
        incrementStat(stats, 'BuffSurge', 0);
        stats['TechCritMult'] = 0.5 + stats['SurgeBonus'] + stats['BuffTechCritMult'] + stats['BuffSurge'];

        incrementStat(stats, 'BuffAlacrity', 0);
        stats['TechSpeed'] = stats['AlacrityBonus'] + stats['BuffAlacrity'];
    }

    function addDefenseStats(stats, lvl, acls, hasShield, comp, isComp, setBonuses) {

        // as of 2.0, damage reduction from armor rating formula uses a different
        // coefficient in the divisor depending on level
        // first index value corresponds to lvls 1-50, rest are 51-55
        var dr_Armor = [200, 208, 216, 224, 232, 240];

        // if xalek has a shield equipped, assume he's in defensive stance with the armor buff
        if (hasShield && comp == 'Xalek')
            incrementStat(stats, 'BuffArmor', 0.42);

        var totalHealth = healthLookup[lvl];

        stats['HealthEnduranceBonus'] = stats['Endurance'] * 10;
        totalHealth += stats['HealthEnduranceBonus'];

        if (isComp)
            totalHealth += stats['PresenceBonusHealth'];

        incrementStat(stats, 'HealthMultiplier', 0);

        stats['Health'] = totalHealth * (1 + stats['HealthMultiplier']);

        stats['ArmorDr'] = stats['Armor'] == 0 ? 0 : stats['Armor'] / (stats['Armor'] + dr_Armor[(lvl > 50 ? lvl - 50 : 0)] * lvl + 800);

        incrementStat(stats, 'BuffDrKinetic', 0);
        incrementStat(stats, 'BuffDrEnergy', 0);
        incrementStat(stats, 'BuffDrInternal', 0);
        incrementStat(stats, 'BuffDrElemental', 0);

        if (!isComp && setBonuses[10] != null && setBonuses[10] >= 4) {
            incrementStat(stats, 'BuffDrKinetic', 0.02);
            incrementStat(stats, 'BuffDrEnergy', 0.02);
            incrementStat(stats, 'BuffDrInternal', 0.02);
            incrementStat(stats, 'BuffDrElemental', 0.02);
        }

        stats['DrKinetic'] = stats['BuffDrKinetic'] + stats['ArmorDr'];
        stats['DrEnergy'] = stats['BuffDrEnergy'] + stats['ArmorDr'];
        stats['DrInternal'] = stats['BuffDrInternal'];
        stats['DrElemental'] = stats['BuffDrElemental'];

        incrementStat(stats, 'BuffDodgeMelee', 0);
        incrementStat(stats, 'BuffResistance', 0);

        if (!isComp && setBonuses[16] != null && setBonuses[16] >= 4) {
            incrementStat(stats, 'BuffDodgeMelee', 0.02);
        }

        stats['DodgeBase'] = 0.05 + (acls == 'Shadow' || acls == 'Assassin' ? 0.05 : 0);
        stats['DodgeMelee'] = stats['DodgeBase'] + stats['BuffDodgeMelee'] + stats['DefenseBonus'];
        stats['Resistance'] = stats['BuffResistance'];

        incrementStat(stats, 'BuffShieldChance', 0);
        stats['GeneratorShieldChance'] = hasShield ? 0.05 : 0;
        stats['BuffShieldChance'] = hasShield ? stats['BuffShieldChance'] : 0;

        // all defensive companions have a 20% shield chance defensive mode that we assume is active if they have a shield equipped
        if (hasShield && comp != null && comp != 'None')
            incrementStat(stats, 'BuffShieldChance', 0.2);

        stats['ShieldChance'] = hasShield ? stats['GeneratorShieldChance'] + stats['BuffShieldChance'] + stats['ShieldingBonus'] : 0;

        incrementStat(stats, 'BuffAbsorb', 0);
        stats['GeneratorAbsorb'] = hasShield ? 0.2 : 0;
        stats['BuffAbsorb'] = hasShield ? stats['BuffAbsorb'] : 0;
        stats['ShieldAbsorb'] = hasShield ? stats['GeneratorAbsorb'] + stats['BuffAbsorb'] + stats['AbsorbBonus'] : 0;
    }

    function addForceStats(stats, lvl, cls, acls, mhDmg, ohDmg, isComp) {

        var isStr = cls == 'SithWarrior' || cls == 'JediKnight';
        var isHeal = acls == 'Sage' || acls == 'Sorcerer';

        // damage
        var totalBonus = 0;
        stats['ForceWillBonusDamage'] = stats['Will'] * 0.2;
        totalBonus += stats['ForceWillBonusDamage'];

        if (isStr) {
            stats['ForceStrengthBonusDamage'] = stats['Strength'] * 0.2;
            totalBonus += stats['ForceStrengthBonusDamage'];
        }

        stats['ForcePowerBonusDamage'] = stats['Power'] * 0.23;
        totalBonus += stats['ForcePowerBonusDamage'];

        var forcePower = 0;
        if (mhDmg != null && mhDmg.powerType == 'Force')
            forcePower += mhDmg.power;
        if (ohDmg != null && ohDmg.powerType == 'Force')
            forcePower += ohDmg.power;
        stats['ForcePower'] = forcePower;

        stats['ForceForcePowerBonusDamage'] = forcePower * 0.23;
        totalBonus += stats['ForceForcePowerBonusDamage'];

        if (isComp)
            totalBonus += stats['PresenceBonusDamage'];

        if (stats['BuffForceBonusDamage'] != null)
            totalBonus *= 1 + stats['BuffForceBonusDamage'];

        stats['ForceBonusDamage'] = totalBonus;

        // healing
        if (isHeal) {
            stats['ForceWillBonusHealing'] = stats['Will'] * 0.14;
            stats['ForceForcePowerBonusHealing'] = forcePower * 0.17;
            stats['ForcePowerBonusHealing'] = stats['Power'] * 0.17;
            stats['ForceBonusHealing'] = stats['ForceWillBonusHealing'] + stats['ForceForcePowerBonusHealing'] + stats['ForcePowerBonusHealing'];

            if (isComp)
                totalBonus += stats['PresenceBonusHealing'];

            if (stats['BuffForceBonusHealing'] != null)
                stats['ForceBonusHealing'] *= 1 + stats['BuffForceBonusHealing'];
        }

        incrementStat(stats, 'BuffForceAccuracy', 0);
        stats['TotalForceAccuracyBonus'] = stats['BuffForceAccuracy'] + stats['AccuracyBonus'];
        stats['ForceAccuracy'] = 1 + stats['TotalForceAccuracyBonus'];

        // crit
        stats['ForceCritWillBonus'] = stats['CritWillBonus'];
        totalBonus = 0.05 + stats['CritWillBonus'] + stats['CritBonus'];
        if (isStr) {
            stats['ForceCritStrengthBonus'] = stats['CritStrengthBonus'];
            totalBonus += stats['CritStrengthBonus'];
        }
        if (stats['BuffForceCritChance'] != null)
            totalBonus += stats['BuffForceCritChance'];

        stats['ForceCritChance'] = totalBonus;

        incrementStat(stats, 'BuffForceCritMult', 0);
        incrementStat(stats, 'BuffSurge', 0);
        stats['ForceCritMult'] = 0.5 + stats['SurgeBonus'] + stats['BuffForceCritMult'] + stats['BuffSurge'];

        incrementStat(stats, 'BuffAlacrity', 0);
        stats['ForceSpeed'] = stats['AlacrityBonus'] + stats['BuffAlacrity'];

	 // As of 2.0, ForceRegen is increased by the Alacrity Bonus
	 incrementStat(stats, 'BuffForceRegen', 0);
	 var totalForceRegenWithoutAlacrity = 8 * (1 + stats['BuffForceRegen']);
	 stats['ForceRegen'] = totalForceRegenWithoutAlacrity + ( totalForceRegenWithoutAlacrity * stats['ForceSpeed']);
    }

    // As of 2.0, calcuation of contribution from Expertise is linear and capped
    function addPvpStats(stats, lvl) {

        //stats['PvpDamage'] = getDiminishedStat('PvpDamage', lvl, stats['Expertise']);
        //stats['PvpHealing'] = getDiminishedStat('PvpHealing', lvl, stats['Expertise']);
        //stats['PvpReduction'] = 1 - 1 / (1 + stats['PvpDamage']);

        var pvp_factor = stats['Expertise'] / 3355;
        pvp_factor = (pvp_factor > 0.6 ? 0.6 : pvp_factor);

        stats['PvpDamage'] = pvp_factor;
        stats['PvpHealing'] = stats['Expertise'] / 9900;
        stats['PvpReduction'] = 1 - (1 / (1 + pvp_factor));
    }

    calc.getWeaponDamage = function (item) {

        if (item == null)
            return null;
        else {
            if (item.ShieldType != null)
                return Tr.Swtor.Calc.Shield.GetPower(item);
            else
                return Tr.Swtor.Calc.Weapon.GetDamage(item);
        }
    };

    // get all relevant stat groups for a class/companion
    calc.getStatGroups = function (profile, comp) {

        var statGroups;
        if (profile != null) {
            var acls = profile.AdvancedClass;
            if (comp != 'None')
                acls = companionToAdvancedClass[comp];

            switch (acls) {
                case 'Guardian':
                case 'Sentinel':
                case 'Shadow':
                case 'Assassin':
                case 'Juggernaut':
                case 'Marauder':
                    statGroups = ['Melee', 'Force', 'Stance'];
                    break;
                case 'Sage':
                case 'Sorcerer':
                    statGroups = ['Melee', 'Force'];
                    break;
                case 'Mercenary':
                case 'Powertech':
                case 'Commando':
                case 'Vanguard':
                    statGroups = ['Ranged', 'Tech', 'Stance'];
                    break;
                case 'Operative':
                case 'Sniper':
                case 'Gunslinger':
                case 'Scoundrel':
                    statGroups = ['Ranged', 'Tech'];
                    break;
                case 'MeleeAim':
                    statGroups = ['Melee', 'Tech'];
                    break;
                default:
                    statGroups = [];
                    break;
            }
        }
        else
            statGroups = [];

        if (comp != 'None') {
            var index = statGroups.indexOf('Stance');
            if (index != -1)
                statGroups.splice(index, 1);
        }

        statGroups.push('Primary', 'Secondary', 'Defense', 'Pvp', 'Buff');
        return statGroups;
    };

    // get all "important" stat groups for a class/companion (expanded by default; unimportant are collapsed by default)
    calc.getImportantStatGroups = function (profile, comp) {

        var statGroups;
        if (profile != null) {
            var acls = profile.AdvancedClass;
            if (comp != null && comp != 'None')
                acls = companionToAdvancedClass[comp];

            switch (acls) {
                case 'Guardian':
                case 'Juggernaut':
                    statGroups = ['Melee', 'Force', 'Stance', 'Defense'];
                    break;
                case 'Sentinel':
                case 'Marauder':
                    statGroups = ['Melee', 'Force', 'Stance'];
                    break;
                case 'Sage':
                case 'Sorcerer':
                    statGroups = ['Force'];
                    break;
                case 'Shadow':
                case 'Assassin':
                    statGroups = ['Melee', 'Force', 'Defense'];
                    break;
                case 'Powertech':
                case 'Vanguard':
                    statGroups = ['Ranged', 'Tech', 'Defense', 'Stance'];
                    break;
                case 'Sniper':
                case 'Gunslinger':
                case 'Scoundrel':
                case 'Operative':
                    statGroups = ['Ranged', 'Tech'];
                    break;
                case 'Mercenary':
                case 'Commando':
                    statGroups = ['Ranged', 'Tech', 'Stance'];
                    break;
                case 'MeleeAim':
                    statGroups = ['Melee', 'Tech'];
                    break;
                default:
                    statGroups = new Array();
                    break;
            }
        }
        else
            statGroups = new Array();

        statGroups.push('Primary', 'Buff');
        return statGroups;
    };

    calc.getStances = function (profile, comp) {

        if (profile != null) {
            var acls = profile.AdvancedClass;
            if (comp != 'None')
                acls = comp;
            switch (acls) {
                case 'Assassin':
                case 'Shadow':
                    return ['DarkCharge', 'LightningCharge', 'SurgingCharge'];
                case 'Powertech':
                case 'Vanguard':
                    return ['Combustible', 'HighEnergy', 'Ion'];
                case 'Mercenary':
                case 'Commando':
                    return ['Combustible', 'CombatSupport', 'HighVelocity'];
                case 'Juggernaut':
                case 'Guardian':
                    return ['ShiiCho', 'Shien', 'Soresu'];
                case 'Marauder':
                case 'Sentinel':
                    return ['ShiiCho', 'Ataru', 'Juyo'];
                default:
                    return [];
            }
        }
        else
            return [];
    };

    calc.getAllStats = function (profile, gearSet, comp, buffs, stance, stimLevel, stimType) {

        var isComp = comp != null && comp != 'None';
        var acls = profile.AdvancedClass;
        if (isComp)
            acls = companionToAdvancedClass[comp];
        var cls = acls == 'MeleeAim' ? 'MeleeAim' : Tr.Swtor.getClassFromAdvancedClass(acls);
        var lvl = profile.Level;

        var stats = getBaseStats(cls, lvl, comp);

        var baseStatAmounts = {};
        for (var stat in stats)
            baseStatAmounts['Base' + stat] = stats[stat];
        for (var stat in baseStatAmounts)
            stats[stat] = baseStatAmounts[stat];

        var setBonuses = {};

        // total up stats on gear, and find active set bonuses
        var statsFromGear = {};
        if (gearSet != null) {
            $.each(gearSet, function (i, item) {

                var setId = Tr.Swtor.getItemSetId(item);
                if (setId != null) {
                    if (setBonuses[setId] == null)
                        setBonuses[setId] = 1;
                    else
                        setBonuses[setId]++;
                }

                if (item.Stats !== null) {
                    for (var statName in item.Stats) {
                        statsFromGear[statName] = (statsFromGear[statName] || 0) + item.Stats[statName];
                    }

                    if (item.Stats.Armor == null) {
                        var armor = null;
                        if (item.ArmorType !== undefined && item.ArmorType !== null) {
                            armor = Tr.Swtor.Calc.Armor.GetArmor(item);
                        }
                        else if (item.Mods != null) {
                            // see if this is adaptive armor (has no armor type, but has an armoring mod)
                            var hasArmoring = false;
                            for (var i = 0; i < item.Mods.length; i++) {
                                if (item.Mods[i].Slot == 'Armoring' && item.Mods[i].Id !== undefined && item.Mods[i].Id != 0) {
                                    hasArmoring = true;
                                    break;
                                }
                            }
                            if (hasArmoring) {
                                // temporarily set the armor type to the proper type depending on the advanced class/companion
                                item.ArmorType = Tr.Swtor.getArmorType(acls, comp);
                                armor = Tr.Swtor.Calc.Armor.GetArmor(item);
                                delete item.ArmorType;
                            }
                        }

                        if (armor !== null) {
                            statsFromGear.Armor = (statsFromGear.Armor || 0) + armor;
                        }
                    }
                }
            });
        }

        for (var i = 0; i < secondaryStats.length; i++) {
            if (statsFromGear[secondaryStats[i]] == null) {
                statsFromGear[secondaryStats[i]] = 0;
            }
        }

        for (var stat in statsFromGear) {
            if (stats[stat] == null)
                stats[stat] = statsFromGear[stat];
            else
                stats[stat] += statsFromGear[stat];
        }

        if (stats['Armor'] == null)
            stats['Armor'] = 0;

        // find the mh and oh weapons, and get the stats on them
        var mhDmg = getWeaponDamageHelper('MainHand', gearSet);
        var ohDmg = getWeaponDamageHelper('OffHand', gearSet);

        var hasShield = false;
        if (gearSet != null) {
            for (var i = 0; i < gearSet.length; i++) {
                if (gearSet[i].ShieldType == 'ForceShield' || gearSet[i].ShieldType == 'TechShield') {
                    hasShield = true;
                    break;
                }
            }
        }

        // apply datacrons to stats
        applyDatacronsToStats(stats, profile.Datacrons, isComp);

        // apply buffs and stances to stats
        applyBuffsToStats(stats, buffs, stance, stimLevel, stimType, isComp);

        // apply skill tree bonuses to stats
        applySkillsToStats(stats, lvl, acls, profile.SkillString, stance, isComp);

        // apply legacy bonuses to stats
        applyLegacyBonusesToStats(stats, profile, isComp);

        // presence stats
        stats['CompanionHealth'] = stats['Presence'] * 5;
        stats['CompanionDamage'] = stats['Presence'] * 0.2;
        stats['CompanionHealing'] = stats['Presence'] * 0.14;

        if (isComp) {
            stats['PresenceBonusHealth'] = stats['CompanionHealth'];
            stats['PresenceBonusDamage'] = stats['CompanionDamage'];
            stats['PresenceBonusHealing'] = stats['CompanionHealing'];
        }

        // multiply base stats by buff multipliers
        for (var i = 0; i < baseStats.length; i++) {
            var baseStat = baseStats[i];
            if (stats[baseStat] != null && stats['Buff' + baseStat] != null)
                stats[baseStat] *= 1 + stats['Buff' + baseStat];
        }

        // multiply armor
        if (stats['Armor'] != null && stats['BuffArmor'] != null)
            stats['Armor'] *= 1 + stats['BuffArmor'];

        // some basic stats used by several types
        stats['AccuracyBonus'] = getDiminishedStat('Accuracy', lvl, stats['Accuracy']);
        stats['CritBonus'] = getDiminishedStat('Crit', lvl, stats['Crit']);
        stats['SurgeBonus'] = getDiminishedStat('Surge', lvl, stats['Surge']);
        stats['AlacrityBonus'] = getDiminishedStat('Alacrity', lvl, stats['Alacrity']);
        stats['DefenseBonus'] = getDiminishedStat('Defense', lvl, stats['Defense']);
        stats['ShieldingBonus'] = getDiminishedStat('Shielding', lvl, stats['Shielding']);
        stats['AbsorbBonus'] = getDiminishedStat('Absorb', lvl, stats['Absorb']);

        stats['CritStrengthBonus'] = getDiminishedStat('CritPrimary', lvl, stats['Strength']);
        stats['CritAimBonus'] = getDiminishedStat('CritPrimary', lvl, stats['Aim']);
        stats['CritCunningBonus'] = getDiminishedStat('CritPrimary', lvl, stats['Cunning']);
        stats['CritWillBonus'] = getDiminishedStat('CritPrimary', lvl, stats['Will']);

        addRangedStats(stats, lvl, cls, mhDmg, ohDmg, isRangedDw(acls), isComp);
        addMeleeStats(stats, lvl, cls, acls, mhDmg, ohDmg, isMeleeDw(acls), isComp);
        addTechStats(stats, lvl, cls, acls, mhDmg, ohDmg, isComp);
        addDefenseStats(stats, lvl, acls, hasShield, comp, isComp, setBonuses);
        addForceStats(stats, lvl, cls, acls, mhDmg, ohDmg, isComp);
        addPvpStats(stats, lvl);

        // determine bonus primary stats (basically just final value minus base)
        $.each(baseStats, function (i, stat) {
            var val = stats[stat];
            var base = stats['Base' + stat];
            stats['Bonus' + stat] = val - base;
        });

        return stats;
    };

    function getWeaponDamageHelper(slot, gearSet) {

        var weapon;
        for (var i = 0; i < gearSet.length; i++) {
            if (gearSet[i].CharacterSlot == slot) {
                weapon = gearSet[i];
                break;
            }
        }

        return calc.getWeaponDamage(weapon);
    }

    function isMeleeDw(acls) {

        return acls == 'Marauder' || acls == 'Sentinel';
    }

    function isRangedDw(acls) {

        return acls == 'Mercenary' || acls == 'Gunslinger';
    }

    function applyDatacronsToStats(stats, datacrons, isComp) {

        if (datacrons == null) return;

        var dcHash = {};
        for (var i = 0; i < datacrons.length; i++)
            dcHash[datacrons[i]] = true;

        var addDatacronStat = function (ids, stat, amt) {
            for (var i = 0; i < ids.length; i++)
                if (dcHash[ids[i]] != null)
                    incrementStat(stats, stat, amt);
        };

        if (!isComp) {
            addDatacronStat([5, 30, 33, 68], 'Strength', 2);
            addDatacronStat([57, 61, 82], 'Strength', 3);
            addDatacronStat([3, 20, 42, 67, 90], 'Strength', 4);

            addDatacronStat([6, 43, 64, 70], 'Aim', 2);
            addDatacronStat([54, 59, 80], 'Aim', 3);
            addDatacronStat([0, 10, 18, 24, 46, 73], 'Aim', 4);

            addDatacronStat([31, 37, 52, 84], 'Endurance', 2);
            addDatacronStat([4, 13, 71], 'Endurance', 3);
            addDatacronStat([16, 40, 47, 66, 89], 'Endurance', 4);
            addDatacronStat([95], 'Endurance', 10);

            addDatacronStat([8, 32, 35, 72], 'Cunning', 2);
            addDatacronStat([56, 78, 83], 'Cunning', 3);
            addDatacronStat([11, 25, 41, 65, 77, 91], 'Cunning', 4);

            addDatacronStat([7, 51, 74, 85], 'Will', 2);
            addDatacronStat([2, 79], 'Will', 3);
            addDatacronStat([14, 15, 21, 27, 49, 76, 87], 'Will', 4);
        }

        addDatacronStat([28, 34, 45, 63], 'Presence', 2);
        addDatacronStat([1, 55, 60], 'Presence', 3);
        addDatacronStat([12, 17, 23, 38, 69, 88], 'Presence', 4);
        addDatacronStat([94], 'Presence', 10);

        if (dcHash[92] != null || dcHash[93] != null) {
            // all stats
            if (!isComp) {
                incrementStat(stats, 'Strength', 10);
                incrementStat(stats, 'Aim', 10);
                incrementStat(stats, 'Endurance', 10);
                incrementStat(stats, 'Cunning', 10);
                incrementStat(stats, 'Will', 10);
            }
            incrementStat(stats, 'Presence', 10);
        }
    }

    function applyLegacyBonusesToStats(stats, profile, isComp) {

        if (profile.LegacyCompanionCount != null)
            incrementStat(stats, 'Presence', profile.LegacyCompanionCount * 10);

        if (profile.LegacyCompanionBonuses != null && !isComp) {
            for (var i = 0; i < profile.LegacyCompanionBonuses.length; i++) {
                switch (profile.LegacyCompanionBonuses[i]) {
                    case 'MeleeTank':
                        incrementStat(stats, 'BuffMeleeAccuracy', 0.01);
                        incrementStat(stats, 'BuffRangedAccuracy', 0.01);
                        incrementStat(stats, 'BuffTechAccuracy', 0.01);
                        incrementStat(stats, 'BuffForceAccuracy', 0.01);
                        break;

                    case 'RangedDamage':
                        incrementStat(stats, 'BuffMeleeCritChance', 0.01);
                        incrementStat(stats, 'BuffRangedCritChance', 0.01);
                        incrementStat(stats, 'BuffTechCritChance', 0.01);
                        incrementStat(stats, 'BuffForceCritChance', 0.01);
                        break;

                    case 'MeleeDamage':
                        incrementStat(stats, 'BuffSurge', 0.01);
                        break;

                    case 'RangedTank':
                        incrementStat(stats, 'HealthMultiplier', 0.01);
                        break;
                }
            }
        }

        if (profile.LegacyHasHuman === true)
            incrementStat(stats, 'Presence', 100);
    }

    var skillFuncs = {};

    function applyBuffsToStats(stats, buffs, stance, stimLevel, stimType, isComp) {

        for (var i = 0; i < buffs.length; i++) {
            switch (buffs[i]) {
                case 'Power':
                    incrementStat(stats, 'BuffMeleeBonusDamage', 0.05);
                    incrementStat(stats, 'BuffRangedBonusDamage', 0.05);
                    incrementStat(stats, 'BuffTechBonusDamage', 0.05);
                    incrementStat(stats, 'BuffForceBonusDamage', 0.05);
                    incrementStat(stats, 'BuffTechBonusHealing', 0.05);
                    incrementStat(stats, 'BuffForceBonusHealing', 0.05);
                    break;
                case 'Stats':
                    incrementStat(stats, 'BuffStrength', 0.05);
                    incrementStat(stats, 'BuffAim', 0.05);
                    incrementStat(stats, 'BuffCunning', 0.05);
                    incrementStat(stats, 'BuffWill', 0.05);
                    incrementStat(stats, 'BuffDrInternal', 0.1);
                    incrementStat(stats, 'BuffDrElemental', 0.1);
                    break;
                case 'Endurance':
                    incrementStat(stats, 'BuffEndurance', 0.05);
                    break;
                case 'Crit':
                    incrementStat(stats, 'BuffMeleeCritChance', 0.05);
                    incrementStat(stats, 'BuffRangedCritChance', 0.05);
                    incrementStat(stats, 'BuffTechCritChance', 0.05);
                    incrementStat(stats, 'BuffForceCritChance', 0.05);
                    break;
            }
        }

        if (!isComp) {
            switch (stance) {
                case 'DarkCharge':
                    incrementStat(stats, 'BuffArmor', 1.15);
                    incrementStat(stats, 'BuffShieldChance', 0.15);
                    incrementStat(stats, 'BuffMeleeBonusDamage', -0.05);
                    break;
                case 'Soresu':
                    incrementStat(stats, 'BuffArmor', 0.615);
                    incrementStat(stats, 'BuffShieldChance', 0.15);
                    incrementStat(stats, 'BuffDrKinetic', 0.06);
                    incrementStat(stats, 'BuffDrEnergy', 0.06);
                    incrementStat(stats, 'BuffDrInternal', 0.06);
                    incrementStat(stats, 'BuffDrElemental', 0.06);
                    break;
                case 'ShiiCho':
                    incrementStat(stats, 'BuffMeleeBonusDamage', 0.03);
                    incrementStat(stats, 'BuffDrKinetic', 0.03);
                    incrementStat(stats, 'BuffDrEnergy', 0.03);
                    incrementStat(stats, 'BuffDrInternal', 0.03);
                    incrementStat(stats, 'BuffDrElemental', 0.03);
                    break;
                case 'Ataru':
                    incrementStat(stats, 'BuffMeleeAccuracy', 0.03);
                    incrementStat(stats, 'BuffForceAccuracy', 0.03);
                    break;
                case 'HighVelocity':
                    incrementStat(stats, 'BuffAlacrity', 0.03);
                    break;
                case 'Ion':
                    incrementStat(stats, 'BuffArmor', 0.615);
                    incrementStat(stats, 'BuffShieldChance', 0.15);
                    incrementStat(stats, 'BuffDrKinetic', 0.05);
                    incrementStat(stats, 'BuffDrEnergy', 0.05);
                    incrementStat(stats, 'BuffDrInternal', 0.05);
                    incrementStat(stats, 'BuffDrElemental', 0.05);
                    break;
            }

            if (stimLevel != 0 && stimType != null) {
                var stat1, stat2;
                switch (stimType) {
                    case 'Will':
                    case 'Strength':
                    case 'Aim':
                    case 'Cunning':
                        stat1 = stimType;
                        stat2 = 'Power';
                        break;
                    case 'Endurance':
                        stat1 = stimType;
                        stat2 = 'Defense';
                        break;
                }

                switch (stimLevel) {
                    case 1:
                        incrementStat(stats, stat1, 128);
                        incrementStat(stats, stat2, 52);
                        break;
                    case 2:
                        incrementStat(stats, stat1, 112);
                        incrementStat(stats, stat2, 46);
                        break;
                    case 3:
                        incrementStat(stats, stat1, 104);
                        incrementStat(stats, stat2, 43);
                        break;
                    case 4:
                        incrementStat(stats, stat1, 169);
                        incrementStat(stats, stat2, 70);
                        break;
                }
            }
        }
    }

    // also applies static ability buffs and party buffs
    function applySkillsToStats(stats, lvl, acls, skillString, stance, isComp) {

        // parse the skill string into an array of ints for each tree
        var trees = null;
        if (skillString != null) {
            trees = new Array();
            var parts = skillString.split('-');
            $.each(parts, function (i, val) {
                var points = new Array();
                for (var i = 0; i < val.length; i++)
                    points.push(parseInt(val[i]));
                points.reverse();
                trees.push(points);
            });
        }

        if (!isComp || (acls == 'Sage' || acls == 'Sorcerer')) {
            // call the custom method for each advanced class that applies skills to stats
            skillFuncs[acls](stats, lvl, trees, stance, isComp);
        }
    }

    // updated stat contributions from skill trees for Powertech/Vanguard 2.0
    skillFuncs.Powertech = function (stats, lvl, trees, stance) {

        if (trees == null) return;

        if (stance == 'Ion') {
            incrementStat(stats, 'BuffDrKinetic', 0.01 * trees[0][12]);
            incrementStat(stats, 'BuffDrEnergy', 0.01 * trees[0][12]);
            incrementStat(stats, 'BuffDrInternal', 0.01 * trees[0][12]);
            incrementStat(stats, 'BuffDrElemental', 0.01 * trees[0][12]);
        }
        else if (stance == 'Combustible') {
            incrementStat(stats, 'BuffTechCritChance', 0.03 * trees[1][3]);
        }

        incrementStat(stats, 'BuffAim', 0.03 * trees[0][1]);
        incrementStat(stats, 'BuffArmor', 0.08 * trees[0][6]);
        incrementStat(stats, 'BuffShieldChance', 0.01 * trees[0][7]);
        incrementStat(stats, 'BuffAbsorb', 0.02 * trees[0][9]);
        incrementStat(stats, 'BuffDodgeMelee', 0.04 * trees[0][18]);

        incrementStat(stats, 'BuffDrKinetic', 0.01 * trees[1][1]);
        incrementStat(stats, 'BuffDrEnergy', 0.01 * trees[1][1]);
        incrementStat(stats, 'BuffDrInternal', 0.01 * trees[1][1]);
        incrementStat(stats, 'BuffDrElemental', 0.01 * trees[1][1]);
        incrementStat(stats, 'BuffRangedAccuracy', 0.01 * trees[1][2]);
        incrementStat(stats, 'BuffTechAccuracy', 0.01 * trees[1][2]);

        incrementStat(stats, 'BuffEndurance', 0.01 * trees[2][2]);
        incrementStat(stats, 'BuffDodgeMelee', 0.01 * trees[2][9]);
        incrementStat(stats, 'BuffTechCritChance', 0.01 * trees[2][16]);
        incrementStat(stats, 'BuffAlacrity', 0.01 * trees[2][17]);
    };
    skillFuncs.Vanguard = skillFuncs.Powertech;

    // updated stat contributions from skill trees for Sorcerer/Sage 2.0
    skillFuncs.Sorcerer = function (stats, lvl, trees, stance, isComp) {

        if (trees == null) return;

        if (!isComp) {
            incrementStat(stats, 'BuffAlacrity', 0.01 * trees[0][0]);
            incrementStat(stats, 'BuffForceCritChance', 0.01 * trees[0][1]);
            incrementStat(stats, 'BuffForceCritChance', 0.01 * trees[0][17]);
            incrementStat(stats, 'BuffDrKinetic', 0.01 * trees[0][17]);
            incrementStat(stats, 'BuffDrEnergy', 0.01 * trees[0][17]);
            incrementStat(stats, 'BuffDrInternal', 0.01 * trees[0][17]);
            incrementStat(stats, 'BuffDrElemental', 0.01 * trees[0][17]);
            incrementStat(stats, 'BuffForceBonusHealing', 0.02 * trees[0][19]);

            incrementStat(stats, 'BuffForceAccuracy', 0.01 * trees[1][2]);

            incrementStat(stats, 'BuffDrKinetic', 0.01 * trees[2][1]);
            incrementStat(stats, 'BuffDrEnergy', 0.01 * trees[2][1]);
            incrementStat(stats, 'BuffDrInternal', 0.01 * trees[2][1]);
            incrementStat(stats, 'BuffDrElemental', 0.01 * trees[2][1]);
            incrementStat(stats, 'BuffWill', trees[2][2] * 0.03);
        }
    };
    skillFuncs.Sage = skillFuncs.Sorcerer;

    // updated stat contributions from skill trees for Assassin/Shadow 2.0
    skillFuncs.Assassin = function (stats, lvl, trees, stance) {

        // assassin/shadow's training ability gives 5 endurance per rank at 10, 20, 30, etc.
        // assassin/shadow's training ability gives an additional rank at level 55
        stats['Endurance'] += 5 * Math.floor(lvl / 10);
        if (lvl >= 55) {
            stats['Endurance'] += 5;
        }

        if (trees == null) return;

        if (stance == 'DarkCharge') {
            incrementStat(stats, 'BuffDrInternal', 0.03 * trees[0][2]);
            incrementStat(stats, 'BuffDrElemental', 0.03 * trees[0][2]);
        }
        else if (stance == 'LightningCharge') {
            incrementStat(stats, 'BuffArmor', 0.15 * trees[1][5]);
        }
        else if (stance == 'SurgingCharge') {
            incrementStat(stats, 'BuffArmor', 0.15 * trees[1][5]);
        }

        incrementStat(stats, 'BuffDodgeMelee', 0.02 * trees[0][1]);
        incrementStat(stats, 'BuffMeleeAccuracy', 0.01 * trees[0][2]);
        incrementStat(stats, 'BuffForceAccuracy', 0.01 * trees[0][2]);
        incrementStat(stats, 'BuffEndurance', 0.01 * trees[0][3]);
        incrementStat(stats, 'BuffDodgeMelee', 0.01 * trees[0][10]);
        incrementStat(stats, 'BuffAbsorb', 0.02 * trees[0][11]);
        incrementStat(stats, 'BuffForceRegen', 0.10 * trees[0][12]);

        incrementStat(stats, 'BuffDrKinetic', 0.01 * trees[2][1]);
        incrementStat(stats, 'BuffDrEnergy', 0.01 * trees[2][1]);
        incrementStat(stats, 'BuffDrInternal', 0.01 * trees[2][1]);
        incrementStat(stats, 'BuffDrElemental', 0.01 * trees[2][1]);
    };
    skillFuncs.Shadow = skillFuncs.Assassin;

    // updated stat contributions from skill trees for Juggernaut/Guardian 2.0
    skillFuncs.Juggernaut = function (stats, lvl, trees, stance) {

        if (trees == null) return;

        if (stance == 'Soresu') {
            incrementStat(stats, 'BuffArmor', 0.15 * trees[0][5]);
            incrementStat(stats, 'BuffDodgeMelee', 0.01 * trees[1][0]);
        }
        else if (stance == 'ShiiCho') {
            incrementStat(stats, 'BuffForceBonusDamage', 0.02 * trees[1][0]);
        }
        else if (stance == 'Shien') {
            incrementStat(stats, 'BuffMeleeBonusDamage', 0.02 * trees[1][0]);
        }

        incrementStat(stats, 'BuffShieldChance', 0.02 * trees[0][16]);
        incrementStat(stats, 'BuffDrInternal', 0.02 * trees[0][17]);
        incrementStat(stats, 'BuffDrElemental', 0.02 * trees[0][17]);

        incrementStat(stats, 'BuffMeleeAccuracy', 0.01 * trees[1][3]);
        incrementStat(stats, 'BuffForceAccuracy', 0.01 * trees[1][3]);
        incrementStat(stats, 'BuffStrength', 0.03 * trees[1][4]);
        incrementStat(stats, 'BuffDrKinetic', 0.02 * trees[1][16]);
        incrementStat(stats, 'BuffDrEnergy', 0.02 * trees[1][16]);
        incrementStat(stats, 'BuffDrInternal', 0.02 * trees[1][16]);
        incrementStat(stats, 'BuffDrElemental', 0.02 * trees[1][16]);

        incrementStat(stats, 'BuffForceCritChance', 0.02 * trees[2][0]);
        incrementStat(stats, 'BuffForceCritMult', 0.05 * trees[2][16]);
    };
    skillFuncs.Guardian = skillFuncs.Juggernaut;

    // updated stat contributions from skill trees for Marauder/Sentinel 2.0
    skillFuncs.Marauder = function (stats, lvl, trees, stance) {

        if (trees == null) return;

        if (stance == 'ShiiCho') {
            incrementStat(stats, 'BuffDrKinetic', 0.01 * trees[1][2]);
            incrementStat(stats, 'BuffDrEnergy', 0.01 * trees[1][2]);
            incrementStat(stats, 'BuffDrInternal', 0.01 * trees[1][2]);
            incrementStat(stats, 'BuffDrElemental', 0.01 * trees[1][2]);
        }
        else if (stance == 'Juyo') {
            incrementStat(stats, 'BuffDrInternal', 0.02 * trees[1][2]);
            incrementStat(stats, 'BuffDrElemental', 0.02 * trees[1][2]);
        }

        incrementStat(stats, 'BuffMeleeAccuracy', 0.01 * trees[1][1]);
        incrementStat(stats, 'BuffForceAccuracy', 0.01 * trees[1][1]);
        incrementStat(stats, 'MeleeDamage2Mult', 0.12 * trees[1][3]);

        incrementStat(stats, 'BuffForceCritChance', 0.02 * trees[2][0]);
        incrementStat(stats, 'BuffForceCritMult', 0.05 * trees[2][16]);
        incrementStat(stats, 'BuffDrKinetic', 0.01 * trees[2][17]);
        incrementStat(stats, 'BuffDrEnergy', 0.01 * trees[2][17]);
        incrementStat(stats, 'BuffDrInternal', 0.01 * trees[2][17]);
        incrementStat(stats, 'BuffDrElemental', 0.01 * trees[2][17]);
    };
    skillFuncs.Sentinel = skillFuncs.Marauder;

    // updated stat contributions from skill trees for Sniper/Gunslinger 2.0
    skillFuncs.Sniper = function (stats, lvl, trees, stance) {

        if (trees == null) return;

        incrementStat(stats, 'BuffRangedAccuracy', 0.01 * trees[0][2]);
        incrementStat(stats, 'BuffTechAccuracy', 0.01 * trees[0][2]);

        incrementStat(stats, 'BuffCunning', 0.03 * trees[1][2]);
        incrementStat(stats, 'BuffEndurance', 0.02 * trees[1][6]);

        incrementStat(stats, 'BuffAlacrity', 0.01 * trees[2][0]);
        incrementStat(stats, 'BuffRangedCritChance', 0.01 * trees[2][1]);
        incrementStat(stats, 'BuffTechCritChance', 0.01 * trees[2][1]);
    };
    skillFuncs.Gunslinger = skillFuncs.Sniper;

    // updated stat contributions from skill trees for Operative/Scoundrel 2.0
    skillFuncs.Operative = function (stats, lvl, trees, stance) {

        if (trees == null) return;

        incrementStat(stats, 'BuffCunning', 0.03 * trees[0][2]);
	//removed to match in-game GUI, known issue
        //incrementStat(stats, 'BuffTechBonusHealing', 0.01 * trees[0][4]);
        incrementStat(stats, 'BuffDrKinetic', 0.02 * trees[0][6]);
        incrementStat(stats, 'BuffDrEnergy', 0.02 * trees[0][6]);
        incrementStat(stats, 'BuffDrInternal', 0.02 * trees[0][6]);
        incrementStat(stats, 'BuffDrElemental', 0.02 * trees[0][6]);

        incrementStat(stats, 'BuffRangedAccuracy', 0.01 * trees[1][1]);
        incrementStat(stats, 'BuffTechAccuracy', 0.01 * trees[1][1]);
        //removed to match in-game GUI, known issue
        //incrementStat(stats, 'BuffTechBonusHealing', 0.01 * trees[1][2]);
        incrementStat(stats, 'BuffEndurance', 0.02 * trees[1][5]);
        incrementStat(stats, 'BuffDodgeMelee', 0.01 * trees[1][6]);

        incrementStat(stats, 'BuffAlacrity', 0.01 * trees[2][0]);
        incrementStat(stats, 'BuffRangedCritChance', 0.01 * trees[2][1]);
        incrementStat(stats, 'BuffTechCritChance', 0.01 * trees[2][1]);
    };
    skillFuncs.Scoundrel = skillFuncs.Operative;

    // updated stat contributions from skill trees for Mercenary/Commando 2.0
    skillFuncs.Mercenary = function (stats, lvl, trees, stance) {

        if (trees == null) return;

        if (stance == 'CombatSupport') {
            incrementStat(stats, 'BuffDrKinetic', 0.02 * trees[0][11]);
            incrementStat(stats, 'BuffDrEnergy', 0.02 * trees[0][11]);
            incrementStat(stats, 'BuffDrInternal', 0.02 * trees[0][11]);
            incrementStat(stats, 'BuffDrElemental', 0.02 * trees[0][11]);
            incrementStat(stats, 'BuffTechCritChance', 0.01 * trees[1][4]);
        }
        else if (stance == 'Combustible') {
            incrementStat(stats, 'BuffRangedCritChance', 0.02 * trees[1][4]);
        }

        incrementStat(stats, 'BuffRangedCritChance', 0.01 * trees[0][2]);
        incrementStat(stats, 'BuffTechCritChance', 0.01 * trees[0][2]);

        incrementStat(stats, 'BuffAim', 0.03 * trees[1][2]);
        incrementStat(stats, 'BuffEndurance', 0.01 * trees[1][5]);

        incrementStat(stats, 'BuffRangedAccuracy', 0.01 * trees[2][0]);
        incrementStat(stats, 'BuffTechAccuracy', 0.01 * trees[2][0]);
        incrementStat(stats, 'BuffAlacrity', 0.01 * trees[2][1]);
        incrementStat(stats, 'BuffEndurance', 0.01 * trees[2][2]);
        incrementStat(stats, 'BuffDodgeMelee', 0.01 * trees[2][9]);
        incrementStat(stats, 'BuffTechCritChance', 0.01 * trees[2][16]);
        incrementStat(stats, 'BuffAlacrity', 0.01 * trees[2][17]);
    };
    skillFuncs.Commando = skillFuncs.Mercenary;

    function incrementStat(stats, stat, inc) {
        if (stats[stat] == null)
            stats[stat] = 0;
        stats[stat] += inc;
    }

    return calc;
}(TeamRobot, jQuery));


/*
THE FOLLOWING BLOCKS WERE EXTRACTED FROM THE GAME CLIENT, AND SHOULD NOT NEED TO BE MODIFIED.
*/

/*
* Copyright 2010-2013 Team Robot, Inc.
* 
* Author: Adam Tegen
*/

//
// TeamRobot.Swtor.Calc.Armor
//
TeamRobot.Swtor.Calc = (function (calc, $) {
    var armor = calc.Armor = calc.Armor || {};

    var lookup = { "Medium": { "Wrist": { "Cheap": [4, 7, 10, 13, 17, 21, 26, 31, 36, 41, 46, 51, 57, 62, 68, 74, 80, 86, 92, 98, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574], "Standard": [10, 13, 17, 21, 26, 31, 36, 41, 46, 51, 57, 62, 68, 74, 80, 86, 92, 98, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585], "Premium": [13, 17, 21, 26, 31, 36, 41, 46, 51, 57, 62, 68, 74, 80, 86, 92, 98, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591], "Prototype": [21, 26, 31, 36, 41, 46, 51, 62, 68, 74, 80, 86, 92, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591, 597, 602, 608, 614, 619], "Artifact": [26, 31, 36, 46, 51, 57, 62, 74, 80, 86, 92, 104, 110, 116, 122, 129, 135, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 250, 256, 261, 267, 273, 278, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591, 597, 602, 608, 614, 619, 625, 631, 636, 642], "Legendary": [31, 36, 46, 51, 57, 68, 74, 86, 92, 98, 110, 116, 122, 129, 141, 148, 154, 161, 168, 174, 181, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591, 597, 602, 608, 614, 619, 625, 631, 636, 642, 648, 653, 659] }, "Belt": { "Cheap": [4, 7, 10, 13, 17, 21, 26, 31, 36, 41, 46, 51, 57, 62, 68, 74, 80, 86, 92, 98, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574], "Standard": [10, 13, 17, 21, 26, 31, 36, 41, 46, 51, 57, 62, 68, 74, 80, 86, 92, 98, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585], "Premium": [13, 17, 21, 26, 31, 36, 41, 46, 51, 57, 62, 68, 74, 80, 86, 92, 98, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591], "Prototype": [21, 26, 31, 36, 41, 46, 51, 62, 68, 74, 80, 86, 92, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591, 597, 602, 608, 614, 619], "Artifact": [26, 31, 36, 46, 51, 57, 62, 74, 80, 86, 92, 104, 110, 116, 122, 129, 135, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 250, 256, 261, 267, 273, 278, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591, 597, 602, 608, 614, 619, 625, 631, 636, 642], "Legendary": [31, 36, 46, 51, 57, 68, 74, 86, 92, 98, 110, 116, 122, 129, 141, 148, 154, 161, 168, 174, 181, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591, 597, 602, 608, 614, 619, 625, 631, 636, 642, 648, 653, 659] }, "Chest": { "Cheap": [7, 11, 16, 20, 27, 34, 42, 49, 57, 65, 74, 82, 91, 100, 109, 118, 127, 137, 146, 156, 166, 176, 186, 196, 206, 216, 226, 237, 247, 258, 268, 279, 290, 300, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 427, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918], "Standard": [16, 20, 27, 34, 42, 49, 57, 65, 74, 82, 91, 100, 109, 118, 127, 137, 146, 156, 166, 176, 186, 196, 206, 216, 226, 237, 247, 258, 268, 279, 290, 300, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 427, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936], "Premium": [20, 27, 34, 42, 49, 57, 65, 74, 82, 91, 100, 109, 118, 127, 137, 146, 156, 166, 176, 186, 196, 206, 216, 226, 237, 247, 258, 268, 279, 290, 300, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 427, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936, 945], "Prototype": [34, 42, 49, 57, 65, 74, 82, 100, 109, 118, 127, 137, 146, 166, 176, 186, 196, 206, 216, 226, 237, 247, 258, 268, 279, 290, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 427, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936, 945, 955, 964, 973, 982, 991], "Artifact": [42, 49, 57, 74, 82, 91, 100, 118, 127, 137, 146, 166, 176, 186, 196, 206, 216, 237, 247, 258, 268, 279, 290, 300, 311, 322, 333, 344, 355, 366, 377, 400, 409, 418, 427, 436, 445, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936, 945, 955, 964, 973, 982, 991, 1000, 1009, 1018, 1027], "Legendary": [49, 57, 74, 82, 91, 109, 118, 137, 146, 156, 176, 186, 196, 206, 226, 237, 247, 258, 268, 279, 290, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936, 945, 955, 964, 973, 982, 991, 1000, 1009, 1018, 1027, 1036, 1045, 1055] }, "Helm": { "Cheap": [7, 11, 16, 20, 27, 34, 42, 49, 57, 65, 74, 82, 91, 100, 109, 118, 127, 137, 146, 156, 166, 176, 186, 196, 206, 216, 226, 237, 247, 258, 268, 279, 290, 300, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 427, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918], "Standard": [16, 20, 27, 34, 42, 49, 57, 65, 74, 82, 91, 100, 109, 118, 127, 137, 146, 156, 166, 176, 186, 196, 206, 216, 226, 237, 247, 258, 268, 279, 290, 300, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 427, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936], "Premium": [20, 27, 34, 42, 49, 57, 65, 74, 82, 91, 100, 109, 118, 127, 137, 146, 156, 166, 176, 186, 196, 206, 216, 226, 237, 247, 258, 268, 279, 290, 300, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 427, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936, 945], "Prototype": [34, 42, 49, 57, 65, 74, 82, 100, 109, 118, 127, 137, 146, 166, 176, 186, 196, 206, 216, 226, 237, 247, 258, 268, 279, 290, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 427, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936, 945, 955, 964, 973, 982, 991], "Artifact": [42, 49, 57, 74, 82, 91, 100, 118, 127, 137, 146, 166, 176, 186, 196, 206, 216, 237, 247, 258, 268, 279, 290, 300, 311, 322, 333, 344, 355, 366, 377, 400, 409, 418, 427, 436, 445, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936, 945, 955, 964, 973, 982, 991, 1000, 1009, 1018, 1027], "Legendary": [49, 57, 74, 82, 91, 109, 118, 137, 146, 156, 176, 186, 196, 206, 226, 237, 247, 258, 268, 279, 290, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936, 945, 955, 964, 973, 982, 991, 1000, 1009, 1018, 1027, 1036, 1045, 1055] }, "Foot": { "Cheap": [4, 7, 10, 13, 17, 21, 26, 31, 36, 41, 46, 51, 57, 62, 68, 74, 80, 86, 92, 98, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574], "Standard": [10, 13, 17, 21, 26, 31, 36, 41, 46, 51, 57, 62, 68, 74, 80, 86, 92, 98, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585], "Premium": [13, 17, 21, 26, 31, 36, 41, 46, 51, 57, 62, 68, 74, 80, 86, 92, 98, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591], "Prototype": [21, 26, 31, 36, 41, 46, 51, 62, 68, 74, 80, 86, 92, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591, 597, 602, 608, 614, 619], "Artifact": [26, 31, 36, 46, 51, 57, 62, 74, 80, 86, 92, 104, 110, 116, 122, 129, 135, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 250, 256, 261, 267, 273, 278, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591, 597, 602, 608, 614, 619, 625, 631, 636, 642], "Legendary": [31, 36, 46, 51, 57, 68, 74, 86, 92, 98, 110, 116, 122, 129, 141, 148, 154, 161, 168, 174, 181, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591, 597, 602, 608, 614, 619, 625, 631, 636, 642, 648, 653, 659] }, "Hands": { "Cheap": [4, 7, 10, 13, 17, 21, 26, 31, 36, 41, 46, 51, 57, 62, 68, 74, 80, 86, 92, 98, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574], "Standard": [10, 13, 17, 21, 26, 31, 36, 41, 46, 51, 57, 62, 68, 74, 80, 86, 92, 98, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585], "Premium": [13, 17, 21, 26, 31, 36, 41, 46, 51, 57, 62, 68, 74, 80, 86, 92, 98, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591], "Prototype": [21, 26, 31, 36, 41, 46, 51, 62, 68, 74, 80, 86, 92, 104, 110, 116, 122, 129, 135, 141, 148, 154, 161, 168, 174, 181, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 267, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591, 597, 602, 608, 614, 619], "Artifact": [26, 31, 36, 46, 51, 57, 62, 74, 80, 86, 92, 104, 110, 116, 122, 129, 135, 148, 154, 161, 168, 174, 181, 188, 194, 201, 208, 215, 222, 229, 236, 250, 256, 261, 267, 273, 278, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 347, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591, 597, 602, 608, 614, 619, 625, 631, 636, 642], "Legendary": [31, 36, 46, 51, 57, 68, 74, 86, 92, 98, 110, 116, 122, 129, 141, 148, 154, 161, 168, 174, 181, 194, 201, 208, 215, 222, 229, 236, 243, 250, 256, 261, 273, 278, 284, 290, 295, 301, 307, 313, 318, 324, 330, 335, 341, 352, 358, 364, 369, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426, 432, 438, 443, 449, 455, 460, 466, 472, 477, 483, 489, 494, 500, 506, 511, 517, 523, 528, 534, 540, 545, 551, 557, 563, 568, 574, 580, 585, 591, 597, 602, 608, 614, 619, 625, 631, 636, 642, 648, 653, 659] }, "Legs": { "Cheap": [7, 11, 16, 20, 27, 34, 42, 49, 57, 65, 74, 82, 91, 100, 109, 118, 127, 137, 146, 156, 166, 176, 186, 196, 206, 216, 226, 237, 247, 258, 268, 279, 290, 300, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 427, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918], "Standard": [16, 20, 27, 34, 42, 49, 57, 65, 74, 82, 91, 100, 109, 118, 127, 137, 146, 156, 166, 176, 186, 196, 206, 216, 226, 237, 247, 258, 268, 279, 290, 300, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 427, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936], "Premium": [20, 27, 34, 42, 49, 57, 65, 74, 82, 91, 100, 109, 118, 127, 137, 146, 156, 166, 176, 186, 196, 206, 216, 226, 237, 247, 258, 268, 279, 290, 300, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 427, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936, 945], "Prototype": [34, 42, 49, 57, 65, 74, 82, 100, 109, 118, 127, 137, 146, 166, 176, 186, 196, 206, 216, 226, 237, 247, 258, 268, 279, 290, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 427, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936, 945, 955, 964, 973, 982, 991], "Artifact": [42, 49, 57, 74, 82, 91, 100, 118, 127, 137, 146, 166, 176, 186, 196, 206, 216, 237, 247, 258, 268, 279, 290, 300, 311, 322, 333, 344, 355, 366, 377, 400, 409, 418, 427, 436, 445, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936, 945, 955, 964, 973, 982, 991, 1000, 1009, 1018, 1027], "Legendary": [49, 57, 74, 82, 91, 109, 118, 137, 146, 156, 176, 186, 196, 206, 226, 237, 247, 258, 268, 279, 290, 311, 322, 333, 344, 355, 366, 377, 389, 400, 409, 418, 436, 445, 455, 464, 473, 482, 491, 500, 509, 518, 527, 536, 545, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 655, 664, 673, 682, 691, 700, 709, 718, 727, 736, 745, 755, 764, 773, 782, 791, 800, 809, 818, 827, 836, 845, 855, 864, 873, 882, 891, 900, 909, 918, 927, 936, 945, 955, 964, 973, 982, 991, 1000, 1009, 1018, 1027, 1036, 1045, 1055] } }, "Droid": { "Wrist": { "Cheap": [7, 11, 16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765], "Standard": [16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780], "Premium": [20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788], "Prototype": [32, 39, 45, 52, 59, 66, 73, 88, 95, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826], "Artifact": [39, 45, 52, 66, 73, 81, 88, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 333, 341, 348, 356, 364, 371, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856], "Legendary": [45, 52, 66, 73, 81, 95, 103, 119, 127, 134, 151, 159, 167, 175, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856, 864, 871, 879] }, "Belt": { "Cheap": [7, 11, 16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765], "Standard": [16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780], "Premium": [20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788], "Prototype": [32, 39, 45, 52, 59, 66, 73, 88, 95, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826], "Artifact": [39, 45, 52, 66, 73, 81, 88, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 333, 341, 348, 356, 364, 371, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856], "Legendary": [45, 52, 66, 73, 81, 95, 103, 119, 127, 134, 151, 159, 167, 175, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856, 864, 871, 879] }, "Chest": { "Cheap": [12, 18, 25, 32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224], "Standard": [25, 32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248], "Premium": [32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261], "Prototype": [52, 62, 72, 83, 94, 106, 117, 141, 153, 165, 177, 190, 202, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321], "Artifact": [62, 72, 83, 106, 117, 129, 141, 165, 177, 190, 202, 228, 241, 254, 267, 281, 294, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 533, 545, 558, 570, 582, 594, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321, 1333, 1345, 1358, 1370], "Legendary": [72, 83, 106, 117, 129, 153, 165, 190, 202, 215, 241, 254, 267, 281, 307, 321, 335, 348, 362, 376, 390, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321, 1333, 1345, 1358, 1370, 1382, 1394, 1406] }, "Helm": { "Cheap": [12, 18, 25, 32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224], "Standard": [25, 32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248], "Premium": [32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261], "Prototype": [52, 62, 72, 83, 94, 106, 117, 141, 153, 165, 177, 190, 202, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321], "Artifact": [62, 72, 83, 106, 117, 129, 141, 165, 177, 190, 202, 228, 241, 254, 267, 281, 294, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 533, 545, 558, 570, 582, 594, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321, 1333, 1345, 1358, 1370], "Legendary": [72, 83, 106, 117, 129, 153, 165, 190, 202, 215, 241, 254, 267, 281, 307, 321, 335, 348, 362, 376, 390, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321, 1333, 1345, 1358, 1370, 1382, 1394, 1406] }, "Foot": { "Cheap": [7, 11, 16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765], "Standard": [16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780], "Premium": [20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788], "Prototype": [32, 39, 45, 52, 59, 66, 73, 88, 95, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826], "Artifact": [39, 45, 52, 66, 73, 81, 88, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 333, 341, 348, 356, 364, 371, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856], "Legendary": [45, 52, 66, 73, 81, 95, 103, 119, 127, 134, 151, 159, 167, 175, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856, 864, 871, 879] }, "Hands": { "Cheap": [7, 11, 16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765], "Standard": [16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780], "Premium": [20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788], "Prototype": [32, 39, 45, 52, 59, 66, 73, 88, 95, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826], "Artifact": [39, 45, 52, 66, 73, 81, 88, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 333, 341, 348, 356, 364, 371, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856], "Legendary": [45, 52, 66, 73, 81, 95, 103, 119, 127, 134, 151, 159, 167, 175, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856, 864, 871, 879] }, "Legs": { "Cheap": [12, 18, 25, 32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224], "Standard": [25, 32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248], "Premium": [32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261], "Prototype": [52, 62, 72, 83, 94, 106, 117, 141, 153, 165, 177, 190, 202, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321], "Artifact": [62, 72, 83, 106, 117, 129, 141, 165, 177, 190, 202, 228, 241, 254, 267, 281, 294, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 533, 545, 558, 570, 582, 594, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321, 1333, 1345, 1358, 1370], "Legendary": [72, 83, 106, 117, 129, 153, 165, 190, 202, 215, 241, 254, 267, 281, 307, 321, 335, 348, 362, 376, 390, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321, 1333, 1345, 1358, 1370, 1382, 1394, 1406] } }, "Light": { "Wrist": { "Cheap": [2, 3, 4, 6, 9, 12, 15, 18, 21, 25, 28, 32, 36, 40, 44, 48, 52, 56, 60, 65, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405], "Standard": [4, 6, 9, 12, 15, 18, 21, 25, 28, 32, 36, 40, 44, 48, 52, 56, 60, 65, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413], "Premium": [6, 9, 12, 15, 18, 21, 25, 28, 32, 36, 40, 44, 48, 52, 56, 60, 65, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417], "Prototype": [12, 15, 18, 21, 25, 28, 32, 40, 44, 48, 52, 56, 60, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417, 421, 425, 429, 433, 437], "Artifact": [15, 18, 21, 28, 32, 36, 40, 48, 52, 56, 60, 69, 74, 78, 83, 87, 92, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 176, 180, 184, 189, 193, 197, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417, 421, 425, 429, 433, 437, 441, 445, 449, 453], "Legendary": [18, 21, 28, 32, 36, 44, 48, 56, 60, 65, 74, 78, 83, 87, 97, 101, 106, 111, 116, 121, 125, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417, 421, 425, 429, 433, 437, 441, 445, 449, 453, 457, 461, 465] }, "Belt": { "Cheap": [2, 3, 4, 6, 9, 12, 15, 18, 21, 25, 28, 32, 36, 40, 44, 48, 52, 56, 60, 65, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405], "Standard": [4, 6, 9, 12, 15, 18, 21, 25, 28, 32, 36, 40, 44, 48, 52, 56, 60, 65, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413], "Premium": [6, 9, 12, 15, 18, 21, 25, 28, 32, 36, 40, 44, 48, 52, 56, 60, 65, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417], "Prototype": [12, 15, 18, 21, 25, 28, 32, 40, 44, 48, 52, 56, 60, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417, 421, 425, 429, 433, 437], "Artifact": [15, 18, 21, 28, 32, 36, 40, 48, 52, 56, 60, 69, 74, 78, 83, 87, 92, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 176, 180, 184, 189, 193, 197, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417, 421, 425, 429, 433, 437, 441, 445, 449, 453], "Legendary": [18, 21, 28, 32, 36, 44, 48, 56, 60, 65, 74, 78, 83, 87, 97, 101, 106, 111, 116, 121, 125, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417, 421, 425, 429, 433, 437, 441, 445, 449, 453, 457, 461, 465] }, "Chest": { "Cheap": [3, 5, 7, 10, 14, 19, 24, 29, 34, 40, 46, 51, 58, 64, 70, 77, 83, 90, 97, 104, 111, 118, 125, 132, 140, 147, 154, 162, 170, 177, 185, 193, 201, 209, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 302, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648], "Standard": [7, 10, 14, 19, 24, 29, 34, 40, 46, 51, 58, 64, 70, 77, 83, 90, 97, 104, 111, 118, 125, 132, 140, 147, 154, 162, 170, 177, 185, 193, 201, 209, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 302, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661], "Premium": [10, 14, 19, 24, 29, 34, 40, 46, 51, 58, 64, 70, 77, 83, 90, 97, 104, 111, 118, 125, 132, 140, 147, 154, 162, 170, 177, 185, 193, 201, 209, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 302, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661, 667], "Prototype": [19, 24, 29, 34, 40, 46, 51, 64, 70, 77, 83, 90, 97, 111, 118, 125, 132, 140, 147, 154, 162, 170, 177, 185, 193, 201, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 302, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661, 667, 674, 680, 687, 693, 699], "Artifact": [24, 29, 34, 46, 51, 58, 64, 77, 83, 90, 97, 111, 118, 125, 132, 140, 147, 162, 170, 177, 185, 193, 201, 209, 217, 225, 233, 241, 249, 257, 266, 282, 289, 295, 302, 308, 314, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661, 667, 674, 680, 687, 693, 699, 706, 712, 719, 725], "Legendary": [29, 34, 46, 51, 58, 70, 77, 90, 97, 104, 118, 125, 132, 140, 154, 162, 170, 177, 185, 193, 201, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661, 667, 674, 680, 687, 693, 699, 706, 712, 719, 725, 732, 738, 744] }, "Helm": { "Cheap": [3, 5, 7, 10, 14, 19, 24, 29, 34, 40, 46, 51, 58, 64, 70, 77, 83, 90, 97, 104, 111, 118, 125, 132, 140, 147, 154, 162, 170, 177, 185, 193, 201, 209, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 302, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648], "Standard": [7, 10, 14, 19, 24, 29, 34, 40, 46, 51, 58, 64, 70, 77, 83, 90, 97, 104, 111, 118, 125, 132, 140, 147, 154, 162, 170, 177, 185, 193, 201, 209, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 302, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661], "Premium": [10, 14, 19, 24, 29, 34, 40, 46, 51, 58, 64, 70, 77, 83, 90, 97, 104, 111, 118, 125, 132, 140, 147, 154, 162, 170, 177, 185, 193, 201, 209, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 302, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661, 667], "Prototype": [19, 24, 29, 34, 40, 46, 51, 64, 70, 77, 83, 90, 97, 111, 118, 125, 132, 140, 147, 154, 162, 170, 177, 185, 193, 201, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 302, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661, 667, 674, 680, 687, 693, 699], "Artifact": [24, 29, 34, 46, 51, 58, 64, 77, 83, 90, 97, 111, 118, 125, 132, 140, 147, 162, 170, 177, 185, 193, 201, 209, 217, 225, 233, 241, 249, 257, 266, 282, 289, 295, 302, 308, 314, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661, 667, 674, 680, 687, 693, 699, 706, 712, 719, 725], "Legendary": [29, 34, 46, 51, 58, 70, 77, 90, 97, 104, 118, 125, 132, 140, 154, 162, 170, 177, 185, 193, 201, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661, 667, 674, 680, 687, 693, 699, 706, 712, 719, 725, 732, 738, 744] }, "Foot": { "Cheap": [2, 3, 4, 6, 9, 12, 15, 18, 21, 25, 28, 32, 36, 40, 44, 48, 52, 56, 60, 65, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405], "Standard": [4, 6, 9, 12, 15, 18, 21, 25, 28, 32, 36, 40, 44, 48, 52, 56, 60, 65, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413], "Premium": [6, 9, 12, 15, 18, 21, 25, 28, 32, 36, 40, 44, 48, 52, 56, 60, 65, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417], "Prototype": [12, 15, 18, 21, 25, 28, 32, 40, 44, 48, 52, 56, 60, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417, 421, 425, 429, 433, 437], "Artifact": [15, 18, 21, 28, 32, 36, 40, 48, 52, 56, 60, 69, 74, 78, 83, 87, 92, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 176, 180, 184, 189, 193, 197, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417, 421, 425, 429, 433, 437, 441, 445, 449, 453], "Legendary": [18, 21, 28, 32, 36, 44, 48, 56, 60, 65, 74, 78, 83, 87, 97, 101, 106, 111, 116, 121, 125, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417, 421, 425, 429, 433, 437, 441, 445, 449, 453, 457, 461, 465] }, "Hands": { "Cheap": [2, 3, 4, 6, 9, 12, 15, 18, 21, 25, 28, 32, 36, 40, 44, 48, 52, 56, 60, 65, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405], "Standard": [4, 6, 9, 12, 15, 18, 21, 25, 28, 32, 36, 40, 44, 48, 52, 56, 60, 65, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413], "Premium": [6, 9, 12, 15, 18, 21, 25, 28, 32, 36, 40, 44, 48, 52, 56, 60, 65, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417], "Prototype": [12, 15, 18, 21, 25, 28, 32, 40, 44, 48, 52, 56, 60, 69, 74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 121, 125, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 189, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417, 421, 425, 429, 433, 437], "Artifact": [15, 18, 21, 28, 32, 36, 40, 48, 52, 56, 60, 69, 74, 78, 83, 87, 92, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 151, 156, 161, 166, 176, 180, 184, 189, 193, 197, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417, 421, 425, 429, 433, 437, 441, 445, 449, 453], "Legendary": [18, 21, 28, 32, 36, 44, 48, 56, 60, 65, 74, 78, 83, 87, 97, 101, 106, 111, 116, 121, 125, 135, 140, 145, 151, 156, 161, 166, 171, 176, 180, 184, 193, 197, 201, 205, 209, 213, 217, 221, 225, 229, 233, 237, 241, 249, 253, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325, 329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 373, 377, 381, 385, 389, 393, 397, 401, 405, 409, 413, 417, 421, 425, 429, 433, 437, 441, 445, 449, 453, 457, 461, 465] }, "Legs": { "Cheap": [3, 5, 7, 10, 14, 19, 24, 29, 34, 40, 46, 51, 58, 64, 70, 77, 83, 90, 97, 104, 111, 118, 125, 132, 140, 147, 154, 162, 170, 177, 185, 193, 201, 209, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 302, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648], "Standard": [7, 10, 14, 19, 24, 29, 34, 40, 46, 51, 58, 64, 70, 77, 83, 90, 97, 104, 111, 118, 125, 132, 140, 147, 154, 162, 170, 177, 185, 193, 201, 209, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 302, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661], "Premium": [10, 14, 19, 24, 29, 34, 40, 46, 51, 58, 64, 70, 77, 83, 90, 97, 104, 111, 118, 125, 132, 140, 147, 154, 162, 170, 177, 185, 193, 201, 209, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 302, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661, 667], "Prototype": [19, 24, 29, 34, 40, 46, 51, 64, 70, 77, 83, 90, 97, 111, 118, 125, 132, 140, 147, 154, 162, 170, 177, 185, 193, 201, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 302, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661, 667, 674, 680, 687, 693, 699], "Artifact": [24, 29, 34, 46, 51, 58, 64, 77, 83, 90, 97, 111, 118, 125, 132, 140, 147, 162, 170, 177, 185, 193, 201, 209, 217, 225, 233, 241, 249, 257, 266, 282, 289, 295, 302, 308, 314, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 391, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661, 667, 674, 680, 687, 693, 699, 706, 712, 719, 725], "Legendary": [29, 34, 46, 51, 58, 70, 77, 90, 97, 104, 118, 125, 132, 140, 154, 162, 170, 177, 185, 193, 201, 217, 225, 233, 241, 249, 257, 266, 274, 282, 289, 295, 308, 314, 321, 327, 334, 340, 347, 353, 359, 366, 372, 379, 385, 398, 404, 411, 417, 424, 430, 436, 443, 449, 456, 462, 468, 475, 481, 488, 494, 501, 507, 513, 520, 526, 533, 539, 545, 552, 558, 565, 571, 578, 584, 590, 597, 603, 610, 616, 622, 629, 635, 642, 648, 655, 661, 667, 674, 680, 687, 693, 699, 706, 712, 719, 725, 732, 738, 744] } }, "Heavy": { "Wrist": { "Cheap": [7, 11, 16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765], "Standard": [16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780], "Premium": [20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788], "Prototype": [32, 39, 45, 52, 59, 66, 73, 88, 95, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826], "Artifact": [39, 45, 52, 66, 73, 81, 88, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 333, 341, 348, 356, 364, 371, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856], "Legendary": [45, 52, 66, 73, 81, 95, 103, 119, 127, 134, 151, 159, 167, 175, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856, 864, 871, 879] }, "Belt": { "Cheap": [7, 11, 16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765], "Standard": [16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780], "Premium": [20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788], "Prototype": [32, 39, 45, 52, 59, 66, 73, 88, 95, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826], "Artifact": [39, 45, 52, 66, 73, 81, 88, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 333, 341, 348, 356, 364, 371, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856], "Legendary": [45, 52, 66, 73, 81, 95, 103, 119, 127, 134, 151, 159, 167, 175, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856, 864, 871, 879] }, "Chest": { "Cheap": [12, 18, 25, 32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224], "Standard": [25, 32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248], "Premium": [32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261], "Prototype": [52, 62, 72, 83, 94, 106, 117, 141, 153, 165, 177, 190, 202, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321], "Artifact": [62, 72, 83, 106, 117, 129, 141, 165, 177, 190, 202, 228, 241, 254, 267, 281, 294, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 533, 545, 558, 570, 582, 594, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321, 1333, 1345, 1358, 1370], "Legendary": [72, 83, 106, 117, 129, 153, 165, 190, 202, 215, 241, 254, 267, 281, 307, 321, 335, 348, 362, 376, 390, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321, 1333, 1345, 1358, 1370, 1382, 1394, 1406] }, "Helm": { "Cheap": [12, 18, 25, 32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224], "Standard": [25, 32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248], "Premium": [32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261], "Prototype": [52, 62, 72, 83, 94, 106, 117, 141, 153, 165, 177, 190, 202, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321], "Artifact": [62, 72, 83, 106, 117, 129, 141, 165, 177, 190, 202, 228, 241, 254, 267, 281, 294, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 533, 545, 558, 570, 582, 594, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321, 1333, 1345, 1358, 1370], "Legendary": [72, 83, 106, 117, 129, 153, 165, 190, 202, 215, 241, 254, 267, 281, 307, 321, 335, 348, 362, 376, 390, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321, 1333, 1345, 1358, 1370, 1382, 1394, 1406] }, "Foot": { "Cheap": [7, 11, 16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765], "Standard": [16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780], "Premium": [20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788], "Prototype": [32, 39, 45, 52, 59, 66, 73, 88, 95, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826], "Artifact": [39, 45, 52, 66, 73, 81, 88, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 333, 341, 348, 356, 364, 371, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856], "Legendary": [45, 52, 66, 73, 81, 95, 103, 119, 127, 134, 151, 159, 167, 175, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856, 864, 871, 879] }, "Hands": { "Cheap": [7, 11, 16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765], "Standard": [16, 20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780], "Premium": [20, 26, 32, 39, 45, 52, 59, 66, 73, 81, 88, 95, 103, 111, 119, 127, 134, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788], "Prototype": [32, 39, 45, 52, 59, 66, 73, 88, 95, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 356, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826], "Artifact": [39, 45, 52, 66, 73, 81, 88, 103, 111, 119, 127, 143, 151, 159, 167, 175, 184, 201, 209, 218, 226, 235, 244, 253, 261, 270, 279, 288, 297, 306, 315, 333, 341, 348, 356, 364, 371, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 462, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856], "Legendary": [45, 52, 66, 73, 81, 95, 103, 119, 127, 134, 151, 159, 167, 175, 192, 201, 209, 218, 226, 235, 244, 261, 270, 279, 288, 297, 306, 315, 324, 333, 341, 348, 364, 371, 379, 386, 394, 402, 409, 417, 424, 432, 439, 447, 455, 470, 477, 485, 492, 500, 508, 515, 523, 530, 538, 545, 553, 561, 568, 576, 583, 591, 598, 606, 614, 621, 629, 636, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712, 720, 727, 735, 742, 750, 758, 765, 773, 780, 788, 795, 803, 811, 818, 826, 833, 841, 848, 856, 864, 871, 879] }, "Legs": { "Cheap": [12, 18, 25, 32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224], "Standard": [25, 32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248], "Premium": [32, 42, 52, 62, 72, 83, 94, 106, 117, 129, 141, 153, 165, 177, 190, 202, 215, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261], "Prototype": [52, 62, 72, 83, 94, 106, 117, 141, 153, 165, 177, 190, 202, 228, 241, 254, 267, 281, 294, 307, 321, 335, 348, 362, 376, 390, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 570, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321], "Artifact": [62, 72, 83, 106, 117, 129, 141, 165, 177, 190, 202, 228, 241, 254, 267, 281, 294, 321, 335, 348, 362, 376, 390, 404, 418, 432, 447, 461, 475, 490, 504, 533, 545, 558, 570, 582, 594, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 739, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321, 1333, 1345, 1358, 1370], "Legendary": [72, 83, 106, 117, 129, 153, 165, 190, 202, 215, 241, 254, 267, 281, 307, 321, 335, 348, 362, 376, 390, 418, 432, 447, 461, 475, 490, 504, 519, 533, 545, 558, 582, 594, 606, 618, 630, 642, 655, 667, 679, 691, 703, 715, 727, 752, 764, 776, 788, 800, 812, 824, 836, 848, 861, 873, 885, 897, 909, 921, 933, 945, 958, 970, 982, 994, 1006, 1018, 1030, 1042, 1055, 1067, 1079, 1091, 1103, 1115, 1127, 1139, 1152, 1164, 1176, 1188, 1200, 1212, 1224, 1236, 1248, 1261, 1273, 1285, 1297, 1309, 1321, 1333, 1345, 1358, 1370, 1382, 1394, 1406] } } };

    armor.GetArmor = function (levelOrItem, quality, slot, armorType) {

        var level = levelOrItem;
        if (typeof (levelOrItem) == 'object') {
            if (Tr.Swtor.isMod(levelOrItem)) return null;

            level = levelOrItem.ItemLevel;
            quality = levelOrItem.Quality;
            slot = levelOrItem.Slot;
            armorType = levelOrItem.ArmorType;

            if (quality == 'Custom') {
                level = 1;
                quality = 'Standard';
            }

            // find armoring mod
            if (levelOrItem.Mods !== undefined && levelOrItem.Mods !== null) {
                for (var i = 0; i < levelOrItem.Mods.length; i++) {
                    if (levelOrItem.Mods[i].Id != null && levelOrItem.Mods[i].Id != 0 && levelOrItem.Mods[i].Slot == 'Armoring') {
                        level = levelOrItem.Mods[i].ItemLevel;
                        quality = levelOrItem.Mods[i].Quality;
                        break;
                    }
                }
            }
        }

        if (quality == 'Legacy')
            quality = 'Legendary';

        if (armorType === undefined || armorType === null) {
            return null;
        }
        var s = lookup[armorType][slot];
        if (s === undefined) {
            return null;
        }
        var q = s[quality];
        if (q === undefined) {
            return null;
        }
        return q[level - 1];
    }

    return calc;
}(TeamRobot.Swtor.Calc || {}, jQuery));

/*
* Copyright 2010-2013 Team Robot, Inc.
* 
* Author: Adam Tegen
*/

//
// TeamRobot.Swtor.Calc.Rating
//
TeamRobot.Swtor.Calc = (function (calc, $) {
    var rating = calc.Rating = calc.Rating || {};

    var lookup = { "Cheap": [2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 66, 68, 70, 72, 74, 76, 78, 80, 82, 84, 86, 88, 90, 92, 94, 96, 98, 100, 102, 104, 106, 108, 110, 112, 114, 116, 118, 120, 122, 124, 126, 128, 130, 132, 134, 136, 138, 140, 142, 144, 146, 148, 150, 152, 154, 156, 158, 160, 162, 164, 166, 168, 170, 172, 174, 176, 178, 180, 182, 184, 186, 188, 190, 192, 194, 196, 198, 200], "Standard": [6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 66, 68, 70, 72, 74, 76, 78, 80, 82, 84, 86, 88, 90, 92, 94, 96, 98, 100, 102, 104, 106, 108, 110, 112, 114, 116, 118, 120, 122, 124, 126, 128, 130, 132, 134, 136, 138, 140, 142, 144, 146, 148, 150, 152, 154, 156, 158, 160, 162, 164, 166, 168, 170, 172, 174, 176, 178, 180, 182, 184, 186, 188, 190, 192, 194, 196, 198, 200, 202, 204], "Premium": [8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 66, 68, 70, 72, 74, 76, 78, 80, 82, 84, 86, 88, 90, 92, 94, 96, 98, 100, 102, 104, 106, 108, 110, 112, 114, 116, 118, 120, 122, 124, 126, 128, 130, 132, 134, 136, 138, 140, 142, 144, 146, 148, 150, 152, 154, 156, 158, 160, 162, 164, 166, 168, 170, 172, 174, 176, 178, 180, 182, 184, 186, 188, 190, 192, 194, 196, 198, 200, 202, 204, 206], "Prototype": [12, 14, 16, 18, 20, 22, 24, 28, 30, 32, 34, 36, 38, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 66, 70, 72, 74, 76, 78, 80, 82, 84, 86, 88, 90, 92, 94, 96, 98, 100, 102, 104, 106, 108, 110, 112, 114, 116, 118, 120, 122, 124, 126, 128, 130, 132, 134, 136, 138, 140, 142, 144, 146, 148, 150, 152, 154, 156, 158, 160, 162, 164, 166, 168, 170, 172, 174, 176, 178, 180, 182, 184, 186, 188, 190, 192, 194, 196, 198, 200, 202, 204, 206, 208, 210, 212, 214, 216], "Artifact": [14, 16, 18, 22, 24, 26, 28, 32, 34, 36, 38, 42, 44, 46, 48, 50, 52, 56, 58, 60, 62, 64, 66, 68, 70, 72, 74, 76, 78, 80, 82, 86, 88, 90, 92, 94, 96, 100, 102, 104, 106, 108, 110, 112, 114, 116, 118, 120, 122, 124, 126, 128, 130, 132, 134, 136, 138, 140, 142, 144, 146, 148, 150, 152, 154, 156, 158, 160, 162, 164, 166, 168, 170, 172, 174, 176, 178, 180, 182, 184, 186, 188, 190, 192, 194, 196, 198, 200, 202, 204, 206, 208, 210, 212, 214, 216, 218, 220, 222, 224], "Legendary": [16, 18, 22, 24, 26, 30, 32, 36, 38, 40, 44, 46, 48, 50, 54, 56, 58, 60, 62, 64, 66, 70, 72, 74, 76, 78, 80, 82, 84, 86, 88, 90, 94, 96, 98, 100, 102, 104, 106, 108, 110, 112, 114, 116, 118, 122, 124, 126, 128, 130, 132, 134, 136, 138, 140, 142, 144, 146, 148, 150, 152, 154, 156, 158, 160, 162, 164, 166, 168, 170, 172, 174, 176, 178, 180, 182, 184, 186, 188, 190, 192, 194, 196, 198, 200, 202, 204, 206, 208, 210, 212, 214, 216, 218, 220, 222, 224, 226, 228, 230] };

    rating.GetRating = function (levelOrItem, quality) {

        var level = levelOrItem;
        if (typeof (levelOrItem) == 'object') {
            if (Tr.Swtor.isMod(levelOrItem)) {
                if (levelOrItem.Slot !== 'Armoring' && levelOrItem.Slot !== 'Hilt' && levelOrItem.Slot !== 'Barrel') {
                    return null;
                }
                level = levelOrItem.ItemLevel;
                quality = levelOrItem.Quality;
            }
            else {

                isArmor = levelOrItem.ArmorType === undefined || levelOrItem.ArmorType === null;
                isWeapon = levelOrItem.WeaponType === undefined || levelOrItem.WeaponType === null;
                if (!isArmor && !isWeapon) {
                    return null;
                }

                level = levelOrItem.ItemLevel;
                quality = levelOrItem.Quality;

                if (quality === 'Custom') {
                    level = 1;
                    quality = 'Standard';
                }

                // find mod
                if (levelOrItem.Mods !== undefined && levelOrItem.Mods !== null) {
                    for (var i = 0; i < levelOrItem.Mods.length; i++) {
                        if (levelOrItem.Mods[i].Id != null && levelOrItem.Mods[i].Id != 0 && (levelOrItem.Mods[i].Slot === 'Armoring' || levelOrItem.Mods[i].Slot === 'Hilt' || levelOrItem.Mods[i].Slot === 'Barrel')) {
                            level = levelOrItem.Mods[i].ItemLevel;
                            quality = levelOrItem.Mods[i].Quality;
                            break;
                        }
                    }
                }
            }
        }

        if (quality == 'Legacy')
            quality = 'Legendary';

        var q = lookup[quality];
        if (q === undefined) {
            return null;
        }
        return q[level - 1];
    }
    return calc;
}(TeamRobot.Swtor.Calc || {}, jQuery));

/*
* Copyright 2010-2013 Team Robot, Inc.
* 
* Author: Adam Tegen
*/

//
// TeamRobot.Swtor.Calc.Shield
//
TeamRobot.Swtor.Calc = (function (calc, $) {
    var shield = calc.Shield = calc.Shield || {};

    var lookup = {
        techPower: { Generator: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] }, TechShield: { Cheap: [28, 30, 32, 34, 36, 39, 41, 44, 47, 49, 52, 55, 57, 60, 63, 65, 68, 71, 73, 76, 79, 81, 84, 87, 90, 93, 96, 99, 102, 106, 110, 113, 118, 122, 127, 132, 137, 143, 149, 155, 162, 169, 177, 184, 193, 201, 210, 219, 229, 238, 248, 258, 269, 279, 290, 301, 311, 320, 331, 343, 354, 366, 378, 390, 402, 415, 428, 441, 454, 467, 480, 494, 507, 521, 536, 550, 565, 579, 594, 609, 624, 639, 655, 671, 686, 702, 718, 734, 751, 767, 784, 800, 817, 834, 850, 867, 884, 901, 918, 935], Standard: [32, 34, 36, 39, 41, 44, 47, 49, 52, 55, 57, 60, 63, 65, 68, 71, 73, 76, 79, 81, 84, 87, 90, 93, 96, 99, 102, 106, 110, 113, 118, 122, 127, 132, 137, 143, 149, 155, 162, 169, 177, 184, 193, 201, 210, 219, 229, 238, 248, 258, 269, 279, 290, 301, 311, 320, 331, 343, 354, 366, 378, 390, 402, 415, 428, 441, 454, 467, 480, 494, 507, 521, 536, 550, 565, 579, 594, 609, 624, 639, 655, 671, 686, 702, 718, 734, 751, 767, 784, 800, 817, 834, 850, 867, 884, 901, 918, 935, 951, 968], Premium: [34, 36, 39, 41, 44, 47, 49, 52, 55, 57, 60, 63, 65, 68, 71, 73, 76, 79, 81, 84, 87, 90, 93, 96, 99, 102, 106, 110, 113, 118, 122, 127, 132, 137, 143, 149, 155, 162, 169, 177, 184, 193, 201, 210, 219, 229, 238, 248, 258, 269, 279, 290, 301, 311, 320, 331, 343, 354, 366, 378, 390, 402, 415, 428, 441, 454, 467, 480, 494, 507, 521, 536, 550, 565, 579, 594, 609, 624, 639, 655, 671, 686, 702, 718, 734, 751, 767, 784, 800, 817, 834, 850, 867, 884, 901, 918, 935, 951, 968, 984], Prototype: [39, 41, 44, 47, 49, 52, 55, 60, 63, 65, 68, 71, 73, 79, 81, 84, 87, 90, 93, 96, 99, 102, 106, 110, 113, 118, 127, 132, 137, 143, 149, 155, 162, 169, 177, 184, 193, 201, 210, 219, 229, 238, 248, 258, 269, 279, 290, 301, 311, 320, 331, 343, 354, 366, 378, 390, 402, 415, 428, 441, 454, 467, 480, 494, 507, 521, 536, 550, 565, 579, 594, 609, 624, 639, 655, 671, 686, 702, 718, 734, 751, 767, 784, 800, 817, 834, 850, 867, 884, 901, 918, 935, 951, 968, 984, 1e3, 1017, 1033, 1049, 1066], Artifact: [41, 44, 47, 52, 55, 57, 60, 65, 68, 71, 73, 79, 81, 84, 87, 90, 93, 99, 102, 106, 110, 113, 118, 122, 127, 132, 137, 143, 149, 155, 162, 177, 184, 193, 201, 210, 219, 238, 248, 258, 269, 279, 290, 301, 311, 320, 331, 343, 354, 366, 378, 390, 402, 415, 428, 441, 454, 467, 480, 494, 507, 521, 536, 550, 565, 579, 594, 609, 624, 639, 655, 671, 686, 702, 718, 734, 751, 767, 784, 800, 817, 834, 850, 867, 884, 901, 918, 935, 951, 968, 984, 1e3, 1017, 1033, 1049, 1066, 1082, 1098, 1115, 1131], Legendary: [44, 47, 52, 55, 57, 63, 65, 71, 73, 76, 81, 84, 87, 90, 96, 99, 102, 106, 110, 113, 118, 127, 132, 137, 143, 149, 155, 162, 169, 177, 184, 193, 210, 219, 229, 238, 248, 258, 269, 279, 290, 301, 311, 320, 331, 354, 366, 378, 390, 402, 415, 428, 441, 454, 467, 480, 494, 507, 521, 536, 550, 565, 579, 594, 609, 624, 639, 655, 671, 686, 702, 718, 734, 751, 767, 784, 800, 817, 834, 850, 867, 884, 901, 918, 935, 951, 968, 984, 1e3, 1017, 1033, 1049, 1066, 1082, 1098, 1115, 1131, 1147, 1164, 1180] } },
        forcePower: { Focus: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] }, ForceShield: { Cheap: [28, 30, 32, 34, 36, 39, 41, 44, 47, 49, 52, 55, 57, 60, 63, 65, 68, 71, 73, 76, 79, 81, 84, 87, 90, 93, 96, 99, 102, 106, 110, 113, 118, 122, 127, 132, 137, 143, 149, 155, 162, 169, 177, 184, 193, 201, 210, 219, 229, 238, 248, 258, 269, 279, 290, 301, 311, 320, 331, 343, 354, 366, 378, 390, 402, 415, 428, 441, 454, 467, 480, 494, 507, 521, 536, 550, 565, 579, 594, 609, 624, 639, 655, 671, 686, 702, 718, 734, 751, 767, 784, 800, 817, 834, 850, 867, 884, 901, 918, 935], Standard: [32, 34, 36, 39, 41, 44, 47, 49, 52, 55, 57, 60, 63, 65, 68, 71, 73, 76, 79, 81, 84, 87, 90, 93, 96, 99, 102, 106, 110, 113, 118, 122, 127, 132, 137, 143, 149, 155, 162, 169, 177, 184, 193, 201, 210, 219, 229, 238, 248, 258, 269, 279, 290, 301, 311, 320, 331, 343, 354, 366, 378, 390, 402, 415, 428, 441, 454, 467, 480, 494, 507, 521, 536, 550, 565, 579, 594, 609, 624, 639, 655, 671, 686, 702, 718, 734, 751, 767, 784, 800, 817, 834, 850, 867, 884, 901, 918, 935, 951, 968], Premium: [34, 36, 39, 41, 44, 47, 49, 52, 55, 57, 60, 63, 65, 68, 71, 73, 76, 79, 81, 84, 87, 90, 93, 96, 99, 102, 106, 110, 113, 118, 122, 127, 132, 137, 143, 149, 155, 162, 169, 177, 184, 193, 201, 210, 219, 229, 238, 248, 258, 269, 279, 290, 301, 311, 320, 331, 343, 354, 366, 378, 390, 402, 415, 428, 441, 454, 467, 480, 494, 507, 521, 536, 550, 565, 579, 594, 609, 624, 639, 655, 671, 686, 702, 718, 734, 751, 767, 784, 800, 817, 834, 850, 867, 884, 901, 918, 935, 951, 968, 984], Prototype: [39, 41, 44, 47, 49, 52, 55, 60, 63, 65, 68, 71, 73, 79, 81, 84, 87, 90, 93, 96, 99, 102, 106, 110, 113, 118, 127, 132, 137, 143, 149, 155, 162, 169, 177, 184, 193, 201, 210, 219, 229, 238, 248, 258, 269, 279, 290, 301, 311, 320, 331, 343, 354, 366, 378, 390, 402, 415, 428, 441, 454, 467, 480, 494, 507, 521, 536, 550, 565, 579, 594, 609, 624, 639, 655, 671, 686, 702, 718, 734, 751, 767, 784, 800, 817, 834, 850, 867, 884, 901, 918, 935, 951, 968, 984, 1e3, 1017, 1033, 1049, 1066], Artifact: [41, 44, 47, 52, 55, 57, 60, 65, 68, 71, 73, 79, 81, 84, 87, 90, 93, 99, 102, 106, 110, 113, 118, 122, 127, 132, 137, 143, 149, 155, 162, 177, 184, 193, 201, 210, 219, 238, 248, 258, 269, 279, 290, 301, 311, 320, 331, 343, 354, 366, 378, 390, 402, 415, 428, 441, 454, 467, 480, 494, 507, 521, 536, 550, 565, 579, 594, 609, 624, 639, 655, 671, 686, 702, 718, 734, 751, 767, 784, 800, 817, 834, 850, 867, 884, 901, 918, 935, 951, 968, 984, 1e3, 1017, 1033, 1049, 1066, 1082, 1098, 1115, 1131], Legendary: [44, 47, 52, 55, 57, 63, 65, 71, 73, 76, 81, 84, 87, 90, 96, 99, 102, 106, 110, 113, 118, 127, 132, 137, 143, 149, 155, 162, 169, 177, 184, 193, 210, 219, 229, 238, 248, 258, 269, 279, 290, 301, 311, 320, 331, 354, 366, 378, 390, 402, 415, 428, 441, 454, 467, 480, 494, 507, 521, 536, 550, 565, 579, 594, 609, 624, 639, 655, 671, 686, 702, 718, 734, 751, 767, 784, 800, 817, 834, 850, 867, 884, 901, 918, 935, 951, 968, 984, 1e3, 1017, 1033, 1049, 1066, 1082, 1098, 1115, 1131, 1147, 1164, 1180] } }
    };

    shield.GetPower = function (levelOrItem, quality, shieldType) {

        var level = levelOrItem;
        if (typeof (levelOrItem) == 'object') {
            if (Tr.Swtor.isMod(levelOrItem)) return null;

            level = levelOrItem.ItemLevel;
            quality = levelOrItem.Quality;
            shieldType = levelOrItem.ShieldType;

            if (quality == 'Custom') {
                level = 1;
                quality = 'Standard';
            }

            // find armoring mod
            if (levelOrItem.Mods !== undefined && levelOrItem.Mods !== null) {
                for (var i = 0; i < levelOrItem.Mods.length; i++) {
                    if (levelOrItem.Mods[i].Slot == 'Armoring') {
                        if (levelOrItem.Mods[i].Id != null && levelOrItem.Mods[i].Id != 0) {
                            level = levelOrItem.Mods[i].ItemLevel;
                            quality = levelOrItem.Mods[i].Quality;
                        }
                        else {
                            // assume same rules as customizable items if no armoring
                            level = 1;
                            quality = 'Standard';
                        }
                        break;
                    }
                }
            }
        }

        if (quality == 'Legacy')
            quality = 'Legendary';

        if (shieldType === undefined || shieldType === null) {
            return null;
        }

        var powerLookup, powerType;
        if (shieldType == 'Focus' || shieldType == 'ForceShield') {
            powerLookup = lookup["forcePower"];
            powerType = 'Force';
        }
        else {
            powerLookup = lookup["techPower"];
            powerType = 'Tech';
        }

        var w = powerLookup[shieldType];
        if (w === undefined) {
            return null;
        }
        var q = w[quality];
        if (q === undefined) {
            return null;
        }

        return { power: q[level - 1], powerType: powerType };
    };

    return calc;
}(TeamRobot.Swtor.Calc || {}, jQuery));

/*
* Copyright 2010-2013 Team Robot, Inc.
* 
* Author: Adam Tegen
*/

//
// TeamRobot.Swtor.Calc.Weapon
//
TeamRobot.Swtor.Calc = (function (calc, $) {
    var weapon = calc.Weapon = calc.Weapon || {};

    var lookup = {
        min: { PoleSaber: { Cheap: [25, 27, 29, 31, 33, 35, 37, 39, 42, 44, 47, 49, 52, 54, 57, 59, 61, 64, 66, 69, 71, 73, 76, 78, 81, 83, 86, 89, 92, 95, 99, 102, 106, 110, 114, 119, 124, 129, 134, 140, 146, 152, 159, 166, 173, 181, 189, 197, 206, 214, 223, 233, 242, 251, 261, 270, 280, 288, 298, 308, 319, 330, 340, 351, 362, 373, 385, 397, 408, 420, 432, 444, 457, 469, 482, 495, 508, 521, 535, 548, 561, 575, 589, 603, 617, 632, 646, 661, 675, 690, 705, 720, 735, 750, 765, 781, 796, 811, 826, 841], Standard: [29, 31, 33, 35, 37, 39, 42, 44, 47, 49, 52, 54, 57, 59, 61, 64, 66, 69, 71, 73, 76, 78, 81, 83, 86, 89, 92, 95, 99, 102, 106, 110, 114, 119, 124, 129, 134, 140, 146, 152, 159, 166, 173, 181, 189, 197, 206, 214, 223, 233, 242, 251, 261, 270, 280, 288, 298, 308, 319, 330, 340, 351, 362, 373, 385, 397, 408, 420, 432, 444, 457, 469, 482, 495, 508, 521, 535, 548, 561, 575, 589, 603, 617, 632, 646, 661, 675, 690, 705, 720, 735, 750, 765, 781, 796, 811, 826, 841, 856, 871], Premium: [31, 33, 35, 37, 39, 42, 44, 47, 49, 52, 54, 57, 59, 61, 64, 66, 69, 71, 73, 76, 78, 81, 83, 86, 89, 92, 95, 99, 102, 106, 110, 114, 119, 124, 129, 134, 140, 146, 152, 159, 166, 173, 181, 189, 197, 206, 214, 223, 233, 242, 251, 261, 270, 280, 288, 298, 308, 319, 330, 340, 351, 362, 373, 385, 397, 408, 420, 432, 444, 457, 469, 482, 495, 508, 521, 535, 548, 561, 575, 589, 603, 617, 632, 646, 661, 675, 690, 705, 720, 735, 750, 765, 781, 796, 811, 826, 841, 856, 871, 885], Prototype: [35, 37, 39, 42, 44, 47, 49, 54, 57, 59, 61, 64, 66, 71, 73, 76, 78, 81, 83, 86, 89, 92, 95, 99, 102, 106, 114, 119, 124, 129, 134, 140, 146, 152, 159, 166, 173, 181, 189, 197, 206, 214, 223, 233, 242, 251, 261, 270, 280, 288, 298, 308, 319, 330, 340, 351, 362, 373, 385, 397, 408, 420, 432, 444, 457, 469, 482, 495, 508, 521, 535, 548, 561, 575, 589, 603, 617, 632, 646, 661, 675, 690, 705, 720, 735, 750, 765, 781, 796, 811, 826, 841, 856, 871, 885, 900, 915, 929, 944, 959], Artifact: [37, 39, 42, 47, 49, 52, 54, 59, 61, 64, 66, 71, 73, 76, 78, 81, 83, 89, 92, 95, 99, 102, 106, 110, 114, 119, 124, 129, 134, 140, 146, 159, 166, 173, 181, 189, 197, 214, 223, 233, 242, 251, 261, 270, 280, 288, 298, 308, 319, 330, 340, 351, 362, 373, 385, 397, 408, 420, 432, 444, 457, 469, 482, 495, 508, 521, 535, 548, 561, 575, 589, 603, 617, 632, 646, 661, 675, 690, 705, 720, 735, 750, 765, 781, 796, 811, 826, 841, 856, 871, 885, 900, 915, 929, 944, 959, 974, 988, 1003, 1018], Legendary: [39, 42, 47, 49, 52, 57, 59, 64, 66, 69, 73, 76, 78, 81, 86, 89, 92, 95, 99, 102, 106, 114, 119, 124, 129, 134, 140, 146, 152, 159, 166, 173, 189, 197, 206, 214, 223, 233, 242, 251, 261, 270, 280, 288, 298, 319, 330, 340, 351, 362, 373, 385, 397, 408, 420, 432, 444, 457, 469, 482, 495, 508, 521, 535, 548, 561, 575, 589, 603, 617, 632, 646, 661, 675, 690, 705, 720, 735, 750, 765, 781, 796, 811, 826, 841, 856, 871, 885, 900, 915, 929, 944, 959, 974, 988, 1003, 1018, 1032, 1047, 1062] }, BlasterRifle: { Cheap: [21, 22, 24, 26, 27, 29, 31, 33, 35, 37, 39, 41, 43, 45, 47, 49, 51, 53, 55, 57, 59, 61, 63, 65, 67, 69, 72, 74, 77, 79, 82, 85, 88, 92, 95, 99, 103, 107, 112, 117, 122, 127, 132, 138, 144, 151, 157, 164, 171, 179, 186, 194, 202, 209, 218, 225, 233, 240, 248, 257, 266, 275, 283, 293, 302, 311, 321, 330, 340, 350, 360, 370, 381, 391, 402, 412, 423, 434, 445, 457, 468, 479, 491, 503, 515, 527, 539, 551, 563, 575, 588, 600, 613, 625, 638, 650, 663, 675, 688, 701], Standard: [24, 26, 27, 29, 31, 33, 35, 37, 39, 41, 43, 45, 47, 49, 51, 53, 55, 57, 59, 61, 63, 65, 67, 69, 72, 74, 77, 79, 82, 85, 88, 92, 95, 99, 103, 107, 112, 117, 122, 127, 132, 138, 144, 151, 157, 164, 171, 179, 186, 194, 202, 209, 218, 225, 233, 240, 248, 257, 266, 275, 283, 293, 302, 311, 321, 330, 340, 350, 360, 370, 381, 391, 402, 412, 423, 434, 445, 457, 468, 479, 491, 503, 515, 527, 539, 551, 563, 575, 588, 600, 613, 625, 638, 650, 663, 675, 688, 701, 713, 726], Premium: [26, 27, 29, 31, 33, 35, 37, 39, 41, 43, 45, 47, 49, 51, 53, 55, 57, 59, 61, 63, 65, 67, 69, 72, 74, 77, 79, 82, 85, 88, 92, 95, 99, 103, 107, 112, 117, 122, 127, 132, 138, 144, 151, 157, 164, 171, 179, 186, 194, 202, 209, 218, 225, 233, 240, 248, 257, 266, 275, 283, 293, 302, 311, 321, 330, 340, 350, 360, 370, 381, 391, 402, 412, 423, 434, 445, 457, 468, 479, 491, 503, 515, 527, 539, 551, 563, 575, 588, 600, 613, 625, 638, 650, 663, 675, 688, 701, 713, 726, 738], Prototype: [29, 31, 33, 35, 37, 39, 41, 45, 47, 49, 51, 53, 55, 59, 61, 63, 65, 67, 69, 72, 74, 77, 79, 82, 85, 88, 95, 99, 103, 107, 112, 117, 122, 127, 132, 138, 144, 151, 157, 164, 171, 179, 186, 194, 202, 209, 218, 225, 233, 240, 248, 257, 266, 275, 283, 293, 302, 311, 321, 330, 340, 350, 360, 370, 381, 391, 402, 412, 423, 434, 445, 457, 468, 479, 491, 503, 515, 527, 539, 551, 563, 575, 588, 600, 613, 625, 638, 650, 663, 675, 688, 701, 713, 726, 738, 750, 762, 775, 787, 799], Artifact: [31, 33, 35, 39, 41, 43, 45, 49, 51, 53, 55, 59, 61, 63, 65, 67, 69, 74, 77, 79, 82, 85, 88, 92, 95, 99, 103, 107, 112, 117, 122, 132, 138, 144, 151, 157, 164, 179, 186, 194, 202, 209, 218, 225, 233, 240, 248, 257, 266, 275, 283, 293, 302, 311, 321, 330, 340, 350, 360, 370, 381, 391, 402, 412, 423, 434, 445, 457, 468, 479, 491, 503, 515, 527, 539, 551, 563, 575, 588, 600, 613, 625, 638, 650, 663, 675, 688, 701, 713, 726, 738, 750, 762, 775, 787, 799, 811, 824, 836, 848], Legendary: [33, 35, 39, 41, 43, 47, 49, 53, 55, 57, 61, 63, 65, 67, 72, 74, 77, 79, 82, 85, 88, 95, 99, 103, 107, 112, 117, 122, 127, 132, 138, 144, 157, 164, 171, 179, 186, 194, 202, 209, 218, 225, 233, 240, 248, 266, 275, 283, 293, 302, 311, 321, 330, 340, 350, 360, 370, 381, 391, 402, 412, 423, 434, 445, 457, 468, 479, 491, 503, 515, 527, 539, 551, 563, 575, 588, 600, 613, 625, 638, 650, 663, 675, 688, 701, 713, 726, 738, 750, 762, 775, 787, 799, 811, 824, 836, 848, 860, 873, 885] }, SniperRifle: { Cheap: [28, 30, 32, 35, 37, 39, 42, 44, 47, 50, 53, 55, 58, 61, 64, 66, 69, 72, 74, 77, 80, 82, 85, 88, 91, 94, 97, 100, 104, 107, 111, 115, 119, 124, 129, 133, 139, 145, 151, 157, 164, 171, 179, 187, 195, 204, 212, 222, 231, 241, 251, 262, 272, 283, 294, 304, 315, 324, 335, 347, 359, 371, 383, 395, 407, 420, 433, 446, 459, 472, 486, 500, 514, 528, 542, 557, 571, 586, 601, 616, 632, 647, 663, 679, 695, 711, 727, 743, 760, 777, 793, 810, 827, 844, 861, 878, 895, 912, 929, 946], Standard: [32, 35, 37, 39, 42, 44, 47, 50, 53, 55, 58, 61, 64, 66, 69, 72, 74, 77, 80, 82, 85, 88, 91, 94, 97, 100, 104, 107, 111, 115, 119, 124, 129, 133, 139, 145, 151, 157, 164, 171, 179, 187, 195, 204, 212, 222, 231, 241, 251, 262, 272, 283, 294, 304, 315, 324, 335, 347, 359, 371, 383, 395, 407, 420, 433, 446, 459, 472, 486, 500, 514, 528, 542, 557, 571, 586, 601, 616, 632, 647, 663, 679, 695, 711, 727, 743, 760, 777, 793, 810, 827, 844, 861, 878, 895, 912, 929, 946, 963, 979], Premium: [35, 37, 39, 42, 44, 47, 50, 53, 55, 58, 61, 64, 66, 69, 72, 74, 77, 80, 82, 85, 88, 91, 94, 97, 100, 104, 107, 111, 115, 119, 124, 129, 133, 139, 145, 151, 157, 164, 171, 179, 187, 195, 204, 212, 222, 231, 241, 251, 262, 272, 283, 294, 304, 315, 324, 335, 347, 359, 371, 383, 395, 407, 420, 433, 446, 459, 472, 486, 500, 514, 528, 542, 557, 571, 586, 601, 616, 632, 647, 663, 679, 695, 711, 727, 743, 760, 777, 793, 810, 827, 844, 861, 878, 895, 912, 929, 946, 963, 979, 996], Prototype: [39, 42, 44, 47, 50, 53, 55, 61, 64, 66, 69, 72, 74, 80, 82, 85, 88, 91, 94, 97, 100, 104, 107, 111, 115, 119, 129, 133, 139, 145, 151, 157, 164, 171, 179, 187, 195, 204, 212, 222, 231, 241, 251, 262, 272, 283, 294, 304, 315, 324, 335, 347, 359, 371, 383, 395, 407, 420, 433, 446, 459, 472, 486, 500, 514, 528, 542, 557, 571, 586, 601, 616, 632, 647, 663, 679, 695, 711, 727, 743, 760, 777, 793, 810, 827, 844, 861, 878, 895, 912, 929, 946, 963, 979, 996, 1013, 1029, 1046, 1062, 1079], Artifact: [42, 44, 47, 53, 55, 58, 61, 66, 69, 72, 74, 80, 82, 85, 88, 91, 94, 100, 104, 107, 111, 115, 119, 124, 129, 133, 139, 145, 151, 157, 164, 179, 187, 195, 204, 212, 222, 241, 251, 262, 272, 283, 294, 304, 315, 324, 335, 347, 359, 371, 383, 395, 407, 420, 433, 446, 459, 472, 486, 500, 514, 528, 542, 557, 571, 586, 601, 616, 632, 647, 663, 679, 695, 711, 727, 743, 760, 777, 793, 810, 827, 844, 861, 878, 895, 912, 929, 946, 963, 979, 996, 1013, 1029, 1046, 1062, 1079, 1095, 1112, 1128, 1145], Legendary: [44, 47, 53, 55, 58, 64, 66, 72, 74, 77, 82, 85, 88, 91, 97, 100, 104, 107, 111, 115, 119, 129, 133, 139, 145, 151, 157, 164, 171, 179, 187, 195, 212, 222, 231, 241, 251, 262, 272, 283, 294, 304, 315, 324, 335, 359, 371, 383, 395, 407, 420, 433, 446, 459, 472, 486, 500, 514, 528, 542, 557, 571, 586, 601, 616, 632, 647, 663, 679, 695, 711, 727, 743, 760, 777, 793, 810, 827, 844, 861, 878, 895, 912, 929, 946, 963, 979, 996, 1013, 1029, 1046, 1062, 1079, 1095, 1112, 1128, 1145, 1162, 1178, 1195] }, Pistol: { Cheap: [18, 20, 21, 22, 24, 25, 27, 29, 31, 32, 34, 36, 38, 39, 41, 43, 45, 47, 48, 50, 52, 53, 55, 57, 59, 61, 63, 65, 67, 69, 72, 74, 77, 80, 83, 86, 90, 94, 98, 102, 106, 111, 116, 121, 126, 132, 138, 144, 150, 156, 163, 170, 176, 183, 190, 197, 204, 210, 217, 225, 233, 240, 248, 256, 264, 272, 281, 289, 298, 306, 315, 324, 333, 342, 351, 361, 370, 380, 390, 400, 409, 419, 430, 440, 450, 461, 471, 482, 492, 503, 514, 525, 536, 547, 558, 569, 580, 591, 602, 613], Standard: [21, 22, 24, 25, 27, 29, 31, 32, 34, 36, 38, 39, 41, 43, 45, 47, 48, 50, 52, 53, 55, 57, 59, 61, 63, 65, 67, 69, 72, 74, 77, 80, 83, 86, 90, 94, 98, 102, 106, 111, 116, 121, 126, 132, 138, 144, 150, 156, 163, 170, 176, 183, 190, 197, 204, 210, 217, 225, 233, 240, 248, 256, 264, 272, 281, 289, 298, 306, 315, 324, 333, 342, 351, 361, 370, 380, 390, 400, 409, 419, 430, 440, 450, 461, 471, 482, 492, 503, 514, 525, 536, 547, 558, 569, 580, 591, 602, 613, 624, 635], Premium: [22, 24, 25, 27, 29, 31, 32, 34, 36, 38, 39, 41, 43, 45, 47, 48, 50, 52, 53, 55, 57, 59, 61, 63, 65, 67, 69, 72, 74, 77, 80, 83, 86, 90, 94, 98, 102, 106, 111, 116, 121, 126, 132, 138, 144, 150, 156, 163, 170, 176, 183, 190, 197, 204, 210, 217, 225, 233, 240, 248, 256, 264, 272, 281, 289, 298, 306, 315, 324, 333, 342, 351, 361, 370, 380, 390, 400, 409, 419, 430, 440, 450, 461, 471, 482, 492, 503, 514, 525, 536, 547, 558, 569, 580, 591, 602, 613, 624, 635, 646], Prototype: [25, 27, 29, 31, 32, 34, 36, 39, 41, 43, 45, 47, 48, 52, 53, 55, 57, 59, 61, 63, 65, 67, 69, 72, 74, 77, 83, 86, 90, 94, 98, 102, 106, 111, 116, 121, 126, 132, 138, 144, 150, 156, 163, 170, 176, 183, 190, 197, 204, 210, 217, 225, 233, 240, 248, 256, 264, 272, 281, 289, 298, 306, 315, 324, 333, 342, 351, 361, 370, 380, 390, 400, 409, 419, 430, 440, 450, 461, 471, 482, 492, 503, 514, 525, 536, 547, 558, 569, 580, 591, 602, 613, 624, 635, 646, 656, 667, 678, 688, 699], Artifact: [27, 29, 31, 34, 36, 38, 39, 43, 45, 47, 48, 52, 53, 55, 57, 59, 61, 65, 67, 69, 72, 74, 77, 80, 83, 86, 90, 94, 98, 102, 106, 116, 121, 126, 132, 138, 144, 156, 163, 170, 176, 183, 190, 197, 204, 210, 217, 225, 233, 240, 248, 256, 264, 272, 281, 289, 298, 306, 315, 324, 333, 342, 351, 361, 370, 380, 390, 400, 409, 419, 430, 440, 450, 461, 471, 482, 492, 503, 514, 525, 536, 547, 558, 569, 580, 591, 602, 613, 624, 635, 646, 656, 667, 678, 688, 699, 710, 721, 731, 742], Legendary: [29, 31, 34, 36, 38, 41, 43, 47, 48, 50, 53, 55, 57, 59, 63, 65, 67, 69, 72, 74, 77, 83, 86, 90, 94, 98, 102, 106, 111, 116, 121, 126, 138, 144, 150, 156, 163, 170, 176, 183, 190, 197, 204, 210, 217, 233, 240, 248, 256, 264, 272, 281, 289, 298, 306, 315, 324, 333, 342, 351, 361, 370, 380, 390, 400, 409, 419, 430, 440, 450, 461, 471, 482, 492, 503, 514, 525, 536, 547, 558, 569, 580, 591, 602, 613, 624, 635, 646, 656, 667, 678, 688, 699, 710, 721, 731, 742, 753, 764, 774] }, LightSaber: { Cheap: [21, 22, 24, 26, 27, 29, 31, 33, 35, 37, 39, 41, 43, 45, 47, 49, 51, 53, 55, 57, 59, 61, 63, 65, 67, 69, 72, 74, 77, 79, 82, 85, 88, 92, 95, 99, 103, 107, 112, 117, 122, 127, 132, 138, 144, 151, 157, 164, 171, 179, 186, 194, 202, 209, 218, 225, 233, 240, 248, 257, 266, 275, 283, 293, 302, 311, 321, 330, 340, 350, 360, 370, 381, 391, 402, 412, 423, 434, 445, 457, 468, 479, 491, 503, 515, 527, 539, 551, 563, 575, 588, 600, 613, 625, 638, 650, 663, 675, 688, 701], Standard: [24, 26, 27, 29, 31, 33, 35, 37, 39, 41, 43, 45, 47, 49, 51, 53, 55, 57, 59, 61, 63, 65, 67, 69, 72, 74, 77, 79, 82, 85, 88, 92, 95, 99, 103, 107, 112, 117, 122, 127, 132, 138, 144, 151, 157, 164, 171, 179, 186, 194, 202, 209, 218, 225, 233, 240, 248, 257, 266, 275, 283, 293, 302, 311, 321, 330, 340, 350, 360, 370, 381, 391, 402, 412, 423, 434, 445, 457, 468, 479, 491, 503, 515, 527, 539, 551, 563, 575, 588, 600, 613, 625, 638, 650, 663, 675, 688, 701, 713, 726], Premium: [26, 27, 29, 31, 33, 35, 37, 39, 41, 43, 45, 47, 49, 51, 53, 55, 57, 59, 61, 63, 65, 67, 69, 72, 74, 77, 79, 82, 85, 88, 92, 95, 99, 103, 107, 112, 117, 122, 127, 132, 138, 144, 151, 157, 164, 171, 179, 186, 194, 202, 209, 218, 225, 233, 240, 248, 257, 266, 275, 283, 293, 302, 311, 321, 330, 340, 350, 360, 370, 381, 391, 402, 412, 423, 434, 445, 457, 468, 479, 491, 503, 515, 527, 539, 551, 563, 575, 588, 600, 613, 625, 638, 650, 663, 675, 688, 701, 713, 726, 738], Prototype: [29, 31, 33, 35, 37, 39, 41, 45, 47, 49, 51, 53, 55, 59, 61, 63, 65, 67, 69, 72, 74, 77, 79, 82, 85, 88, 95, 99, 103, 107, 112, 117, 122, 127, 132, 138, 144, 151, 157, 164, 171, 179, 186, 194, 202, 209, 218, 225, 233, 240, 248, 257, 266, 275, 283, 293, 302, 311, 321, 330, 340, 350, 360, 370, 381, 391, 402, 412, 423, 434, 445, 457, 468, 479, 491, 503, 515, 527, 539, 551, 563, 575, 588, 600, 613, 625, 638, 650, 663, 675, 688, 701, 713, 726, 738, 750, 762, 775, 787, 799], Artifact: [31, 33, 35, 39, 41, 43, 45, 49, 51, 53, 55, 59, 61, 63, 65, 67, 69, 74, 77, 79, 82, 85, 88, 92, 95, 99, 103, 107, 112, 117, 122, 132, 138, 144, 151, 157, 164, 179, 186, 194, 202, 209, 218, 225, 233, 240, 248, 257, 266, 275, 283, 293, 302, 311, 321, 330, 340, 350, 360, 370, 381, 391, 402, 412, 423, 434, 445, 457, 468, 479, 491, 503, 515, 527, 539, 551, 563, 575, 588, 600, 613, 625, 638, 650, 663, 675, 688, 701, 713, 726, 738, 750, 762, 775, 787, 799, 811, 824, 836, 848], Legendary: [33, 35, 39, 41, 43, 47, 49, 53, 55, 57, 61, 63, 65, 67, 72, 74, 77, 79, 82, 85, 88, 95, 99, 103, 107, 112, 117, 122, 127, 132, 138, 144, 157, 164, 171, 179, 186, 194, 202, 209, 218, 225, 233, 240, 248, 266, 275, 283, 293, 302, 311, 321, 330, 340, 350, 360, 370, 381, 391, 402, 412, 423, 434, 445, 457, 468, 479, 491, 503, 515, 527, 539, 551, 563, 575, 588, 600, 613, 625, 638, 650, 663, 675, 688, 701, 713, 726, 738, 750, 762, 775, 787, 799, 811, 824, 836, 848, 860, 873, 885] }, TechBlade: { Cheap: [21, 23, 25, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 55, 56, 59, 60, 63, 65, 67, 69, 71, 74, 76, 79, 81, 84, 87, 90, 94, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 155, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718], Standard: [25, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 55, 56, 59, 60, 63, 65, 67, 69, 71, 74, 76, 79, 81, 84, 87, 90, 94, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 155, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744], Premium: [26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 55, 56, 59, 60, 63, 65, 67, 69, 71, 74, 76, 79, 81, 84, 87, 90, 94, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 155, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744, 756], Prototype: [30, 32, 34, 36, 38, 40, 42, 46, 48, 50, 52, 55, 56, 60, 63, 65, 67, 69, 71, 74, 76, 79, 81, 84, 87, 90, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 155, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744, 756, 769, 781, 794, 807, 819], Artifact: [32, 34, 36, 40, 42, 44, 46, 50, 52, 55, 56, 60, 63, 65, 67, 69, 71, 76, 79, 81, 84, 87, 90, 94, 98, 101, 106, 110, 114, 119, 125, 136, 142, 148, 155, 161, 168, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744, 756, 769, 781, 794, 807, 819, 832, 844, 857, 869], Legendary: [34, 36, 40, 42, 44, 48, 50, 55, 56, 59, 63, 65, 67, 69, 74, 76, 79, 81, 84, 87, 90, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744, 756, 769, 781, 794, 807, 819, 832, 844, 857, 869, 882, 894, 907] }, TrainingSaber: { Cheap: [21, 23, 25, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 55, 56, 59, 60, 63, 65, 67, 69, 71, 74, 76, 79, 81, 84, 87, 90, 94, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 155, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718], Standard: [25, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 55, 56, 59, 60, 63, 65, 67, 69, 71, 74, 76, 79, 81, 84, 87, 90, 94, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 155, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744], Premium: [26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 55, 56, 59, 60, 63, 65, 67, 69, 71, 74, 76, 79, 81, 84, 87, 90, 94, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 155, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744, 756], Prototype: [30, 32, 34, 36, 38, 40, 42, 46, 48, 50, 52, 55, 56, 60, 63, 65, 67, 69, 71, 74, 76, 79, 81, 84, 87, 90, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 155, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744, 756, 769, 781, 794, 807, 819], Artifact: [32, 34, 36, 40, 42, 44, 46, 50, 52, 55, 56, 60, 63, 65, 67, 69, 71, 76, 79, 81, 84, 87, 90, 94, 98, 101, 106, 110, 114, 119, 125, 136, 142, 148, 155, 161, 168, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744, 756, 769, 781, 794, 807, 819, 832, 844, 857, 869], Legendary: [34, 36, 40, 42, 44, 48, 50, 55, 56, 59, 63, 65, 67, 69, 74, 76, 79, 81, 84, 87, 90, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744, 756, 769, 781, 794, 807, 819, 832, 844, 857, 869, 882, 894, 907] }, ElectroStaff: { Cheap: [26, 28, 30, 31, 33, 36, 38, 40, 43, 46, 48, 50, 53, 55, 58, 60, 63, 65, 68, 70, 72, 75, 78, 80, 83, 85, 88, 91, 94, 98, 101, 105, 108, 113, 117, 122, 127, 132, 137, 143, 149, 156, 163, 170, 178, 185, 193, 202, 211, 220, 229, 238, 248, 258, 268, 277, 287, 295, 305, 316, 327, 338, 349, 360, 371, 383, 395, 406, 418, 430, 443, 456, 468, 481, 494, 507, 521, 534, 548, 562, 575, 590, 604, 618, 633, 648, 662, 677, 692, 708, 723, 738, 754, 769, 784, 800, 815, 831, 847, 862], Standard: [30, 31, 33, 36, 38, 40, 43, 46, 48, 50, 53, 55, 58, 60, 63, 65, 68, 70, 72, 75, 78, 80, 83, 85, 88, 91, 94, 98, 101, 105, 108, 113, 117, 122, 127, 132, 137, 143, 149, 156, 163, 170, 178, 185, 193, 202, 211, 220, 229, 238, 248, 258, 268, 277, 287, 295, 305, 316, 327, 338, 349, 360, 371, 383, 395, 406, 418, 430, 443, 456, 468, 481, 494, 507, 521, 534, 548, 562, 575, 590, 604, 618, 633, 648, 662, 677, 692, 708, 723, 738, 754, 769, 784, 800, 815, 831, 847, 862, 877, 892], Premium: [31, 33, 36, 38, 40, 43, 46, 48, 50, 53, 55, 58, 60, 63, 65, 68, 70, 72, 75, 78, 80, 83, 85, 88, 91, 94, 98, 101, 105, 108, 113, 117, 122, 127, 132, 137, 143, 149, 156, 163, 170, 178, 185, 193, 202, 211, 220, 229, 238, 248, 258, 268, 277, 287, 295, 305, 316, 327, 338, 349, 360, 371, 383, 395, 406, 418, 430, 443, 456, 468, 481, 494, 507, 521, 534, 548, 562, 575, 590, 604, 618, 633, 648, 662, 677, 692, 708, 723, 738, 754, 769, 784, 800, 815, 831, 847, 862, 877, 892, 907], Prototype: [36, 38, 40, 43, 46, 48, 50, 55, 58, 60, 63, 65, 68, 72, 75, 78, 80, 83, 85, 88, 91, 94, 98, 101, 105, 108, 117, 122, 127, 132, 137, 143, 149, 156, 163, 170, 178, 185, 193, 202, 211, 220, 229, 238, 248, 258, 268, 277, 287, 295, 305, 316, 327, 338, 349, 360, 371, 383, 395, 406, 418, 430, 443, 456, 468, 481, 494, 507, 521, 534, 548, 562, 575, 590, 604, 618, 633, 648, 662, 677, 692, 708, 723, 738, 754, 769, 784, 800, 815, 831, 847, 862, 877, 892, 907, 923, 938, 953, 968, 983], Artifact: [38, 40, 43, 48, 50, 53, 55, 60, 63, 65, 68, 72, 75, 78, 80, 83, 85, 91, 94, 98, 101, 105, 108, 113, 117, 122, 127, 132, 137, 143, 149, 163, 170, 178, 185, 193, 202, 220, 229, 238, 248, 258, 268, 277, 287, 295, 305, 316, 327, 338, 349, 360, 371, 383, 395, 406, 418, 430, 443, 456, 468, 481, 494, 507, 521, 534, 548, 562, 575, 590, 604, 618, 633, 648, 662, 677, 692, 708, 723, 738, 754, 769, 784, 800, 815, 831, 847, 862, 877, 892, 907, 923, 938, 953, 968, 983, 998, 1013, 1028, 1043], Legendary: [40, 43, 48, 50, 53, 58, 60, 65, 68, 70, 75, 78, 80, 83, 88, 91, 94, 98, 101, 105, 108, 117, 122, 127, 132, 137, 143, 149, 156, 163, 170, 178, 193, 202, 211, 220, 229, 238, 248, 258, 268, 277, 287, 295, 305, 327, 338, 349, 360, 371, 383, 395, 406, 418, 430, 443, 456, 468, 481, 494, 507, 521, 534, 548, 562, 575, 590, 604, 618, 633, 648, 662, 677, 692, 708, 723, 738, 754, 769, 784, 800, 815, 831, 847, 862, 877, 892, 907, 923, 938, 953, 968, 983, 998, 1013, 1028, 1043, 1058, 1073, 1088] }, VibroSword: { Cheap: [21, 23, 25, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 55, 56, 59, 60, 63, 65, 67, 69, 71, 74, 76, 79, 81, 84, 87, 90, 94, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 155, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718], Standard: [25, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 55, 56, 59, 60, 63, 65, 67, 69, 71, 74, 76, 79, 81, 84, 87, 90, 94, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 155, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744], Premium: [26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 55, 56, 59, 60, 63, 65, 67, 69, 71, 74, 76, 79, 81, 84, 87, 90, 94, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 155, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744, 756], Prototype: [30, 32, 34, 36, 38, 40, 42, 46, 48, 50, 52, 55, 56, 60, 63, 65, 67, 69, 71, 74, 76, 79, 81, 84, 87, 90, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 155, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744, 756, 769, 781, 794, 807, 819], Artifact: [32, 34, 36, 40, 42, 44, 46, 50, 52, 55, 56, 60, 63, 65, 67, 69, 71, 76, 79, 81, 84, 87, 90, 94, 98, 101, 106, 110, 114, 119, 125, 136, 142, 148, 155, 161, 168, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 263, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744, 756, 769, 781, 794, 807, 819, 832, 844, 857, 869], Legendary: [34, 36, 40, 42, 44, 48, 50, 55, 56, 59, 63, 65, 67, 69, 74, 76, 79, 81, 84, 87, 90, 98, 101, 106, 110, 114, 119, 125, 130, 136, 142, 148, 161, 168, 176, 183, 191, 199, 207, 215, 223, 231, 239, 246, 254, 272, 281, 291, 300, 309, 319, 329, 339, 349, 359, 369, 380, 390, 401, 412, 423, 434, 445, 457, 468, 480, 491, 503, 515, 527, 540, 552, 564, 577, 590, 602, 615, 628, 641, 654, 667, 680, 692, 705, 718, 731, 744, 756, 769, 781, 794, 807, 819, 832, 844, 857, 869, 882, 894, 907] }, AssaultCannon: { Cheap: [22, 24, 25, 27, 28, 30, 33, 35, 37, 39, 41, 43, 45, 47, 50, 51, 54, 56, 58, 60, 62, 64, 66, 68, 71, 73, 75, 78, 81, 83, 86, 89, 93, 96, 100, 104, 108, 113, 117, 122, 128, 133, 139, 145, 152, 158, 165, 173, 180, 188, 196, 203, 212, 220, 228, 237, 245, 252, 261, 270, 279, 288, 298, 307, 317, 327, 337, 347, 357, 367, 378, 389, 400, 410, 422, 433, 444, 456, 468, 479, 491, 503, 516, 528, 540, 553, 565, 578, 591, 604, 617, 630, 643, 656, 670, 683, 696, 709, 723, 736], Standard: [25, 27, 28, 30, 33, 35, 37, 39, 41, 43, 45, 47, 50, 51, 54, 56, 58, 60, 62, 64, 66, 68, 71, 73, 75, 78, 81, 83, 86, 89, 93, 96, 100, 104, 108, 113, 117, 122, 128, 133, 139, 145, 152, 158, 165, 173, 180, 188, 196, 203, 212, 220, 228, 237, 245, 252, 261, 270, 279, 288, 298, 307, 317, 327, 337, 347, 357, 367, 378, 389, 400, 410, 422, 433, 444, 456, 468, 479, 491, 503, 516, 528, 540, 553, 565, 578, 591, 604, 617, 630, 643, 656, 670, 683, 696, 709, 723, 736, 749, 762], Premium: [27, 28, 30, 33, 35, 37, 39, 41, 43, 45, 47, 50, 51, 54, 56, 58, 60, 62, 64, 66, 68, 71, 73, 75, 78, 81, 83, 86, 89, 93, 96, 100, 104, 108, 113, 117, 122, 128, 133, 139, 145, 152, 158, 165, 173, 180, 188, 196, 203, 212, 220, 228, 237, 245, 252, 261, 270, 279, 288, 298, 307, 317, 327, 337, 347, 357, 367, 378, 389, 400, 410, 422, 433, 444, 456, 468, 479, 491, 503, 516, 528, 540, 553, 565, 578, 591, 604, 617, 630, 643, 656, 670, 683, 696, 709, 723, 736, 749, 762, 775], Prototype: [30, 33, 35, 37, 39, 41, 43, 47, 50, 51, 54, 56, 58, 62, 64, 66, 68, 71, 73, 75, 78, 81, 83, 86, 89, 93, 100, 104, 108, 113, 117, 122, 128, 133, 139, 145, 152, 158, 165, 173, 180, 188, 196, 203, 212, 220, 228, 237, 245, 252, 261, 270, 279, 288, 298, 307, 317, 327, 337, 347, 357, 367, 378, 389, 400, 410, 422, 433, 444, 456, 468, 479, 491, 503, 516, 528, 540, 553, 565, 578, 591, 604, 617, 630, 643, 656, 670, 683, 696, 709, 723, 736, 749, 762, 775, 788, 800, 813, 826, 839], Artifact: [33, 35, 37, 41, 43, 45, 47, 51, 54, 56, 58, 62, 64, 66, 68, 71, 73, 78, 81, 83, 86, 89, 93, 96, 100, 104, 108, 113, 117, 122, 128, 139, 145, 152, 158, 165, 173, 188, 196, 203, 212, 220, 228, 237, 245, 252, 261, 270, 279, 288, 298, 307, 317, 327, 337, 347, 357, 367, 378, 389, 400, 410, 422, 433, 444, 456, 468, 479, 491, 503, 516, 528, 540, 553, 565, 578, 591, 604, 617, 630, 643, 656, 670, 683, 696, 709, 723, 736, 749, 762, 775, 788, 800, 813, 826, 839, 852, 865, 878, 891], Legendary: [35, 37, 41, 43, 45, 50, 51, 56, 58, 60, 64, 66, 68, 71, 75, 78, 81, 83, 86, 89, 93, 100, 104, 108, 113, 117, 122, 128, 133, 139, 145, 152, 165, 173, 180, 188, 196, 203, 212, 220, 228, 237, 245, 252, 261, 279, 288, 298, 307, 317, 327, 337, 347, 357, 367, 378, 389, 400, 410, 422, 433, 444, 456, 468, 479, 491, 503, 516, 528, 540, 553, 565, 578, 591, 604, 617, 630, 643, 656, 670, 683, 696, 709, 723, 736, 749, 762, 775, 788, 800, 813, 826, 839, 852, 865, 878, 891, 903, 916, 929] }, TechStaff: { Cheap: [26, 28, 30, 31, 33, 36, 38, 40, 43, 46, 48, 50, 53, 55, 58, 60, 63, 65, 68, 70, 72, 75, 78, 80, 83, 85, 88, 91, 94, 98, 101, 105, 108, 113, 117, 122, 127, 132, 137, 143, 149, 156, 163, 170, 178, 185, 193, 202, 211, 220, 229, 238, 248, 258, 268, 277, 287, 295, 305, 316, 327, 338, 349, 360, 371, 383, 395, 406, 418, 430, 443, 456, 468, 481, 494, 507, 521, 534, 548, 562, 575, 590, 604, 618, 633, 648, 662, 677, 692, 708, 723, 738, 754, 769, 784, 800, 815, 831, 847, 862], Standard: [30, 31, 33, 36, 38, 40, 43, 46, 48, 50, 53, 55, 58, 60, 63, 65, 68, 70, 72, 75, 78, 80, 83, 85, 88, 91, 94, 98, 101, 105, 108, 113, 117, 122, 127, 132, 137, 143, 149, 156, 163, 170, 178, 185, 193, 202, 211, 220, 229, 238, 248, 258, 268, 277, 287, 295, 305, 316, 327, 338, 349, 360, 371, 383, 395, 406, 418, 430, 443, 456, 468, 481, 494, 507, 521, 534, 548, 562, 575, 590, 604, 618, 633, 648, 662, 677, 692, 708, 723, 738, 754, 769, 784, 800, 815, 831, 847, 862, 877, 892], Premium: [31, 33, 36, 38, 40, 43, 46, 48, 50, 53, 55, 58, 60, 63, 65, 68, 70, 72, 75, 78, 80, 83, 85, 88, 91, 94, 98, 101, 105, 108, 113, 117, 122, 127, 132, 137, 143, 149, 156, 163, 170, 178, 185, 193, 202, 211, 220, 229, 238, 248, 258, 268, 277, 287, 295, 305, 316, 327, 338, 349, 360, 371, 383, 395, 406, 418, 430, 443, 456, 468, 481, 494, 507, 521, 534, 548, 562, 575, 590, 604, 618, 633, 648, 662, 677, 692, 708, 723, 738, 754, 769, 784, 800, 815, 831, 847, 862, 877, 892, 907], Prototype: [36, 38, 40, 43, 46, 48, 50, 55, 58, 60, 63, 65, 68, 72, 75, 78, 80, 83, 85, 88, 91, 94, 98, 101, 105, 108, 117, 122, 127, 132, 137, 143, 149, 156, 163, 170, 178, 185, 193, 202, 211, 220, 229, 238, 248, 258, 268, 277, 287, 295, 305, 316, 327, 338, 349, 360, 371, 383, 395, 406, 418, 430, 443, 456, 468, 481, 494, 507, 521, 534, 548, 562, 575, 590, 604, 618, 633, 648, 662, 677, 692, 708, 723, 738, 754, 769, 784, 800, 815, 831, 847, 862, 877, 892, 907, 923, 938, 953, 968, 983], Artifact: [38, 40, 43, 48, 50, 53, 55, 60, 63, 65, 68, 72, 75, 78, 80, 83, 85, 91, 94, 98, 101, 105, 108, 113, 117, 122, 127, 132, 137, 143, 149, 163, 170, 178, 185, 193, 202, 220, 229, 238, 248, 258, 268, 277, 287, 295, 305, 316, 327, 338, 349, 360, 371, 383, 395, 406, 418, 430, 443, 456, 468, 481, 494, 507, 521, 534, 548, 562, 575, 590, 604, 618, 633, 648, 662, 677, 692, 708, 723, 738, 754, 769, 784, 800, 815, 831, 847, 862, 877, 892, 907, 923, 938, 953, 968, 983, 998, 1013, 1028, 1043], Legendary: [40, 43, 48, 50, 53, 58, 60, 65, 68, 70, 75, 78, 80, 83, 88, 91, 94, 98, 101, 105, 108, 117, 122, 127, 132, 137, 143, 149, 156, 163, 170, 178, 193, 202, 211, 220, 229, 238, 248, 258, 268, 277, 287, 295, 305, 327, 338, 349, 360, 371, 383, 395, 406, 418, 430, 443, 456, 468, 481, 494, 507, 521, 534, 548, 562, 575, 590, 604, 618, 633, 648, 662, 677, 692, 708, 723, 738, 754, 769, 784, 800, 815, 831, 847, 862, 877, 892, 907, 923, 938, 953, 968, 983, 998, 1013, 1028, 1043, 1058, 1073, 1088] }, ScatterGun: { Cheap: [21, 23, 24, 26, 27, 29, 31, 33, 35, 37, 40, 41, 43, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 66, 68, 70, 72, 75, 77, 80, 83, 86, 89, 92, 96, 100, 104, 108, 113, 118, 123, 128, 134, 140, 146, 152, 159, 166, 173, 180, 188, 196, 204, 212, 220, 228, 236, 242, 251, 259, 268, 277, 286, 296, 305, 314, 324, 334, 343, 354, 364, 374, 384, 395, 406, 416, 428, 439, 450, 461, 473, 484, 496, 508, 520, 532, 544, 556, 568, 581, 593, 606, 619, 631, 644, 657, 670, 682, 695, 708], Standard: [24, 26, 27, 29, 31, 33, 35, 37, 40, 41, 43, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 66, 68, 70, 72, 75, 77, 80, 83, 86, 89, 92, 96, 100, 104, 108, 113, 118, 123, 128, 134, 140, 146, 152, 159, 166, 173, 180, 188, 196, 204, 212, 220, 228, 236, 242, 251, 259, 268, 277, 286, 296, 305, 314, 324, 334, 343, 354, 364, 374, 384, 395, 406, 416, 428, 439, 450, 461, 473, 484, 496, 508, 520, 532, 544, 556, 568, 581, 593, 606, 619, 631, 644, 657, 670, 682, 695, 708, 720, 733], Premium: [26, 27, 29, 31, 33, 35, 37, 40, 41, 43, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 66, 68, 70, 72, 75, 77, 80, 83, 86, 89, 92, 96, 100, 104, 108, 113, 118, 123, 128, 134, 140, 146, 152, 159, 166, 173, 180, 188, 196, 204, 212, 220, 228, 236, 242, 251, 259, 268, 277, 286, 296, 305, 314, 324, 334, 343, 354, 364, 374, 384, 395, 406, 416, 428, 439, 450, 461, 473, 484, 496, 508, 520, 532, 544, 556, 568, 581, 593, 606, 619, 631, 644, 657, 670, 682, 695, 708, 720, 733, 745], Prototype: [29, 31, 33, 35, 37, 40, 41, 46, 48, 50, 52, 54, 56, 60, 62, 64, 66, 68, 70, 72, 75, 77, 80, 83, 86, 89, 96, 100, 104, 108, 113, 118, 123, 128, 134, 140, 146, 152, 159, 166, 173, 180, 188, 196, 204, 212, 220, 228, 236, 242, 251, 259, 268, 277, 286, 296, 305, 314, 324, 334, 343, 354, 364, 374, 384, 395, 406, 416, 428, 439, 450, 461, 473, 484, 496, 508, 520, 532, 544, 556, 568, 581, 593, 606, 619, 631, 644, 657, 670, 682, 695, 708, 720, 733, 745, 758, 770, 782, 795, 807], Artifact: [31, 33, 35, 40, 41, 43, 46, 50, 52, 54, 56, 60, 62, 64, 66, 68, 70, 75, 77, 80, 83, 86, 89, 92, 96, 100, 104, 108, 113, 118, 123, 134, 140, 146, 152, 159, 166, 180, 188, 196, 204, 212, 220, 228, 236, 242, 251, 259, 268, 277, 286, 296, 305, 314, 324, 334, 343, 354, 364, 374, 384, 395, 406, 416, 428, 439, 450, 461, 473, 484, 496, 508, 520, 532, 544, 556, 568, 581, 593, 606, 619, 631, 644, 657, 670, 682, 695, 708, 720, 733, 745, 758, 770, 782, 795, 807, 819, 832, 844, 857], Legendary: [33, 35, 40, 41, 43, 48, 50, 54, 56, 58, 62, 64, 66, 68, 72, 75, 77, 80, 83, 86, 89, 96, 100, 104, 108, 113, 118, 123, 128, 134, 140, 146, 159, 166, 173, 180, 188, 196, 204, 212, 220, 228, 236, 242, 251, 268, 277, 286, 296, 305, 314, 324, 334, 343, 354, 364, 374, 384, 395, 406, 416, 428, 439, 450, 461, 473, 484, 496, 508, 520, 532, 544, 556, 568, 581, 593, 606, 619, 631, 644, 657, 670, 682, 695, 708, 720, 733, 745, 758, 770, 782, 795, 807, 819, 832, 844, 857, 869, 881, 894] }, VibroKnife: { Cheap: [22, 24, 26, 28, 29, 31, 33, 35, 38, 40, 42, 44, 46, 49, 51, 53, 55, 57, 59, 62, 64, 66, 68, 70, 72, 75, 77, 80, 83, 85, 89, 92, 95, 99, 103, 106, 111, 115, 120, 126, 131, 137, 143, 149, 156, 162, 169, 177, 185, 192, 201, 209, 217, 226, 234, 243, 251, 258, 267, 277, 286, 296, 305, 315, 325, 335, 346, 356, 366, 377, 388, 399, 410, 421, 433, 444, 456, 468, 480, 492, 504, 516, 529, 542, 554, 567, 580, 593, 606, 620, 633, 647, 660, 673, 687, 701, 714, 728, 741, 755], Standard: [26, 28, 29, 31, 33, 35, 38, 40, 42, 44, 46, 49, 51, 53, 55, 57, 59, 62, 64, 66, 68, 70, 72, 75, 77, 80, 83, 85, 89, 92, 95, 99, 103, 106, 111, 115, 120, 126, 131, 137, 143, 149, 156, 162, 169, 177, 185, 192, 201, 209, 217, 226, 234, 243, 251, 258, 267, 277, 286, 296, 305, 315, 325, 335, 346, 356, 366, 377, 388, 399, 410, 421, 433, 444, 456, 468, 480, 492, 504, 516, 529, 542, 554, 567, 580, 593, 606, 620, 633, 647, 660, 673, 687, 701, 714, 728, 741, 755, 768, 782], Premium: [28, 29, 31, 33, 35, 38, 40, 42, 44, 46, 49, 51, 53, 55, 57, 59, 62, 64, 66, 68, 70, 72, 75, 77, 80, 83, 85, 89, 92, 95, 99, 103, 106, 111, 115, 120, 126, 131, 137, 143, 149, 156, 162, 169, 177, 185, 192, 201, 209, 217, 226, 234, 243, 251, 258, 267, 277, 286, 296, 305, 315, 325, 335, 346, 356, 366, 377, 388, 399, 410, 421, 433, 444, 456, 468, 480, 492, 504, 516, 529, 542, 554, 567, 580, 593, 606, 620, 633, 647, 660, 673, 687, 701, 714, 728, 741, 755, 768, 782, 795], Prototype: [31, 33, 35, 38, 40, 42, 44, 49, 51, 53, 55, 57, 59, 64, 66, 68, 70, 72, 75, 77, 80, 83, 85, 89, 92, 95, 103, 106, 111, 115, 120, 126, 131, 137, 143, 149, 156, 162, 169, 177, 185, 192, 201, 209, 217, 226, 234, 243, 251, 258, 267, 277, 286, 296, 305, 315, 325, 335, 346, 356, 366, 377, 388, 399, 410, 421, 433, 444, 456, 468, 480, 492, 504, 516, 529, 542, 554, 567, 580, 593, 606, 620, 633, 647, 660, 673, 687, 701, 714, 728, 741, 755, 768, 782, 795, 808, 821, 834, 848, 861], Artifact: [33, 35, 38, 42, 44, 46, 49, 53, 55, 57, 59, 64, 66, 68, 70, 72, 75, 80, 83, 85, 89, 92, 95, 99, 103, 106, 111, 115, 120, 126, 131, 143, 149, 156, 162, 169, 177, 192, 201, 209, 217, 226, 234, 243, 251, 258, 267, 277, 286, 296, 305, 315, 325, 335, 346, 356, 366, 377, 388, 399, 410, 421, 433, 444, 456, 468, 480, 492, 504, 516, 529, 542, 554, 567, 580, 593, 606, 620, 633, 647, 660, 673, 687, 701, 714, 728, 741, 755, 768, 782, 795, 808, 821, 834, 848, 861, 874, 887, 901, 914], Legendary: [35, 38, 42, 44, 46, 51, 53, 57, 59, 62, 66, 68, 70, 72, 77, 80, 83, 85, 89, 92, 95, 103, 106, 111, 115, 120, 126, 131, 137, 143, 149, 156, 169, 177, 185, 192, 201, 209, 217, 226, 234, 243, 251, 258, 267, 286, 296, 305, 315, 325, 335, 346, 356, 366, 377, 388, 399, 410, 421, 433, 444, 456, 468, 480, 492, 504, 516, 529, 542, 554, 567, 580, 593, 606, 620, 633, 647, 660, 673, 687, 701, 714, 728, 741, 755, 768, 782, 795, 808, 821, 834, 848, 861, 874, 887, 901, 914, 927, 940, 953] } },
        max: { PoleSaber: { Cheap: [38, 40, 43, 46, 49, 52, 56, 59, 63, 67, 70, 74, 77, 81, 85, 88, 92, 96, 99, 103, 106, 110, 114, 117, 121, 125, 129, 133, 138, 143, 148, 153, 159, 165, 171, 178, 185, 193, 201, 210, 219, 228, 238, 249, 260, 271, 283, 296, 308, 322, 335, 349, 363, 377, 392, 406, 420, 432, 447, 462, 478, 494, 510, 527, 543, 560, 577, 595, 612, 630, 648, 667, 685, 704, 723, 742, 762, 782, 802, 822, 842, 863, 884, 905, 926, 948, 969, 991, 1013, 1036, 1058, 1080, 1103, 1125, 1148, 1171, 1193, 1216, 1239, 1261], Standard: [43, 46, 49, 52, 56, 59, 63, 67, 70, 74, 77, 81, 85, 88, 92, 96, 99, 103, 106, 110, 114, 117, 121, 125, 129, 133, 138, 143, 148, 153, 159, 165, 171, 178, 185, 193, 201, 210, 219, 228, 238, 249, 260, 271, 283, 296, 308, 322, 335, 349, 363, 377, 392, 406, 420, 432, 447, 462, 478, 494, 510, 527, 543, 560, 577, 595, 612, 630, 648, 667, 685, 704, 723, 742, 762, 782, 802, 822, 842, 863, 884, 905, 926, 948, 969, 991, 1013, 1036, 1058, 1080, 1103, 1125, 1148, 1171, 1193, 1216, 1239, 1261, 1284, 1306], Premium: [46, 49, 52, 56, 59, 63, 67, 70, 74, 77, 81, 85, 88, 92, 96, 99, 103, 106, 110, 114, 117, 121, 125, 129, 133, 138, 143, 148, 153, 159, 165, 171, 178, 185, 193, 201, 210, 219, 228, 238, 249, 260, 271, 283, 296, 308, 322, 335, 349, 363, 377, 392, 406, 420, 432, 447, 462, 478, 494, 510, 527, 543, 560, 577, 595, 612, 630, 648, 667, 685, 704, 723, 742, 762, 782, 802, 822, 842, 863, 884, 905, 926, 948, 969, 991, 1013, 1036, 1058, 1080, 1103, 1125, 1148, 1171, 1193, 1216, 1239, 1261, 1284, 1306, 1328], Prototype: [52, 56, 59, 63, 67, 70, 74, 81, 85, 88, 92, 96, 99, 106, 110, 114, 117, 121, 125, 129, 133, 138, 143, 148, 153, 159, 171, 178, 185, 193, 201, 210, 219, 228, 238, 249, 260, 271, 283, 296, 308, 322, 335, 349, 363, 377, 392, 406, 420, 432, 447, 462, 478, 494, 510, 527, 543, 560, 577, 595, 612, 630, 648, 667, 685, 704, 723, 742, 762, 782, 802, 822, 842, 863, 884, 905, 926, 948, 969, 991, 1013, 1036, 1058, 1080, 1103, 1125, 1148, 1171, 1193, 1216, 1239, 1261, 1284, 1306, 1328, 1350, 1372, 1394, 1416, 1438], Artifact: [56, 59, 63, 70, 74, 77, 81, 88, 92, 96, 99, 106, 110, 114, 117, 121, 125, 133, 138, 143, 148, 153, 159, 165, 171, 178, 185, 193, 201, 210, 219, 238, 249, 260, 271, 283, 296, 322, 335, 349, 363, 377, 392, 406, 420, 432, 447, 462, 478, 494, 510, 527, 543, 560, 577, 595, 612, 630, 648, 667, 685, 704, 723, 742, 762, 782, 802, 822, 842, 863, 884, 905, 926, 948, 969, 991, 1013, 1036, 1058, 1080, 1103, 1125, 1148, 1171, 1193, 1216, 1239, 1261, 1284, 1306, 1328, 1350, 1372, 1394, 1416, 1438, 1460, 1482, 1505, 1527], Legendary: [59, 63, 70, 74, 77, 85, 88, 96, 99, 103, 110, 114, 117, 121, 129, 133, 138, 143, 148, 153, 159, 171, 178, 185, 193, 201, 210, 219, 228, 238, 249, 260, 283, 296, 308, 322, 335, 349, 363, 377, 392, 406, 420, 432, 447, 478, 494, 510, 527, 543, 560, 577, 595, 612, 630, 648, 667, 685, 704, 723, 742, 762, 782, 802, 822, 842, 863, 884, 905, 926, 948, 969, 991, 1013, 1036, 1058, 1080, 1103, 1125, 1148, 1171, 1193, 1216, 1239, 1261, 1284, 1306, 1328, 1350, 1372, 1394, 1416, 1438, 1460, 1482, 1505, 1527, 1549, 1571, 1593] }, BlasterRifle: { Cheap: [31, 34, 36, 38, 41, 43, 47, 49, 52, 56, 59, 61, 65, 68, 71, 74, 77, 80, 83, 86, 88, 92, 95, 98, 101, 104, 108, 111, 115, 119, 123, 128, 132, 137, 143, 148, 155, 161, 167, 175, 182, 190, 199, 207, 217, 226, 236, 246, 257, 268, 279, 291, 302, 314, 326, 338, 350, 360, 372, 385, 399, 412, 425, 439, 453, 467, 481, 496, 510, 525, 540, 556, 571, 586, 602, 618, 635, 651, 668, 685, 702, 719, 737, 754, 772, 790, 808, 826, 844, 863, 881, 900, 919, 938, 956, 976, 994, 1013, 1032, 1051], Standard: [36, 38, 41, 43, 47, 49, 52, 56, 59, 61, 65, 68, 71, 74, 77, 80, 83, 86, 88, 92, 95, 98, 101, 104, 108, 111, 115, 119, 123, 128, 132, 137, 143, 148, 155, 161, 167, 175, 182, 190, 199, 207, 217, 226, 236, 246, 257, 268, 279, 291, 302, 314, 326, 338, 350, 360, 372, 385, 399, 412, 425, 439, 453, 467, 481, 496, 510, 525, 540, 556, 571, 586, 602, 618, 635, 651, 668, 685, 702, 719, 737, 754, 772, 790, 808, 826, 844, 863, 881, 900, 919, 938, 956, 976, 994, 1013, 1032, 1051, 1070, 1088], Premium: [38, 41, 43, 47, 49, 52, 56, 59, 61, 65, 68, 71, 74, 77, 80, 83, 86, 88, 92, 95, 98, 101, 104, 108, 111, 115, 119, 123, 128, 132, 137, 143, 148, 155, 161, 167, 175, 182, 190, 199, 207, 217, 226, 236, 246, 257, 268, 279, 291, 302, 314, 326, 338, 350, 360, 372, 385, 399, 412, 425, 439, 453, 467, 481, 496, 510, 525, 540, 556, 571, 586, 602, 618, 635, 651, 668, 685, 702, 719, 737, 754, 772, 790, 808, 826, 844, 863, 881, 900, 919, 938, 956, 976, 994, 1013, 1032, 1051, 1070, 1088, 1107], Prototype: [43, 47, 49, 52, 56, 59, 61, 68, 71, 74, 77, 80, 83, 88, 92, 95, 98, 101, 104, 108, 111, 115, 119, 123, 128, 132, 143, 148, 155, 161, 167, 175, 182, 190, 199, 207, 217, 226, 236, 246, 257, 268, 279, 291, 302, 314, 326, 338, 350, 360, 372, 385, 399, 412, 425, 439, 453, 467, 481, 496, 510, 525, 540, 556, 571, 586, 602, 618, 635, 651, 668, 685, 702, 719, 737, 754, 772, 790, 808, 826, 844, 863, 881, 900, 919, 938, 956, 976, 994, 1013, 1032, 1051, 1070, 1088, 1107, 1125, 1143, 1162, 1180, 1199], Artifact: [47, 49, 52, 59, 61, 65, 68, 74, 77, 80, 83, 88, 92, 95, 98, 101, 104, 111, 115, 119, 123, 128, 132, 137, 143, 148, 155, 161, 167, 175, 182, 199, 207, 217, 226, 236, 246, 268, 279, 291, 302, 314, 326, 338, 350, 360, 372, 385, 399, 412, 425, 439, 453, 467, 481, 496, 510, 525, 540, 556, 571, 586, 602, 618, 635, 651, 668, 685, 702, 719, 737, 754, 772, 790, 808, 826, 844, 863, 881, 900, 919, 938, 956, 976, 994, 1013, 1032, 1051, 1070, 1088, 1107, 1125, 1143, 1162, 1180, 1199, 1217, 1235, 1254, 1272], Legendary: [49, 52, 59, 61, 65, 71, 74, 80, 83, 86, 92, 95, 98, 101, 108, 111, 115, 119, 123, 128, 132, 143, 148, 155, 161, 167, 175, 182, 190, 199, 207, 217, 236, 246, 257, 268, 279, 291, 302, 314, 326, 338, 350, 360, 372, 399, 412, 425, 439, 453, 467, 481, 496, 510, 525, 540, 556, 571, 586, 602, 618, 635, 651, 668, 685, 702, 719, 737, 754, 772, 790, 808, 826, 844, 863, 881, 900, 919, 938, 956, 976, 994, 1013, 1032, 1051, 1070, 1088, 1107, 1125, 1143, 1162, 1180, 1199, 1217, 1235, 1254, 1272, 1291, 1309, 1327] }, SniperRifle: { Cheap: [34, 37, 40, 42, 45, 48, 51, 54, 58, 61, 65, 68, 71, 74, 78, 81, 84, 88, 91, 94, 97, 101, 104, 108, 111, 114, 118, 122, 127, 131, 136, 140, 145, 151, 157, 163, 170, 177, 184, 192, 201, 209, 219, 228, 238, 249, 259, 271, 283, 295, 307, 320, 333, 346, 359, 372, 385, 396, 410, 424, 438, 453, 468, 483, 498, 513, 529, 545, 561, 577, 594, 611, 628, 645, 663, 680, 698, 716, 735, 753, 772, 791, 810, 830, 849, 869, 889, 908, 929, 949, 970, 990, 1011, 1031, 1052, 1073, 1094, 1115, 1136, 1156], Standard: [40, 42, 45, 48, 51, 54, 58, 61, 65, 68, 71, 74, 78, 81, 84, 88, 91, 94, 97, 101, 104, 108, 111, 114, 118, 122, 127, 131, 136, 140, 145, 151, 157, 163, 170, 177, 184, 192, 201, 209, 219, 228, 238, 249, 259, 271, 283, 295, 307, 320, 333, 346, 359, 372, 385, 396, 410, 424, 438, 453, 468, 483, 498, 513, 529, 545, 561, 577, 594, 611, 628, 645, 663, 680, 698, 716, 735, 753, 772, 791, 810, 830, 849, 869, 889, 908, 929, 949, 970, 990, 1011, 1031, 1052, 1073, 1094, 1115, 1136, 1156, 1177, 1197], Premium: [42, 45, 48, 51, 54, 58, 61, 65, 68, 71, 74, 78, 81, 84, 88, 91, 94, 97, 101, 104, 108, 111, 114, 118, 122, 127, 131, 136, 140, 145, 151, 157, 163, 170, 177, 184, 192, 201, 209, 219, 228, 238, 249, 259, 271, 283, 295, 307, 320, 333, 346, 359, 372, 385, 396, 410, 424, 438, 453, 468, 483, 498, 513, 529, 545, 561, 577, 594, 611, 628, 645, 663, 680, 698, 716, 735, 753, 772, 791, 810, 830, 849, 869, 889, 908, 929, 949, 970, 990, 1011, 1031, 1052, 1073, 1094, 1115, 1136, 1156, 1177, 1197, 1217], Prototype: [48, 51, 54, 58, 61, 65, 68, 74, 78, 81, 84, 88, 91, 97, 101, 104, 108, 111, 114, 118, 122, 127, 131, 136, 140, 145, 157, 163, 170, 177, 184, 192, 201, 209, 219, 228, 238, 249, 259, 271, 283, 295, 307, 320, 333, 346, 359, 372, 385, 396, 410, 424, 438, 453, 468, 483, 498, 513, 529, 545, 561, 577, 594, 611, 628, 645, 663, 680, 698, 716, 735, 753, 772, 791, 810, 830, 849, 869, 889, 908, 929, 949, 970, 990, 1011, 1031, 1052, 1073, 1094, 1115, 1136, 1156, 1177, 1197, 1217, 1238, 1258, 1278, 1298, 1318], Artifact: [51, 54, 58, 65, 68, 71, 74, 81, 84, 88, 91, 97, 101, 104, 108, 111, 114, 122, 127, 131, 136, 140, 145, 151, 157, 163, 170, 177, 184, 192, 201, 219, 228, 238, 249, 259, 271, 295, 307, 320, 333, 346, 359, 372, 385, 396, 410, 424, 438, 453, 468, 483, 498, 513, 529, 545, 561, 577, 594, 611, 628, 645, 663, 680, 698, 716, 735, 753, 772, 791, 810, 830, 849, 869, 889, 908, 929, 949, 970, 990, 1011, 1031, 1052, 1073, 1094, 1115, 1136, 1156, 1177, 1197, 1217, 1238, 1258, 1278, 1298, 1318, 1339, 1359, 1379, 1399], Legendary: [54, 58, 65, 68, 71, 78, 81, 88, 91, 94, 101, 104, 108, 111, 118, 122, 127, 131, 136, 140, 145, 157, 163, 170, 177, 184, 192, 201, 209, 219, 228, 238, 259, 271, 283, 295, 307, 320, 333, 346, 359, 372, 385, 396, 410, 438, 453, 468, 483, 498, 513, 529, 545, 561, 577, 594, 611, 628, 645, 663, 680, 698, 716, 735, 753, 772, 791, 810, 830, 849, 869, 889, 908, 929, 949, 970, 990, 1011, 1031, 1052, 1073, 1094, 1115, 1136, 1156, 1177, 1197, 1217, 1238, 1258, 1278, 1298, 1318, 1339, 1359, 1379, 1399, 1420, 1440, 1460] }, Pistol: { Cheap: [34, 36, 39, 42, 44, 47, 50, 53, 57, 60, 64, 67, 70, 73, 77, 80, 83, 86, 89, 93, 96, 99, 103, 106, 109, 113, 117, 120, 125, 129, 133, 138, 143, 149, 155, 161, 167, 174, 181, 189, 197, 206, 215, 225, 235, 245, 256, 267, 278, 290, 303, 315, 328, 340, 353, 366, 379, 390, 403, 417, 432, 446, 461, 476, 490, 506, 521, 537, 553, 569, 585, 602, 618, 635, 653, 670, 688, 706, 724, 742, 760, 779, 798, 817, 836, 856, 875, 895, 915, 935, 955, 975, 996, 1016, 1036, 1057, 1077, 1098, 1118, 1139], Standard: [39, 42, 44, 47, 50, 53, 57, 60, 64, 67, 70, 73, 77, 80, 83, 86, 89, 93, 96, 99, 103, 106, 109, 113, 117, 120, 125, 129, 133, 138, 143, 149, 155, 161, 167, 174, 181, 189, 197, 206, 215, 225, 235, 245, 256, 267, 278, 290, 303, 315, 328, 340, 353, 366, 379, 390, 403, 417, 432, 446, 461, 476, 490, 506, 521, 537, 553, 569, 585, 602, 618, 635, 653, 670, 688, 706, 724, 742, 760, 779, 798, 817, 836, 856, 875, 895, 915, 935, 955, 975, 996, 1016, 1036, 1057, 1077, 1098, 1118, 1139, 1159, 1179], Premium: [42, 44, 47, 50, 53, 57, 60, 64, 67, 70, 73, 77, 80, 83, 86, 89, 93, 96, 99, 103, 106, 109, 113, 117, 120, 125, 129, 133, 138, 143, 149, 155, 161, 167, 174, 181, 189, 197, 206, 215, 225, 235, 245, 256, 267, 278, 290, 303, 315, 328, 340, 353, 366, 379, 390, 403, 417, 432, 446, 461, 476, 490, 506, 521, 537, 553, 569, 585, 602, 618, 635, 653, 670, 688, 706, 724, 742, 760, 779, 798, 817, 836, 856, 875, 895, 915, 935, 955, 975, 996, 1016, 1036, 1057, 1077, 1098, 1118, 1139, 1159, 1179, 1199], Prototype: [47, 50, 53, 57, 60, 64, 67, 73, 77, 80, 83, 86, 89, 96, 99, 103, 106, 109, 113, 117, 120, 125, 129, 133, 138, 143, 155, 161, 167, 174, 181, 189, 197, 206, 215, 225, 235, 245, 256, 267, 278, 290, 303, 315, 328, 340, 353, 366, 379, 390, 403, 417, 432, 446, 461, 476, 490, 506, 521, 537, 553, 569, 585, 602, 618, 635, 653, 670, 688, 706, 724, 742, 760, 779, 798, 817, 836, 856, 875, 895, 915, 935, 955, 975, 996, 1016, 1036, 1057, 1077, 1098, 1118, 1139, 1159, 1179, 1199, 1219, 1239, 1259, 1279, 1299], Artifact: [50, 53, 57, 64, 67, 70, 73, 80, 83, 86, 89, 96, 99, 103, 106, 109, 113, 120, 125, 129, 133, 138, 143, 149, 155, 161, 167, 174, 181, 189, 197, 215, 225, 235, 245, 256, 267, 290, 303, 315, 328, 340, 353, 366, 379, 390, 403, 417, 432, 446, 461, 476, 490, 506, 521, 537, 553, 569, 585, 602, 618, 635, 653, 670, 688, 706, 724, 742, 760, 779, 798, 817, 836, 856, 875, 895, 915, 935, 955, 975, 996, 1016, 1036, 1057, 1077, 1098, 1118, 1139, 1159, 1179, 1199, 1219, 1239, 1259, 1279, 1299, 1318, 1338, 1358, 1378], Legendary: [53, 57, 64, 67, 70, 77, 80, 86, 89, 93, 99, 103, 106, 109, 117, 120, 125, 129, 133, 138, 143, 155, 161, 167, 174, 181, 189, 197, 206, 215, 225, 235, 256, 267, 278, 290, 303, 315, 328, 340, 353, 366, 379, 390, 403, 432, 446, 461, 476, 490, 506, 521, 537, 553, 569, 585, 602, 618, 635, 653, 670, 688, 706, 724, 742, 760, 779, 798, 817, 836, 856, 875, 895, 915, 935, 955, 975, 996, 1016, 1036, 1057, 1077, 1098, 1118, 1139, 1159, 1179, 1199, 1219, 1239, 1259, 1279, 1299, 1318, 1338, 1358, 1378, 1398, 1418, 1438] }, LightSaber: { Cheap: [31, 34, 36, 38, 41, 43, 47, 49, 52, 56, 59, 61, 65, 68, 71, 74, 77, 80, 83, 86, 88, 92, 95, 98, 101, 104, 108, 111, 115, 119, 123, 128, 132, 137, 143, 148, 155, 161, 167, 175, 182, 190, 199, 207, 217, 226, 236, 246, 257, 268, 279, 291, 302, 314, 326, 338, 350, 360, 372, 385, 399, 412, 425, 439, 453, 467, 481, 496, 510, 525, 540, 556, 571, 586, 602, 618, 635, 651, 668, 685, 702, 719, 737, 754, 772, 790, 808, 826, 844, 863, 881, 900, 919, 938, 956, 976, 994, 1013, 1032, 1051], Standard: [36, 38, 41, 43, 47, 49, 52, 56, 59, 61, 65, 68, 71, 74, 77, 80, 83, 86, 88, 92, 95, 98, 101, 104, 108, 111, 115, 119, 123, 128, 132, 137, 143, 148, 155, 161, 167, 175, 182, 190, 199, 207, 217, 226, 236, 246, 257, 268, 279, 291, 302, 314, 326, 338, 350, 360, 372, 385, 399, 412, 425, 439, 453, 467, 481, 496, 510, 525, 540, 556, 571, 586, 602, 618, 635, 651, 668, 685, 702, 719, 737, 754, 772, 790, 808, 826, 844, 863, 881, 900, 919, 938, 956, 976, 994, 1013, 1032, 1051, 1070, 1088], Premium: [38, 41, 43, 47, 49, 52, 56, 59, 61, 65, 68, 71, 74, 77, 80, 83, 86, 88, 92, 95, 98, 101, 104, 108, 111, 115, 119, 123, 128, 132, 137, 143, 148, 155, 161, 167, 175, 182, 190, 199, 207, 217, 226, 236, 246, 257, 268, 279, 291, 302, 314, 326, 338, 350, 360, 372, 385, 399, 412, 425, 439, 453, 467, 481, 496, 510, 525, 540, 556, 571, 586, 602, 618, 635, 651, 668, 685, 702, 719, 737, 754, 772, 790, 808, 826, 844, 863, 881, 900, 919, 938, 956, 976, 994, 1013, 1032, 1051, 1070, 1088, 1107], Prototype: [43, 47, 49, 52, 56, 59, 61, 68, 71, 74, 77, 80, 83, 88, 92, 95, 98, 101, 104, 108, 111, 115, 119, 123, 128, 132, 143, 148, 155, 161, 167, 175, 182, 190, 199, 207, 217, 226, 236, 246, 257, 268, 279, 291, 302, 314, 326, 338, 350, 360, 372, 385, 399, 412, 425, 439, 453, 467, 481, 496, 510, 525, 540, 556, 571, 586, 602, 618, 635, 651, 668, 685, 702, 719, 737, 754, 772, 790, 808, 826, 844, 863, 881, 900, 919, 938, 956, 976, 994, 1013, 1032, 1051, 1070, 1088, 1107, 1125, 1143, 1162, 1180, 1199], Artifact: [47, 49, 52, 59, 61, 65, 68, 74, 77, 80, 83, 88, 92, 95, 98, 101, 104, 111, 115, 119, 123, 128, 132, 137, 143, 148, 155, 161, 167, 175, 182, 199, 207, 217, 226, 236, 246, 268, 279, 291, 302, 314, 326, 338, 350, 360, 372, 385, 399, 412, 425, 439, 453, 467, 481, 496, 510, 525, 540, 556, 571, 586, 602, 618, 635, 651, 668, 685, 702, 719, 737, 754, 772, 790, 808, 826, 844, 863, 881, 900, 919, 938, 956, 976, 994, 1013, 1032, 1051, 1070, 1088, 1107, 1125, 1143, 1162, 1180, 1199, 1217, 1235, 1254, 1272], Legendary: [49, 52, 59, 61, 65, 71, 74, 80, 83, 86, 92, 95, 98, 101, 108, 111, 115, 119, 123, 128, 132, 143, 148, 155, 161, 167, 175, 182, 190, 199, 207, 217, 236, 246, 257, 268, 279, 291, 302, 314, 326, 338, 350, 360, 372, 399, 412, 425, 439, 453, 467, 481, 496, 510, 525, 540, 556, 571, 586, 602, 618, 635, 651, 668, 685, 702, 719, 737, 754, 772, 790, 808, 826, 844, 863, 881, 900, 919, 938, 956, 976, 994, 1013, 1032, 1051, 1070, 1088, 1107, 1125, 1143, 1162, 1180, 1199, 1217, 1235, 1254, 1272, 1291, 1309, 1327] }, TechBlade: { Cheap: [31, 33, 35, 38, 40, 43, 46, 48, 52, 55, 58, 60, 63, 67, 70, 72, 75, 78, 81, 84, 87, 90, 93, 96, 99, 102, 106, 109, 113, 117, 121, 125, 130, 135, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 222, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034], Standard: [35, 38, 40, 43, 46, 48, 52, 55, 58, 60, 63, 67, 70, 72, 75, 78, 81, 84, 87, 90, 93, 96, 99, 102, 106, 109, 113, 117, 121, 125, 130, 135, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 222, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070], Premium: [38, 40, 43, 46, 48, 52, 55, 58, 60, 63, 67, 70, 72, 75, 78, 81, 84, 87, 90, 93, 96, 99, 102, 106, 109, 113, 117, 121, 125, 130, 135, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 222, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070, 1088], Prototype: [43, 46, 48, 52, 55, 58, 60, 67, 70, 72, 75, 78, 81, 87, 90, 93, 96, 99, 102, 106, 109, 113, 117, 121, 125, 130, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 222, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070, 1088, 1106, 1124, 1142, 1161, 1179], Artifact: [46, 48, 52, 58, 60, 63, 67, 72, 75, 78, 81, 87, 90, 93, 96, 99, 102, 109, 113, 117, 121, 125, 130, 135, 140, 146, 152, 158, 165, 172, 179, 195, 204, 213, 222, 232, 242, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070, 1088, 1106, 1124, 1142, 1161, 1179, 1197, 1215, 1233, 1251], Legendary: [48, 52, 58, 60, 63, 70, 72, 78, 81, 84, 90, 93, 96, 99, 106, 109, 113, 117, 121, 125, 130, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070, 1088, 1106, 1124, 1142, 1161, 1179, 1197, 1215, 1233, 1251, 1269, 1287, 1305] }, TrainingSaber: { Cheap: [31, 33, 35, 38, 40, 43, 46, 48, 52, 55, 58, 60, 63, 67, 70, 72, 75, 78, 81, 84, 87, 90, 93, 96, 99, 102, 106, 109, 113, 117, 121, 125, 130, 135, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 222, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034], Standard: [35, 38, 40, 43, 46, 48, 52, 55, 58, 60, 63, 67, 70, 72, 75, 78, 81, 84, 87, 90, 93, 96, 99, 102, 106, 109, 113, 117, 121, 125, 130, 135, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 222, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070], Premium: [38, 40, 43, 46, 48, 52, 55, 58, 60, 63, 67, 70, 72, 75, 78, 81, 84, 87, 90, 93, 96, 99, 102, 106, 109, 113, 117, 121, 125, 130, 135, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 222, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070, 1088], Prototype: [43, 46, 48, 52, 55, 58, 60, 67, 70, 72, 75, 78, 81, 87, 90, 93, 96, 99, 102, 106, 109, 113, 117, 121, 125, 130, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 222, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070, 1088, 1106, 1124, 1142, 1161, 1179], Artifact: [46, 48, 52, 58, 60, 63, 67, 72, 75, 78, 81, 87, 90, 93, 96, 99, 102, 109, 113, 117, 121, 125, 130, 135, 140, 146, 152, 158, 165, 172, 179, 195, 204, 213, 222, 232, 242, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070, 1088, 1106, 1124, 1142, 1161, 1179, 1197, 1215, 1233, 1251], Legendary: [48, 52, 58, 60, 63, 70, 72, 78, 81, 84, 90, 93, 96, 99, 106, 109, 113, 117, 121, 125, 130, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070, 1088, 1106, 1124, 1142, 1161, 1179, 1197, 1215, 1233, 1251, 1269, 1287, 1305] }, ElectroStaff: { Cheap: [37, 40, 42, 45, 48, 51, 55, 58, 62, 66, 69, 72, 76, 80, 84, 87, 90, 94, 97, 101, 104, 108, 112, 115, 119, 123, 127, 131, 136, 140, 145, 150, 156, 162, 168, 175, 182, 190, 198, 206, 215, 224, 234, 245, 256, 267, 278, 291, 303, 316, 330, 343, 357, 371, 385, 399, 413, 425, 439, 455, 470, 486, 502, 518, 534, 551, 568, 585, 602, 619, 637, 655, 673, 692, 711, 730, 749, 769, 788, 808, 828, 848, 869, 890, 911, 932, 953, 974, 996, 1018, 1040, 1062, 1084, 1106, 1129, 1151, 1173, 1196, 1218, 1240], Standard: [42, 45, 48, 51, 55, 58, 62, 66, 69, 72, 76, 80, 84, 87, 90, 94, 97, 101, 104, 108, 112, 115, 119, 123, 127, 131, 136, 140, 145, 150, 156, 162, 168, 175, 182, 190, 198, 206, 215, 224, 234, 245, 256, 267, 278, 291, 303, 316, 330, 343, 357, 371, 385, 399, 413, 425, 439, 455, 470, 486, 502, 518, 534, 551, 568, 585, 602, 619, 637, 655, 673, 692, 711, 730, 749, 769, 788, 808, 828, 848, 869, 890, 911, 932, 953, 974, 996, 1018, 1040, 1062, 1084, 1106, 1129, 1151, 1173, 1196, 1218, 1240, 1263, 1284], Premium: [45, 48, 51, 55, 58, 62, 66, 69, 72, 76, 80, 84, 87, 90, 94, 97, 101, 104, 108, 112, 115, 119, 123, 127, 131, 136, 140, 145, 150, 156, 162, 168, 175, 182, 190, 198, 206, 215, 224, 234, 245, 256, 267, 278, 291, 303, 316, 330, 343, 357, 371, 385, 399, 413, 425, 439, 455, 470, 486, 502, 518, 534, 551, 568, 585, 602, 619, 637, 655, 673, 692, 711, 730, 749, 769, 788, 808, 828, 848, 869, 890, 911, 932, 953, 974, 996, 1018, 1040, 1062, 1084, 1106, 1129, 1151, 1173, 1196, 1218, 1240, 1263, 1284, 1306], Prototype: [51, 55, 58, 62, 66, 69, 72, 80, 84, 87, 90, 94, 97, 104, 108, 112, 115, 119, 123, 127, 131, 136, 140, 145, 150, 156, 168, 175, 182, 190, 198, 206, 215, 224, 234, 245, 256, 267, 278, 291, 303, 316, 330, 343, 357, 371, 385, 399, 413, 425, 439, 455, 470, 486, 502, 518, 534, 551, 568, 585, 602, 619, 637, 655, 673, 692, 711, 730, 749, 769, 788, 808, 828, 848, 869, 890, 911, 932, 953, 974, 996, 1018, 1040, 1062, 1084, 1106, 1129, 1151, 1173, 1196, 1218, 1240, 1263, 1284, 1306, 1328, 1349, 1371, 1393, 1414], Artifact: [55, 58, 62, 69, 72, 76, 80, 87, 90, 94, 97, 104, 108, 112, 115, 119, 123, 131, 136, 140, 145, 150, 156, 162, 168, 175, 182, 190, 198, 206, 215, 234, 245, 256, 267, 278, 291, 316, 330, 343, 357, 371, 385, 399, 413, 425, 439, 455, 470, 486, 502, 518, 534, 551, 568, 585, 602, 619, 637, 655, 673, 692, 711, 730, 749, 769, 788, 808, 828, 848, 869, 890, 911, 932, 953, 974, 996, 1018, 1040, 1062, 1084, 1106, 1129, 1151, 1173, 1196, 1218, 1240, 1263, 1284, 1306, 1328, 1349, 1371, 1393, 1414, 1436, 1458, 1479, 1501], Legendary: [58, 62, 69, 72, 76, 84, 87, 94, 97, 101, 108, 112, 115, 119, 127, 131, 136, 140, 145, 150, 156, 168, 175, 182, 190, 198, 206, 215, 224, 234, 245, 256, 278, 291, 303, 316, 330, 343, 357, 371, 385, 399, 413, 425, 439, 470, 486, 502, 518, 534, 551, 568, 585, 602, 619, 637, 655, 673, 692, 711, 730, 749, 769, 788, 808, 828, 848, 869, 890, 911, 932, 953, 974, 996, 1018, 1040, 1062, 1084, 1106, 1129, 1151, 1173, 1196, 1218, 1240, 1263, 1284, 1306, 1328, 1349, 1371, 1393, 1414, 1436, 1458, 1479, 1501, 1523, 1545, 1566] }, VibroSword: { Cheap: [31, 33, 35, 38, 40, 43, 46, 48, 52, 55, 58, 60, 63, 67, 70, 72, 75, 78, 81, 84, 87, 90, 93, 96, 99, 102, 106, 109, 113, 117, 121, 125, 130, 135, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 222, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034], Standard: [35, 38, 40, 43, 46, 48, 52, 55, 58, 60, 63, 67, 70, 72, 75, 78, 81, 84, 87, 90, 93, 96, 99, 102, 106, 109, 113, 117, 121, 125, 130, 135, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 222, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070], Premium: [38, 40, 43, 46, 48, 52, 55, 58, 60, 63, 67, 70, 72, 75, 78, 81, 84, 87, 90, 93, 96, 99, 102, 106, 109, 113, 117, 121, 125, 130, 135, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 222, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070, 1088], Prototype: [43, 46, 48, 52, 55, 58, 60, 67, 70, 72, 75, 78, 81, 87, 90, 93, 96, 99, 102, 106, 109, 113, 117, 121, 125, 130, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 222, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070, 1088, 1106, 1124, 1142, 1161, 1179], Artifact: [46, 48, 52, 58, 60, 63, 67, 72, 75, 78, 81, 87, 90, 93, 96, 99, 102, 109, 113, 117, 121, 125, 130, 135, 140, 146, 152, 158, 165, 172, 179, 195, 204, 213, 222, 232, 242, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 379, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070, 1088, 1106, 1124, 1142, 1161, 1179, 1197, 1215, 1233, 1251], Legendary: [48, 52, 58, 60, 63, 70, 72, 78, 81, 84, 90, 93, 96, 99, 106, 109, 113, 117, 121, 125, 130, 140, 146, 152, 158, 165, 172, 179, 187, 195, 204, 213, 232, 242, 253, 264, 275, 286, 297, 309, 321, 332, 344, 354, 366, 392, 405, 418, 432, 445, 459, 473, 487, 502, 516, 531, 546, 561, 577, 592, 608, 624, 640, 657, 674, 690, 707, 724, 742, 759, 777, 794, 812, 830, 849, 867, 885, 904, 922, 941, 959, 978, 996, 1015, 1034, 1052, 1070, 1088, 1106, 1124, 1142, 1161, 1179, 1197, 1215, 1233, 1251, 1269, 1287, 1305] }, AssaultCannon: { Cheap: [41, 44, 47, 50, 53, 56, 61, 64, 68, 72, 76, 80, 84, 88, 92, 96, 100, 104, 107, 111, 115, 119, 123, 127, 131, 135, 140, 144, 150, 155, 160, 166, 172, 179, 186, 193, 201, 209, 218, 227, 237, 247, 258, 270, 282, 294, 307, 320, 334, 348, 363, 378, 393, 408, 424, 439, 455, 468, 484, 501, 518, 536, 553, 571, 588, 607, 626, 644, 663, 682, 702, 722, 742, 762, 783, 804, 825, 847, 869, 890, 912, 935, 958, 980, 1003, 1027, 1050, 1074, 1097, 1122, 1146, 1170, 1195, 1219, 1243, 1268, 1293, 1317, 1342, 1366], Standard: [47, 50, 53, 56, 61, 64, 68, 72, 76, 80, 84, 88, 92, 96, 100, 104, 107, 111, 115, 119, 123, 127, 131, 135, 140, 144, 150, 155, 160, 166, 172, 179, 186, 193, 201, 209, 218, 227, 237, 247, 258, 270, 282, 294, 307, 320, 334, 348, 363, 378, 393, 408, 424, 439, 455, 468, 484, 501, 518, 536, 553, 571, 588, 607, 626, 644, 663, 682, 702, 722, 742, 762, 783, 804, 825, 847, 869, 890, 912, 935, 958, 980, 1003, 1027, 1050, 1074, 1097, 1122, 1146, 1170, 1195, 1219, 1243, 1268, 1293, 1317, 1342, 1366, 1391, 1415], Premium: [50, 53, 56, 61, 64, 68, 72, 76, 80, 84, 88, 92, 96, 100, 104, 107, 111, 115, 119, 123, 127, 131, 135, 140, 144, 150, 155, 160, 166, 172, 179, 186, 193, 201, 209, 218, 227, 237, 247, 258, 270, 282, 294, 307, 320, 334, 348, 363, 378, 393, 408, 424, 439, 455, 468, 484, 501, 518, 536, 553, 571, 588, 607, 626, 644, 663, 682, 702, 722, 742, 762, 783, 804, 825, 847, 869, 890, 912, 935, 958, 980, 1003, 1027, 1050, 1074, 1097, 1122, 1146, 1170, 1195, 1219, 1243, 1268, 1293, 1317, 1342, 1366, 1391, 1415, 1439], Prototype: [56, 61, 64, 68, 72, 76, 80, 88, 92, 96, 100, 104, 107, 115, 119, 123, 127, 131, 135, 140, 144, 150, 155, 160, 166, 172, 186, 193, 201, 209, 218, 227, 237, 247, 258, 270, 282, 294, 307, 320, 334, 348, 363, 378, 393, 408, 424, 439, 455, 468, 484, 501, 518, 536, 553, 571, 588, 607, 626, 644, 663, 682, 702, 722, 742, 762, 783, 804, 825, 847, 869, 890, 912, 935, 958, 980, 1003, 1027, 1050, 1074, 1097, 1122, 1146, 1170, 1195, 1219, 1243, 1268, 1293, 1317, 1342, 1366, 1391, 1415, 1439, 1463, 1487, 1510, 1534, 1558], Artifact: [61, 64, 68, 76, 80, 84, 88, 96, 100, 104, 107, 115, 119, 123, 127, 131, 135, 144, 150, 155, 160, 166, 172, 179, 186, 193, 201, 209, 218, 227, 237, 258, 270, 282, 294, 307, 320, 348, 363, 378, 393, 408, 424, 439, 455, 468, 484, 501, 518, 536, 553, 571, 588, 607, 626, 644, 663, 682, 702, 722, 742, 762, 783, 804, 825, 847, 869, 890, 912, 935, 958, 980, 1003, 1027, 1050, 1074, 1097, 1122, 1146, 1170, 1195, 1219, 1243, 1268, 1293, 1317, 1342, 1366, 1391, 1415, 1439, 1463, 1487, 1510, 1534, 1558, 1582, 1606, 1630, 1654], Legendary: [64, 68, 76, 80, 84, 92, 96, 104, 107, 111, 119, 123, 127, 131, 140, 144, 150, 155, 160, 166, 172, 186, 193, 201, 209, 218, 227, 237, 247, 258, 270, 282, 307, 320, 334, 348, 363, 378, 393, 408, 424, 439, 455, 468, 484, 518, 536, 553, 571, 588, 607, 626, 644, 663, 682, 702, 722, 742, 762, 783, 804, 825, 847, 869, 890, 912, 935, 958, 980, 1003, 1027, 1050, 1074, 1097, 1122, 1146, 1170, 1195, 1219, 1243, 1268, 1293, 1317, 1342, 1366, 1391, 1415, 1439, 1463, 1487, 1510, 1534, 1558, 1582, 1606, 1630, 1654, 1678, 1702, 1726] }, TechStaff: { Cheap: [37, 40, 42, 45, 48, 51, 55, 58, 62, 66, 69, 72, 76, 80, 84, 87, 90, 94, 97, 101, 104, 108, 112, 115, 119, 123, 127, 131, 136, 140, 145, 150, 156, 162, 168, 175, 182, 190, 198, 206, 215, 224, 234, 245, 256, 267, 278, 291, 303, 316, 330, 343, 357, 371, 385, 399, 413, 425, 439, 455, 470, 486, 502, 518, 534, 551, 568, 585, 602, 619, 637, 655, 673, 692, 711, 730, 749, 769, 788, 808, 828, 848, 869, 890, 911, 932, 953, 974, 996, 1018, 1040, 1062, 1084, 1106, 1129, 1151, 1173, 1196, 1218, 1240], Standard: [42, 45, 48, 51, 55, 58, 62, 66, 69, 72, 76, 80, 84, 87, 90, 94, 97, 101, 104, 108, 112, 115, 119, 123, 127, 131, 136, 140, 145, 150, 156, 162, 168, 175, 182, 190, 198, 206, 215, 224, 234, 245, 256, 267, 278, 291, 303, 316, 330, 343, 357, 371, 385, 399, 413, 425, 439, 455, 470, 486, 502, 518, 534, 551, 568, 585, 602, 619, 637, 655, 673, 692, 711, 730, 749, 769, 788, 808, 828, 848, 869, 890, 911, 932, 953, 974, 996, 1018, 1040, 1062, 1084, 1106, 1129, 1151, 1173, 1196, 1218, 1240, 1263, 1284], Premium: [45, 48, 51, 55, 58, 62, 66, 69, 72, 76, 80, 84, 87, 90, 94, 97, 101, 104, 108, 112, 115, 119, 123, 127, 131, 136, 140, 145, 150, 156, 162, 168, 175, 182, 190, 198, 206, 215, 224, 234, 245, 256, 267, 278, 291, 303, 316, 330, 343, 357, 371, 385, 399, 413, 425, 439, 455, 470, 486, 502, 518, 534, 551, 568, 585, 602, 619, 637, 655, 673, 692, 711, 730, 749, 769, 788, 808, 828, 848, 869, 890, 911, 932, 953, 974, 996, 1018, 1040, 1062, 1084, 1106, 1129, 1151, 1173, 1196, 1218, 1240, 1263, 1284, 1306], Prototype: [51, 55, 58, 62, 66, 69, 72, 80, 84, 87, 90, 94, 97, 104, 108, 112, 115, 119, 123, 127, 131, 136, 140, 145, 150, 156, 168, 175, 182, 190, 198, 206, 215, 224, 234, 245, 256, 267, 278, 291, 303, 316, 330, 343, 357, 371, 385, 399, 413, 425, 439, 455, 470, 486, 502, 518, 534, 551, 568, 585, 602, 619, 637, 655, 673, 692, 711, 730, 749, 769, 788, 808, 828, 848, 869, 890, 911, 932, 953, 974, 996, 1018, 1040, 1062, 1084, 1106, 1129, 1151, 1173, 1196, 1218, 1240, 1263, 1284, 1306, 1328, 1349, 1371, 1393, 1414], Artifact: [55, 58, 62, 69, 72, 76, 80, 87, 90, 94, 97, 104, 108, 112, 115, 119, 123, 131, 136, 140, 145, 150, 156, 162, 168, 175, 182, 190, 198, 206, 215, 234, 245, 256, 267, 278, 291, 316, 330, 343, 357, 371, 385, 399, 413, 425, 439, 455, 470, 486, 502, 518, 534, 551, 568, 585, 602, 619, 637, 655, 673, 692, 711, 730, 749, 769, 788, 808, 828, 848, 869, 890, 911, 932, 953, 974, 996, 1018, 1040, 1062, 1084, 1106, 1129, 1151, 1173, 1196, 1218, 1240, 1263, 1284, 1306, 1328, 1349, 1371, 1393, 1414, 1436, 1458, 1479, 1501], Legendary: [58, 62, 69, 72, 76, 84, 87, 94, 97, 101, 108, 112, 115, 119, 127, 131, 136, 140, 145, 150, 156, 168, 175, 182, 190, 198, 206, 215, 224, 234, 245, 256, 278, 291, 303, 316, 330, 343, 357, 371, 385, 399, 413, 425, 439, 470, 486, 502, 518, 534, 551, 568, 585, 602, 619, 637, 655, 673, 692, 711, 730, 749, 769, 788, 808, 828, 848, 869, 890, 911, 932, 953, 974, 996, 1018, 1040, 1062, 1084, 1106, 1129, 1151, 1173, 1196, 1218, 1240, 1263, 1284, 1306, 1328, 1349, 1371, 1393, 1414, 1436, 1458, 1479, 1501, 1523, 1545, 1566] }, ScatterGun: { Cheap: [35, 38, 40, 43, 46, 49, 52, 55, 59, 62, 66, 69, 72, 76, 79, 83, 86, 90, 93, 96, 99, 103, 106, 110, 113, 117, 121, 125, 129, 133, 138, 143, 148, 154, 160, 166, 173, 180, 188, 196, 205, 213, 223, 233, 243, 254, 265, 277, 288, 301, 313, 326, 339, 353, 366, 379, 393, 404, 418, 432, 447, 462, 477, 493, 508, 524, 540, 556, 572, 589, 606, 623, 641, 658, 676, 694, 713, 731, 750, 769, 788, 807, 827, 846, 866, 886, 907, 927, 947, 968, 989, 1010, 1031, 1052, 1073, 1095, 1116, 1137, 1159, 1180], Standard: [40, 43, 46, 49, 52, 55, 59, 62, 66, 69, 72, 76, 79, 83, 86, 90, 93, 96, 99, 103, 106, 110, 113, 117, 121, 125, 129, 133, 138, 143, 148, 154, 160, 166, 173, 180, 188, 196, 205, 213, 223, 233, 243, 254, 265, 277, 288, 301, 313, 326, 339, 353, 366, 379, 393, 404, 418, 432, 447, 462, 477, 493, 508, 524, 540, 556, 572, 589, 606, 623, 641, 658, 676, 694, 713, 731, 750, 769, 788, 807, 827, 846, 866, 886, 907, 927, 947, 968, 989, 1010, 1031, 1052, 1073, 1095, 1116, 1137, 1159, 1180, 1201, 1221], Premium: [43, 46, 49, 52, 55, 59, 62, 66, 69, 72, 76, 79, 83, 86, 90, 93, 96, 99, 103, 106, 110, 113, 117, 121, 125, 129, 133, 138, 143, 148, 154, 160, 166, 173, 180, 188, 196, 205, 213, 223, 233, 243, 254, 265, 277, 288, 301, 313, 326, 339, 353, 366, 379, 393, 404, 418, 432, 447, 462, 477, 493, 508, 524, 540, 556, 572, 589, 606, 623, 641, 658, 676, 694, 713, 731, 750, 769, 788, 807, 827, 846, 866, 886, 907, 927, 947, 968, 989, 1010, 1031, 1052, 1073, 1095, 1116, 1137, 1159, 1180, 1201, 1221, 1242], Prototype: [49, 52, 55, 59, 62, 66, 69, 76, 79, 83, 86, 90, 93, 99, 103, 106, 110, 113, 117, 121, 125, 129, 133, 138, 143, 148, 160, 166, 173, 180, 188, 196, 205, 213, 223, 233, 243, 254, 265, 277, 288, 301, 313, 326, 339, 353, 366, 379, 393, 404, 418, 432, 447, 462, 477, 493, 508, 524, 540, 556, 572, 589, 606, 623, 641, 658, 676, 694, 713, 731, 750, 769, 788, 807, 827, 846, 866, 886, 907, 927, 947, 968, 989, 1010, 1031, 1052, 1073, 1095, 1116, 1137, 1159, 1180, 1201, 1221, 1242, 1263, 1283, 1304, 1325, 1345], Artifact: [52, 55, 59, 66, 69, 72, 76, 83, 86, 90, 93, 99, 103, 106, 110, 113, 117, 125, 129, 133, 138, 143, 148, 154, 160, 166, 173, 180, 188, 196, 205, 223, 233, 243, 254, 265, 277, 301, 313, 326, 339, 353, 366, 379, 393, 404, 418, 432, 447, 462, 477, 493, 508, 524, 540, 556, 572, 589, 606, 623, 641, 658, 676, 694, 713, 731, 750, 769, 788, 807, 827, 846, 866, 886, 907, 927, 947, 968, 989, 1010, 1031, 1052, 1073, 1095, 1116, 1137, 1159, 1180, 1201, 1221, 1242, 1263, 1283, 1304, 1325, 1345, 1366, 1386, 1407, 1428], Legendary: [55, 59, 66, 69, 72, 79, 83, 90, 93, 96, 103, 106, 110, 113, 121, 125, 129, 133, 138, 143, 148, 160, 166, 173, 180, 188, 196, 205, 213, 223, 233, 243, 265, 277, 288, 301, 313, 326, 339, 353, 366, 379, 393, 404, 418, 447, 462, 477, 493, 508, 524, 540, 556, 572, 589, 606, 623, 641, 658, 676, 694, 713, 731, 750, 769, 788, 807, 827, 846, 866, 886, 907, 927, 947, 968, 989, 1010, 1031, 1052, 1073, 1095, 1116, 1137, 1159, 1180, 1201, 1221, 1242, 1263, 1283, 1304, 1325, 1345, 1366, 1386, 1407, 1428, 1448, 1469, 1490] }, VibroKnife: { Cheap: [34, 36, 39, 41, 44, 47, 50, 53, 56, 60, 63, 66, 70, 73, 76, 79, 83, 86, 89, 92, 95, 99, 102, 105, 109, 112, 116, 120, 124, 128, 133, 137, 142, 148, 154, 160, 166, 173, 180, 188, 196, 205, 214, 223, 233, 244, 254, 266, 277, 289, 301, 313, 326, 338, 352, 364, 377, 388, 401, 415, 429, 444, 458, 473, 488, 503, 518, 534, 550, 566, 582, 598, 615, 632, 649, 666, 684, 702, 720, 738, 756, 775, 794, 813, 832, 851, 870, 890, 910, 930, 950, 970, 990, 1010, 1030, 1051, 1071, 1092, 1112, 1132], Standard: [39, 41, 44, 47, 50, 53, 56, 60, 63, 66, 70, 73, 76, 79, 83, 86, 89, 92, 95, 99, 102, 105, 109, 112, 116, 120, 124, 128, 133, 137, 142, 148, 154, 160, 166, 173, 180, 188, 196, 205, 214, 223, 233, 244, 254, 266, 277, 289, 301, 313, 326, 338, 352, 364, 377, 388, 401, 415, 429, 444, 458, 473, 488, 503, 518, 534, 550, 566, 582, 598, 615, 632, 649, 666, 684, 702, 720, 738, 756, 775, 794, 813, 832, 851, 870, 890, 910, 930, 950, 970, 990, 1010, 1030, 1051, 1071, 1092, 1112, 1132, 1153, 1173], Premium: [41, 44, 47, 50, 53, 56, 60, 63, 66, 70, 73, 76, 79, 83, 86, 89, 92, 95, 99, 102, 105, 109, 112, 116, 120, 124, 128, 133, 137, 142, 148, 154, 160, 166, 173, 180, 188, 196, 205, 214, 223, 233, 244, 254, 266, 277, 289, 301, 313, 326, 338, 352, 364, 377, 388, 401, 415, 429, 444, 458, 473, 488, 503, 518, 534, 550, 566, 582, 598, 615, 632, 649, 666, 684, 702, 720, 738, 756, 775, 794, 813, 832, 851, 870, 890, 910, 930, 950, 970, 990, 1010, 1030, 1051, 1071, 1092, 1112, 1132, 1153, 1173, 1192], Prototype: [47, 50, 53, 56, 60, 63, 66, 73, 76, 79, 83, 86, 89, 95, 99, 102, 105, 109, 112, 116, 120, 124, 128, 133, 137, 142, 154, 160, 166, 173, 180, 188, 196, 205, 214, 223, 233, 244, 254, 266, 277, 289, 301, 313, 326, 338, 352, 364, 377, 388, 401, 415, 429, 444, 458, 473, 488, 503, 518, 534, 550, 566, 582, 598, 615, 632, 649, 666, 684, 702, 720, 738, 756, 775, 794, 813, 832, 851, 870, 890, 910, 930, 950, 970, 990, 1010, 1030, 1051, 1071, 1092, 1112, 1132, 1153, 1173, 1192, 1212, 1232, 1252, 1272, 1291], Artifact: [50, 53, 56, 63, 66, 70, 73, 79, 83, 86, 89, 95, 99, 102, 105, 109, 112, 120, 124, 128, 133, 137, 142, 148, 154, 160, 166, 173, 180, 188, 196, 214, 223, 233, 244, 254, 266, 289, 301, 313, 326, 338, 352, 364, 377, 388, 401, 415, 429, 444, 458, 473, 488, 503, 518, 534, 550, 566, 582, 598, 615, 632, 649, 666, 684, 702, 720, 738, 756, 775, 794, 813, 832, 851, 870, 890, 910, 930, 950, 970, 990, 1010, 1030, 1051, 1071, 1092, 1112, 1132, 1153, 1173, 1192, 1212, 1232, 1252, 1272, 1291, 1311, 1331, 1351, 1371], Legendary: [53, 56, 63, 66, 70, 76, 79, 86, 89, 92, 99, 102, 105, 109, 116, 120, 124, 128, 133, 137, 142, 154, 160, 166, 173, 180, 188, 196, 205, 214, 223, 233, 254, 266, 277, 289, 301, 313, 326, 338, 352, 364, 377, 388, 401, 429, 444, 458, 473, 488, 503, 518, 534, 550, 566, 582, 598, 615, 632, 649, 666, 684, 702, 720, 738, 756, 775, 794, 813, 832, 851, 870, 890, 910, 930, 950, 970, 990, 1010, 1030, 1051, 1071, 1092, 1112, 1132, 1153, 1173, 1192, 1212, 1232, 1252, 1272, 1291, 1311, 1331, 1351, 1371, 1390, 1410, 1430] } },
        techPower: { BlasterRifle: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] }, SniperRifle: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] }, Pistol: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] }, TechBlade: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] }, AssaultCannon: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] }, TechStaff: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] }, ScatterGun: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] }, VibroKnife: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] } },
        forcePower: { PoleSaber: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] }, LightSaber: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] }, TrainingSaber: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] }, ElectroStaff: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] }, VibroSword: { Cheap: [42, 45, 48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402], Standard: [48, 51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451], Premium: [51, 54, 58, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98, 102, 106, 110, 114, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476], Prototype: [58, 62, 66, 70, 74, 78, 82, 90, 94, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 302, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599], Artifact: [62, 66, 70, 78, 82, 86, 90, 98, 102, 106, 110, 118, 122, 126, 130, 135, 139, 148, 153, 159, 164, 170, 176, 183, 190, 198, 206, 214, 223, 233, 243, 265, 277, 289, 302, 315, 329, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 514, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697], Legendary: [66, 70, 78, 82, 86, 94, 98, 106, 110, 114, 122, 126, 130, 135, 143, 148, 153, 159, 164, 170, 176, 190, 198, 206, 214, 223, 233, 243, 254, 265, 277, 289, 315, 329, 343, 357, 373, 388, 403, 419, 435, 451, 467, 480, 497, 532, 549, 567, 585, 604, 622, 642, 661, 680, 700, 721, 741, 761, 782, 803, 825, 847, 869, 891, 914, 936, 959, 982, 1006, 1029, 1053, 1077, 1101, 1126, 1151, 1175, 1201, 1226, 1251, 1276, 1301, 1326, 1351, 1377, 1402, 1427, 1451, 1476, 1501, 1525, 1550, 1574, 1599, 1623, 1648, 1672, 1697, 1721, 1746, 1770] } }
    };

    weapon.GetDamage = function (levelOrItem, quality, weaponType) {

        var level = levelOrItem;
        if (typeof (levelOrItem) == 'object') {
            if (Tr.Swtor.isMod(levelOrItem)) return null;

            level = levelOrItem.ItemLevel;
            quality = levelOrItem.Quality;
            weaponType = levelOrItem.WeaponType;

            if (quality == 'Custom') {
                level = 1;
                quality = 'Standard';
            }

            // find barrel or hilt mod
            if (levelOrItem.Mods !== undefined && levelOrItem.Mods !== null) {
                for (var i = 0; i < levelOrItem.Mods.length; i++) {
                    if (levelOrItem.Mods[i].Id != null && levelOrItem.Mods[i].Id != 0 && (levelOrItem.Mods[i].Slot == 'Barrel' || levelOrItem.Mods[i].Slot == 'Hilt')) {
                        level = levelOrItem.Mods[i].ItemLevel;
                        quality = levelOrItem.Mods[i].Quality;
                        break;
                    }
                }
            }
        }

        if (quality == 'Legacy')
            quality = 'Legendary';

        if (weaponType === undefined || weaponType === null) {
            return null;
        }
        var w = lookup.min[weaponType];
        if (w === undefined) {
            return null;
        }
        var q = w[quality];
        if (q === undefined) {
            return null;
        }

        var min, max, power, powerType;
        min = q[level - 1];
        max = lookup.max[weaponType][quality][level - 1];

        var p = lookup.techPower[weaponType];
        if (p === undefined) {
            p = lookup.forcePower[weaponType];
            powerType = 'Force';
        }
        else
            powerType = 'Tech';

        if (p !== undefined) {
            power = p[quality][level - 1];
        }

        return { min: min, max: max, power: power, powerType: powerType };
    }

    return calc;
}(TeamRobot.Swtor.Calc || {}, jQuery));


Tr.Swtor.Calc.getProfileStats = function (profile) {

    var gearSet = [];
    if (profile.GearSet != null) {
        for (var slot in profile.GearSet) {
            var item = profile.GearSet[slot];
            item.CharacterSlot = slot;
            gearSet.push(item);
        }
    }

    var buffs = profile.Buffs;
    if (buffs == null) buffs = [];

    return Tr.Swtor.Calc.getAllStats(profile, gearSet, 'None', buffs, profile.Stance, profile.StimLevel, profile.StimType);
};
